/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			AnimateView.cpp
//		DESCRIPTION:	implementation of the CAnimateView class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "AnimateView.h"
#include "GLFloor.h"
#include "GlIsoSystem.h"
#include "Project.h"
#include "Building.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                           CAnimateView CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CAnimateView, CWnd)
	//{{AFX_MSG_MAP(CAnimateView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_DESTROY()
	ON_COMMAND(ID_PROJECT_PRINT, OnPrint)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_PRINT, OnUpdatePrint)
	ON_COMMAND(ID_ANIMATION_PLAY, OnAnimationPlay)
	ON_UPDATE_COMMAND_UI(ID_ANIMATION_PLAY, OnUpdateAnimationPlay)
	ON_COMMAND(ID_ANIMATION_STOP, OnAnimationStop)
	ON_UPDATE_COMMAND_UI(ID_ANIMATION_STOP, OnUpdateAnimationStop)
	ON_COMMAND(ID_ANIMATION_SLOW, OnAnimationSlow)
	ON_UPDATE_COMMAND_UI(ID_ANIMATION_SLOW, OnUpdateAnimationSlow)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//**************************** Setup/cleanup ********************************
/////////////////////////////////////////////////////////////////////////////
CAnimateView::CAnimateView() :
m_AniMan(m_apFloors, m_pGlIsoSystem, m_ViewerGauge)
{
}

/////////////////////////////////////////////////////////////////////////////
BOOL CAnimateView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
	{
		return FALSE;
	}

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	cs.lpszClass = AfxRegisterWndClass(
		CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), 
		NULL,
		NULL);

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
int CAnimateView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	CWaitCursor wait;

	if (CWnd::OnCreate(lpCreateStruct) == -1)
	{
		return -1;
	}
	
	// Context --------------------------------------------------------------
	
	HRESULT hr = CoCreateInstance(
		CLSID_GlContext, 
		NULL, 
		CLSCTX_INPROC_SERVER, 
		IID_IGlContext, 
		(void**)&m_pContext);

	if (FAILED(hr))
	{
		ASSERT(FALSE);
		return -1;
	}

	m_pContext->Initialize(DWORD(m_hWnd));
	m_pContext->SetBackgroundColor(GlColor(0.0f, 0.0f, 0.0f));

	// Viewer ---------------------------------------------------------------

	const CBuilding* pBldg = g_Application.GetProject().GetBuilding();
	int nFloors = pBldg->GetNumberOfFloors();
	double dHalfExtents = 0.5 * double(nFloors) * GlFloor::GetFloorHeight() + 5.0;
	GlPoint minExtent(-dHalfExtents, -dHalfExtents, -dHalfExtents);
	GlPoint maxExtent(+dHalfExtents, +dHalfExtents, +dHalfExtents);

	m_pContext->CreateViewer(TRUE, TRUE, &m_pViewer);
	m_pViewer->SetEyeZNearAndFar(1.5f, 100.0f);
	m_pViewer->SetOrthoExtents(minExtent, maxExtent);

	m_pViewer->GetCoord(&m_pViewerCS);
	m_pViewerCS->MoveOrigin(GlPoint(0.0, dHalfExtents - 10.0, 0.0));

	// Gauge ----------------------------------------------------------------

	IGlFont* pFont = NULL;
	m_pContext->CreateSystemFont(&pFont);
	ASSERT(pFont);

	m_ViewerGauge.SetFont(pFont);
	m_ViewerGauge.Show(TRUE);
	m_pViewer->AddGauge(&m_ViewerGauge);
	m_pViewer->ShowGauges(TRUE);

	// Light ----------------------------------------------------------------

	m_pContext->CreateLight(&m_pLight);
	m_pLight->Enable(TRUE);
	m_pLight->SetAsSpotlight(FALSE);

	m_pLight->SetAmbientLight(GlColor(0.1f, 0.1f, 0.2f));
	m_pLight->SetDiffuseLight(GlColor(0.7f, 0.6f, 0.5f));
	m_pLight->SetSpecularLight(GlColor(0.9f, 0.6f, 0.3f));

	IGlCoord* pCoord;
	m_pLight->GetCoord(&pCoord);
	pCoord->RotateAboutY_RAD(0.2);
	pCoord->RotateAboutX_RAD(-1.3);
	pCoord->Release();

	// Space ----------------------------------------------------------------

	hr = CoCreateInstance(
		CLSID_VolSpace, 
		NULL, 
		CLSCTX_INPROC_SERVER, 
		IID_IVolSpace, 
		(void**)&m_pSpace);

	if (FAILED(hr))
	{
		ASSERT(FALSE);
		return -1;
	}

	m_pSpace->AddContext(m_pContext);

	// Create scene ---------------------------------------------------------

	for (int iFloor = 0; iFloor < nFloors; iFloor++)
	{
		GlColor color(iFloor ? 1.0 : 1.0, iFloor ? 1.0 : 0.0, iFloor ? 0.5 : 0.2);
		m_apFloors.push_back(new GlFloor(iFloor, m_pSpace, color));
	}

	GlColor color(1.0, 1.0, 1.0);
	m_pGlIsoSystem = new GlIsoSystem(m_pSpace, color, 5.0 * nFloors);

	m_AniMan.Start();

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
void CAnimateView::OnDestroy() 
{
	m_pViewerCS->Release();
	m_pSpace->RemoveContext(m_pContext);	
	m_pContext->Release();
	m_pSpace->Release();

	CWnd::OnDestroy();
}

/////////////////////////////////////////////////////////////////////////////
CAnimateView::~CAnimateView()
{
	delete m_pGlIsoSystem;

	for (int i = 0, nFloors = m_apFloors.size(); i < nFloors; i++)
	{
		delete m_apFloors[i];
	}
}

//******************************* Messages **********************************
/////////////////////////////////////////////////////////////////////////////
void CAnimateView::OnSize(UINT nType, int cx, int cy) 
{
	m_pContext->SetViewport(0, 0, cx, cy);
	CWnd::OnSize(nType, cx, cy);
}

/////////////////////////////////////////////////////////////////////////////
BOOL CAnimateView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

//******************************* Commands **********************************
/////////////////////////////////////////////////////////////////////////////
void CAnimateView::OnAnimationPlay() 
{
	m_AniMan.IsPlaying() ? m_AniMan.Pause() : m_AniMan.Play();
}

void CAnimateView::OnUpdateAnimationPlay(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(!m_AniMan.IsStopped() && !m_AniMan.IsPaused());
}

/////////////////////////////////////////////////////////////////////////////
void CAnimateView::OnAnimationStop() 
{
	m_AniMan.Stop();
}

void CAnimateView::OnUpdateAnimationStop(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_AniMan.IsStopped());
}

/////////////////////////////////////////////////////////////////////////////
void CAnimateView::OnAnimationSlow() 
{
	m_AniMan.SetSlowSpeed(!m_AniMan.IsSlowSpeed());
}

void CAnimateView::OnUpdateAnimationSlow(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_AniMan.IsSlowSpeed());
}

/////////////////////////////////////////////////////////////////////////////
void CAnimateView::OnPrint() 
{
	/*
	CPrintDialog dlg(false);

	if (dlg.DoModal() == IDOK)
	{
		CDC dc;
		dc.Attach(dlg.GetPrinterDC());

		DOCINFO di;
		di.cbSize = sizeof(di);
		di.lpszDocName = "Snapshot View";
		di.lpszOutput = NULL;
		di.lpszDatatype = NULL;
		di.fwType = 0;

		dc.StartDoc(&di);
		dc.StartPage();

		// Define a destination rectangle
		CRect rectDest;
		dc.GetWindow()->GetClientRect(&rectDest);

		// Define a source rectangle
		int iSrcWidth, iSrcHeight;
		m_pContext->GetViewportWidth(&iSrcWidth);
		m_pContext->GetViewportHeight(&iSrcHeight);

		// Draw onto the destination dc
		m_pContext->DrawToDC(
			DWORD(dc.GetSafeHdc()),
			0, 
			0, 
			iSrcWidth, 
			iSrcHeight,
			rectDest.left, 
			rectDest.top, 
			rectDest.Width(), 
			rectDest.Height(),
			m_PrintBkgdColor,
			m_fPrintQuality);

		dc.EndPage();
		dc.EndDoc();

		dc.DeleteDC();
	}
	*/
}

void CAnimateView::OnUpdatePrint(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

//=================================== EOF ===================================
