/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			AnimationManager.cpp
//		DESCRIPTION:	implementation of the CAnimationManager class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "AnimationManager.h"
#include <process.h>
#include "GLFloor.h"
#include "GlIsoSystem.h"
#include "GlViewerGauge.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

typedef unsigned (__stdcall *PTHREAD_START) (void *);

#define chBEGINTHREADEX(psa, cbStack, pfnStartAddr, \
     pvParam, fdwCreate, pdwThreadID)               \
       ((HANDLE) _beginthreadex(                    \
          (void *) (psa),                           \
          (unsigned) (cbStack),                     \
          (PTHREAD_START) (pfnStartAddr),           \
          (void *) (pvParam),                       \
          (unsigned) (fdwCreate),                   \
          (unsigned *) (pdwThreadID))) 


//***************************************************************************
//                        CAnimationManager CLASS
//***************************************************************************

bool CAnimationManager::m_bPlaying(false);
bool CAnimationManager::m_bSlowSpeed(false);
double CAnimationManager::m_dTime_SEC(0.0);
double CAnimationManager::m_dDispCoeff(0.3);

//**************************** Setup/cleanup ********************************
/////////////////////////////////////////////////////////////////////////////
CAnimationManager::CAnimationManager(vector<GlFloor*>& aFloors, 
									 GlIsoSystem*& aIsoSystem,
									 GlViewerGauge& aGauge) :
m_Gauge(aGauge),
m_apFloors(aFloors),
m_pGlIsoSystem(aIsoSystem),
m_pProject(NULL),
m_pIsoSystem(NULL),
m_hThread(0),
m_dwThreadID(0)
{
	m_pProject = &g_Application.GetProject();
	m_pIsoSystem = g_Application.GetSelIsoSystem();
	ASSERT(m_pIsoSystem);

	m_Gauge.SetTitle(CIsoSystem::GetNameByType(m_pIsoSystem->GetType()));
	m_Gauge.SetMode(GetModeString());
	m_Gauge.SetTime_SEC("Elapsed: 0.00 sec");

	CString str;
	str.Format("Total: %.2f sec", m_pProject->GetEq()->GetTotalTime_SEC());
	m_Gauge.SetTotalTime_SEC(str);

	// Begin the thread
	m_hThread = chBEGINTHREADEX (NULL, // pointer to thread security attributes 
		0, // initial thread stack size, in bytes 
		PThreadLoop, // pointer to thread function 
		this, // argument for new thread 
		CREATE_SUSPENDED, // creation flags 
		&m_dwThreadID); // pointer to returned thread identifier
}

/////////////////////////////////////////////////////////////////////////////
CAnimationManager::~CAnimationManager()
{
	TerminateThread(m_hThread, 0);
	CloseHandle(m_hThread);
}

//****************************** Operations *********************************
/////////////////////////////////////////////////////////////////////////////
void CAnimationManager::Play()
{
	m_bPlaying = true;
	m_Gauge.SetMode(GetModeString());
}

/////////////////////////////////////////////////////////////////////////////
void CAnimationManager::Pause()
{
	m_bPlaying = false;
	m_Gauge.SetMode(GetModeString());
}

/////////////////////////////////////////////////////////////////////////////
void CAnimationManager::Stop()
{
	m_bPlaying = false;
	m_dTime_SEC = 0.0;

	m_pGlIsoSystem->SetAbsDisp(0.0);

	for (int iFloor = 0, nFloors = m_apFloors.size(); iFloor < nFloors; iFloor++)
	{
		m_apFloors[iFloor]->SetAbsDisp(0.0);
	}

	m_Gauge.SetMode(GetModeString());
	m_Gauge.SetTime_SEC("Elapsed: 0.00 sec");
}

/////////////////////////////////////////////////////////////////////////////
void CAnimationManager::SetSlowSpeed(bool aSlow)
{
	m_bSlowSpeed = aSlow;
	m_Gauge.SetMode(GetModeString());
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
CString CAnimationManager::GetModeString() const
{
	CString str;

	if (m_bPlaying)
	{
		str = m_bSlowSpeed ? "PLAYING SLOW" : "PLAYING";
	}
	else
	{
		str = m_dTime_SEC ? "PAUSE" : "STOP";
	}

	return str;
}

/////////////////////////////////////////////////////////////////////////////
bool CAnimationManager::UpdateFloorDisp(double aTime)
{
	const CEarthquake* pEq = m_pProject->GetEq();
	ASSERT(pEq);

	if (pEq->GetTotalTime_SEC() < aTime)
	{
		return false;
	}

	double dDisp = m_dDispCoeff * pEq->GetAbsDisp_CM(aTime);
	m_pGlIsoSystem->SetAbsDisp(dDisp);

	for (int iFloor = 0, nFloors = m_apFloors.size(); iFloor < nFloors; iFloor++)
	{
		double dDisp = m_dDispCoeff * m_pProject->GetAbsDisp_CM(m_pIsoSystem, iFloor, aTime);
		m_apFloors[iFloor]->SetAbsDisp(dDisp);
	}

	CString str;
	str.Format("Elapsed: %.2f sec", aTime);
	m_Gauge.SetTime_SEC(str);

	return true;
}

//******************************* Thread loop *******************************
/////////////////////////////////////////////////////////////////////////////
DWORD WINAPI PThreadLoop (void *APThread)
{
	#define _PTR ((CAnimationManager*)APThread)

	DWORD dwElapsed_MSEC = GetTickCount();

	while (true)
	{
		if (_PTR->IsPlaying())
		{
			if (!_PTR->UpdateFloorDisp(_PTR->m_dTime_SEC))
			{
				_PTR->Stop();
			}
			else
			{
				double dTimeCoeff = _PTR->m_bSlowSpeed ? 0.00025 : 0.001;
				_PTR->m_dTime_SEC += (dTimeCoeff * double(GetTickCount() - dwElapsed_MSEC));
			}
		}

		dwElapsed_MSEC = GetTickCount();
		Sleep(20);
	}

	return 0;

	#undef _PTR
}

//=================================== EOF ===================================
