/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			AnimationManager.h
//		DESCRIPTION:	interface for the CAnimationManager class
//
/////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_ANIMATIONMANAGER_H__C819B1D8_3A91_4754_A4F1_30A778A45618__INCLUDED_)
#define AFX_ANIMATIONMANAGER_H__C819B1D8_3A91_4754_A4F1_30A778A45618__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Lock.h"

class GlFloor;
class GlIsoSystem;
class GlViewerGauge;
class CProject;
class CIsoSystem;


//***************************************************************************
//                        CAnimationManager CLASS
//***************************************************************************

class CAnimationManager  
{
private:

	vector<GlFloor*>& m_apFloors;
	GlIsoSystem*& m_pGlIsoSystem;
	GlViewerGauge& m_Gauge;

	const CProject* m_pProject;
	const CIsoSystem* m_pIsoSystem;

	static bool m_bPlaying;
	static bool m_bSlowSpeed;
	static double m_dTime_SEC;
	static double m_dDispCoeff;

	HANDLE m_hThread;
	DWORD m_dwThreadID;

public:

	CAnimationManager(vector<GlFloor*>& aFloors, GlIsoSystem*& aIsoSystem, GlViewerGauge& aGauge);
	void Start() { ResumeThread(m_hThread); }
	~CAnimationManager();

	inline bool IsPlaying() const   { return m_bPlaying; }
	inline bool IsPaused() const    { return !m_bPlaying && m_dTime_SEC; }
	inline bool IsStopped() const   { return !m_bPlaying && !m_dTime_SEC; }
	inline bool IsSlowSpeed() const { return m_bSlowSpeed; }

	void Play();
	void Pause();
	void Stop();
	void SetSlowSpeed(bool aSlow);

protected:

	CString GetModeString() const;

	bool UpdateFloorDisp(double aTime);
	friend static DWORD WINAPI PThreadLoop (void *AArg);
};

#endif // !defined(AFX_ANIMATIONMANAGER_H__C819B1D8_3A91_4754_A4F1_30A778A45618__INCLUDED_)

//=================================== EOF ===================================
