/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			Building.cpp
//		DESCRIPTION:	implementation of the CBuilding class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "Building.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//***************************************************************************
//                             CBuilding CLASS
//***************************************************************************

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CBuilding::CBuilding(long aID) :
m_lID(aID),
m_strDescription("")
{
	if (m_lID)
	{
		IDbTable* pTable = g_Application.GetDatabase()->GetBuildingsTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		long lFieldID;

		BSTR bstrDesc;
		pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
		pRS->GetFieldAsString(m_lID, lFieldID, &bstrDesc);

		if (bstrDesc)
		{
			m_strDescription = bstrDesc;
			SysFreeString(bstrDesc);
		}

		for (int i = 0; i < MAX_NUMBER_OF_FLOORS; i++)
		{
			// Get mass
			double dValue;
			CString str;
			str.Format("MASS%u_KG", i);
			pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
			pRS->GetFieldAsFloat(m_lID, lFieldID, &dValue);

			if (dValue)
			{
				m_adMass.push_back(dValue);
			}
			else
			{
				// Bail out
				break;
			}

			// Get quality factor
			long iValue;
			str.Format("QUALITY_FACTOR%u", i);
			pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
			pRS->GetFieldAsInteger(m_lID, lFieldID, &iValue);
			m_aiQualityFactor.push_back(iValue);

			// Get elastic limit
			str.Format("ELASTIC_LIMIT%u_CM", i);
			pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
			pRS->GetFieldAsFloat(m_lID, lFieldID, &dValue);
			m_adElasticLimit.push_back(dValue);

			// Get rigidity coeff
			str.Format("RIGIDITY_COEFF%u", i);
			pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
			pRS->GetFieldAsInteger(m_lID, lFieldID, &iValue);
			m_aiRigidityCoeff.push_back(iValue);
		}
	}
	else
	{
		// At least set the basement & 1st floor
		m_adMass.push_back(DEFAULT_FLOOR_MASS);
		m_adMass.push_back(DEFAULT_FLOOR_MASS);
		m_aiQualityFactor.push_back(DEFAULT_QUALITY_FACTOR);
		m_aiQualityFactor.push_back(DEFAULT_QUALITY_FACTOR);
		m_adElasticLimit.push_back(DEFAULT_ELASTIC_LIMIT);
		m_adElasticLimit.push_back(DEFAULT_ELASTIC_LIMIT);
		m_aiRigidityCoeff.push_back(DEFAULT_RIGIDITY_COEFF);
		m_aiRigidityCoeff.push_back(DEFAULT_RIGIDITY_COEFF);
	}
}

/////////////////////////////////////////////////////////////////////////////
CBuilding::CBuilding(const CBuilding& aBldg)
{
	m_lID             = aBldg.m_lID;
	m_strDescription  = aBldg.m_strDescription;
	m_adMass          = aBldg.m_adMass;
	m_aiQualityFactor = aBldg.m_aiQualityFactor;
	m_adElasticLimit  = aBldg.m_adElasticLimit;
	m_aiRigidityCoeff = aBldg.m_aiRigidityCoeff;
}

/////////////////////////////////////////////////////////////////////////////
const CBuilding& CBuilding::operator = (const CBuilding& aBldg)
{
	m_lID             = aBldg.m_lID;
	m_strDescription  = aBldg.m_strDescription;
	m_adMass          = aBldg.m_adMass;
	m_aiQualityFactor = aBldg.m_aiQualityFactor;
	m_adElasticLimit  = aBldg.m_adElasticLimit;
	m_aiRigidityCoeff = aBldg.m_aiRigidityCoeff;

	return *this;
}

/////////////////////////////////////////////////////////////////////////////
CBuilding::~CBuilding()
{
}

//******************************* Archiving *********************************
/////////////////////////////////////////////////////////////////////////////
bool CBuilding::Save()
{
	if (!IsValid())
	{
		return false;
	}

	IDbTable* pTable = g_Application.GetDatabase()->GetBuildingsTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	if (!m_lID)
	{
		// Create record
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &m_lID);
		ASSERT(m_lID);
	}

	long lFieldID;
	pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
	pRS->SetFieldToString(m_lID, lFieldID, m_strDescription.IsEmpty() ? NULL : A2BSTR(m_strDescription));

	for (int i = 0, nFloors = m_adMass.size(); i < nFloors; i++)
	{
		if (!m_adMass[i])
		{
			break;
		}

		// Set mass
		CString str;
		str.Format("MASS%u_KG", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToFloat(m_lID, lFieldID, m_adMass[i]);

		// Set quality factor
		str.Format("QUALITY_FACTOR%u", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToInteger(m_lID, lFieldID, m_aiQualityFactor[i]);

		// Set elastic limit
		str.Format("ELASTIC_LIMIT%u_CM", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToFloat(m_lID, lFieldID, m_adElasticLimit[i]);

		// Set rigidity coeff
		str.Format("RIGIDITY_COEFF%u", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToInteger(m_lID, lFieldID, m_aiRigidityCoeff[i]);
	}

	for (; i < MAX_NUMBER_OF_FLOORS; i++)
	{
		// Set mass
		CString str;
		str.Format("MASS%u_KG", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToFloat(m_lID, lFieldID, 0.0);

		// Set quality factor
		str.Format("QUALITY_FACTOR%u", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToInteger(m_lID, lFieldID, 0);

		// Set elastic limit
		str.Format("ELASTIC_LIMIT%u_CM", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToFloat(m_lID, lFieldID, 0.0);

		// Set rigidity coeff
		str.Format("RIGIDITY_COEFF%u", i);
		pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);
		pRS->SetFieldToInteger(m_lID, lFieldID, 0.0);
	}

	g_Application.GetDatabase()->Commit();

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CBuilding::IsValid() const
{
	return m_adMass.size() > 1;
}

//****************************** Getting data *******************************
/////////////////////////////////////////////////////////////////////////////
CString CBuilding::GetParamDesc() const
{
	CString str;
	str.Format(
		"%s Num. floors: %d", 
		m_strDescription, 
		m_adMass.size() - 1);

	return str;
}

/////////////////////////////////////////////////////////////////////////////
double CBuilding::GetFloorMass_KG(int aFloorIndex) const 
{ 
	if (aFloorIndex < 0 || aFloorIndex >= m_adMass.size())
		return 0.0;

	return m_adMass.at(aFloorIndex); 
}

/////////////////////////////////////////////////////////////////////////////
int CBuilding::GetQualityFactor(int aFloorIndex) const
{
	if (aFloorIndex < 0 || aFloorIndex >= m_adMass.size())
		return 0;

	return m_aiQualityFactor.at(aFloorIndex); 
}

/////////////////////////////////////////////////////////////////////////////
double CBuilding::GetElasticLimit_CM(int aFloorIndex) const
{
	if (aFloorIndex < 0 || aFloorIndex >= m_adMass.size())
		return 0.0;

	return m_adElasticLimit.at(aFloorIndex); 
}

/////////////////////////////////////////////////////////////////////////////
int CBuilding::GetRigidityCoeff(int aFloorIndex) const
{
	if (aFloorIndex < 0 || aFloorIndex >= m_adMass.size())
		return 0;

	return m_aiRigidityCoeff.at(aFloorIndex); 
}

/////////////////////////////////////////////////////////////////////////////
double CBuilding::GetTotalMass_KG() const
{
	double dMass(0.0);

	for (int i = 0, nCount = m_adMass.size(); i < nCount; i++)
	{
		dMass += m_adMass[i];
	}

	return dMass;
}

/////////////////////////////////////////////////////////////////////////////
double CBuilding::GetInitialRigidity_KG_SEC2(int aFloorIndex) const
{
	if (aFloorIndex < 0 || aFloorIndex >= m_adMass.size())
		return 0.0;

	return double(m_aiRigidityCoeff.at(aFloorIndex)) * m_adMass.at(aFloorIndex) / 
		   double(m_aiQualityFactor.at(aFloorIndex));
}

/////////////////////////////////////////////////////////////////////////////
double CBuilding::GetInitialDamping_KG_SEC(int aFloorIndex) const
{
	if (aFloorIndex < 0 || aFloorIndex >= m_adMass.size())
		return 0.0;

	return 1.233 * m_adMass.at(aFloorIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CBuilding::GetRigidity_KG_SEC2(int aFloorIndex, double aRelDisp) const
{
	if (aFloorIndex < 0 || aFloorIndex >= m_adMass.size())
		return 0.0;

	aRelDisp = min(fabs(aRelDisp), 
		double(m_aiQualityFactor.at(aFloorIndex)) * m_adElasticLimit.at(aFloorIndex));

	return GetInitialRigidity_KG_SEC2(aFloorIndex) - 
	       1000.0 * m_adMass.at(aFloorIndex) / double(m_aiQualityFactor.at(aFloorIndex)) * 
		   sqrt(aRelDisp / m_adElasticLimit.at(aFloorIndex));
}

/////////////////////////////////////////////////////////////////////////////
double CBuilding::GetDamping_KG_SEC(int aFloorIndex, double aRelDisp) const
{
	if (aFloorIndex < 0 || aFloorIndex >= m_adMass.size())
		return 0.0;

	aRelDisp = min(fabs(aRelDisp), 
		double(m_aiQualityFactor.at(aFloorIndex)) * m_adElasticLimit.at(aFloorIndex));

	return GetInitialDamping_KG_SEC(aFloorIndex) * 
		   (1.0 + sqrt(aRelDisp / m_adElasticLimit.at(aFloorIndex)) / 
		   double(m_aiQualityFactor.at(aFloorIndex)));
}

//***************************** Modifying data ******************************
/////////////////////////////////////////////////////////////////////////////
bool CBuilding::SetDescription(const char* aValue)
{
	m_strDescription = aValue;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CBuilding::SetQualityFactor(int aFloorIndex, int aValue)
{
	int nCount = m_aiQualityFactor.size();

	if (aValue < MIN_QUALITY_FACTOR ||
		aValue > MAX_QUALITY_FACTOR ||
		aFloorIndex < 0 || 
		aFloorIndex >= MAX_NUMBER_OF_FLOORS ||
		aFloorIndex > nCount)
	{
		return false;
	}

	if (aValue == 0.0)
	{
		m_aiQualityFactor.resize(aFloorIndex);
	}
	else if (aFloorIndex == nCount)
	{
		m_aiQualityFactor.push_back(aValue);
	}
	else
	{
		m_aiQualityFactor[aFloorIndex] = aValue;
	}

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CBuilding::SetElasticLimit_CM(int aFloorIndex, double aValue)
{
	int nCount = m_adElasticLimit.size();

	if (aValue < MIN_ELASTIC_LIMIT || 
		aValue > MAX_ELASTIC_LIMIT ||
		aFloorIndex < 0 || 
		aFloorIndex >= MAX_NUMBER_OF_FLOORS || 
		aFloorIndex > nCount)
	{
		return false;
	}

	if (aValue == 0.0)
	{
		m_adElasticLimit.resize(aFloorIndex);
	}
	else if (aFloorIndex == nCount)
	{
		m_adElasticLimit.push_back(aValue);
	}
	else
	{
		m_adElasticLimit[aFloorIndex] = aValue;
	}

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CBuilding::SetRigidityCoeff(int aFloorIndex, int aValue)
{
	int nCount = m_aiRigidityCoeff.size();

	if (aValue < MIN_RIGIDITY_COEFF ||
		aValue > MAX_RIGIDITY_COEFF ||
		aFloorIndex < 0 || 
		aFloorIndex >= MAX_NUMBER_OF_FLOORS ||
		aFloorIndex > nCount)
	{
		return false;
	}

	if (aValue == 0.0)
	{
		m_aiRigidityCoeff.resize(aFloorIndex);
	}
	else if (aFloorIndex == nCount)
	{
		m_aiRigidityCoeff.push_back(aValue);
	}
	else
	{
		m_aiRigidityCoeff[aFloorIndex] = aValue;
	}

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CBuilding::SetFloorMass_KG(int aFloorIndex, double aValue)
{
	int nCount = m_adMass.size();

	if (aValue < 0.0 || 
		aFloorIndex < 0 || 
		aFloorIndex >= MAX_NUMBER_OF_FLOORS || 
		aFloorIndex > nCount)
	{
		return false;
	}

	if (aValue == 0.0)
	{
		m_adMass.resize(aFloorIndex);
	}
	else if (aFloorIndex == nCount)
	{
		m_adMass.push_back(aValue);
	}
	else
	{
		m_adMass[aFloorIndex] = aValue;
	}

	return true;
}

//=================================== EOF ===================================
