/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			Building.h
//		DESCRIPTION:	interface for the CBuilding class
//
/////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_BUILDING_H__447C8805_6B01_45DD_B0F2_6BB3EC4095AB__INCLUDED_)
#define AFX_BUILDING_H__447C8805_6B01_45DD_B0F2_6BB3EC4095AB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MIN_QUALITY_FACTOR 2
#define MAX_QUALITY_FACTOR 6
#define MIN_ELASTIC_LIMIT  0.5
#define MAX_ELASTIC_LIMIT  1.0
#define MIN_RIGIDITY_COEFF 3000
#define MAX_RIGIDITY_COEFF 10000

#define DEFAULT_FLOOR_MASS 15000
#define DEFAULT_QUALITY_FACTOR 4
#define DEFAULT_ELASTIC_LIMIT 0.75
#define DEFAULT_RIGIDITY_COEFF 6000


//***************************************************************************
//                             CBuilding CLASS
//***************************************************************************

class CBuilding  
{
	long m_lID;
	CString m_strDescription;
	vector<double> m_adMass;
	vector<int> m_aiQualityFactor;
	vector<double> m_adElasticLimit;
	vector<int> m_aiRigidityCoeff;

public:

	CBuilding(long aID = ID_NEW_RECORD);
	CBuilding(const CBuilding& aBldg);
	const CBuilding& operator = (const CBuilding& aBldg);
	~CBuilding();

	bool Save();
	bool IsValid() const;

	// Get ------------------------------------------------------------------

	long GetID() const { return m_lID; }
	const char* GetDescription() const { return m_strDescription; }
	CString GetParamDesc() const;

	double GetFloorMass_KG(int aFloorIndex) const;
	int GetQualityFactor(int aFloorIndex) const;
	double GetElasticLimit_CM(int aFloorIndex) const;
	int GetRigidityCoeff(int aFloorIndex) const;

	double GetTotalMass_KG() const;
	inline int GetNumberOfFloors() const { return m_adMass.size(); }

	double GetInitialRigidity_KG_SEC2(int aFloorIndex) const;
	double GetInitialDamping_KG_SEC(int aFloorIndex) const;
	double GetRigidity_KG_SEC2(int aFloorIndex, double aRelDisp) const;
	double GetDamping_KG_SEC(int aFloorIndex, double aRelDisp) const;

	// Set ------------------------------------------------------------------

	bool SetDescription(const char* aValue);

	bool SetFloorMass_KG(int aFloorIndex, double aValue);
	bool SetQualityFactor(int aFloorIndex, int aValue);
	bool SetElasticLimit_CM(int aFloorIndex, double aValue);
	bool SetRigidityCoeff(int aFloorIndex, int aValue);
};

#endif // !defined(AFX_BUILDING_H__447C8805_6B01_45DD_B0F2_6BB3EC4095AB__INCLUDED_)

//=================================== EOF ===================================
