/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			ChildView.cpp
//		DESCRIPTION:	implementation of the CChildView class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "ChildView.h"
#include "Building.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MARGIN_X 20
#define MARGIN_Y 20
#define GRAPH_WIDTH 200


//***************************************************************************
//                             CChildView CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CChildView,CWnd )
	//{{AFX_MSG_MAP(CChildView)
	ON_WM_PAINT()
	ON_WM_VSCROLL()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_HSCROLL()
	ON_WM_MOUSEWHEEL()
	ON_COMMAND(ID_PROJECT_PRINT, OnProjectPrint)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_PRINT, OnUpdateProjectPrint)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//**************************** Setup/cleanup ********************************
/////////////////////////////////////////////////////////////////////////////
CChildView::CChildView() :
m_Project(g_Application.GetProject()),
m_iSnapshotFloor(g_Application.GetSnapshotFloor()),
m_dMaxExtentsX(0.0)
{
	UpdateFromProject();
}

/////////////////////////////////////////////////////////////////////////////
BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
	{
		return FALSE;
	}

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.style |= WS_VSCROLL | WS_HSCROLL;
	cs.lpszClass = AfxRegisterWndClass(
		CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), 
		HBRUSH(COLOR_WINDOW + 1), 
		NULL);

	m_sizeClient = CSize(0, MARGIN_Y);

	for (int i = 0, nTables = m_apTables.size(); i < nTables; i++)
	{
		SIZE tableSize = m_apTables[i]->GetSize();
		m_sizeClient.cx = max(m_sizeClient.cx, (i == nTables - 1) ? tableSize.cx : tableSize.cx + GRAPH_WIDTH + MARGIN_X);
		m_sizeClient.cy += (tableSize.cy + MARGIN_Y);
	}
	
	m_sizeClient.cx += (2 * MARGIN_X);

	cs.cx = m_sizeClient.cx;
	cs.cy = m_sizeClient.cy;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
int CChildView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd ::OnCreate(lpCreateStruct) == -1)
	{
		return -1;
	}
	
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
CChildView::~CChildView()
{
	// Delete tables
	for (int i = 0, nItems = m_apTables.size(); i < nItems; i++)
	{
		delete m_apTables[i];
	}

	// Delete graphs
	for (i = 0, nItems = m_apGraphs.size(); i < nItems; i++)
	{
		delete m_apGraphs[i];
	}
}

//******************************* Commands **********************************
/////////////////////////////////////////////////////////////////////////////
void CChildView::OnProjectPrint() 
{
	CPrintDialog dlg(false);

	if (dlg.DoModal() == IDOK)
	{
		// Create printer DC, start printing document

		CDC dc;
		dc.Attach(dlg.GetPrinterDC());

		DOCINFO di;
		di.cbSize = sizeof(di);
		di.lpszDocName = "Snapshot View";
		di.lpszOutput = NULL;
		di.lpszDatatype = NULL;
		di.fwType = 0;

		dc.StartDoc(&di);
		dc.StartPage();

		// Create window-compatible DC and draw on it

		CPaintDC winDc(this); 

		CDC memDc; 
		memDc.CreateCompatibleDC(&winDc);

		CBitmap memBitmap;
		memBitmap.CreateCompatibleBitmap(&winDc, m_sizeClient.cx, m_sizeClient.cy);
		CBitmap* pOldBitmap = memDc.SelectObject(&memBitmap);
		memDc.PatBlt(0, 0, m_sizeClient.cx, m_sizeClient.cy, WHITENESS);

		OnDraw(&memDc);

		// Stretch bitmap onto printer DC

		int iMargins = int(0.1 * double(dc.GetDeviceCaps(HORZRES)));
		int iTargetPixelsX = dc.GetDeviceCaps(HORZRES) - iMargins;
		float fScale = float(iTargetPixelsX) / float(m_sizeClient.cx);
		int iTargetPixelsY = int(float(m_sizeClient.cy) * fScale);

		dc.StretchBlt(
			iMargins / 2, 
			iMargins / 2, 
			iTargetPixelsX, 
			iTargetPixelsY, 
			&memDc, 
			0, 
			0, 
			m_sizeClient.cx, 
			m_sizeClient.cy, 
			SRCCOPY);

		// End printing

		dc.EndPage();
		dc.EndDoc();

		// Cleanup

		memDc.SelectObject(pOldBitmap);
		memDc.DeleteDC();
		memBitmap.DeleteObject();

		dc.DeleteDC();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::OnUpdateProjectPrint(CCmdUI* pCmdUI) 
{
}

//******************************* Messages **********************************
/////////////////////////////////////////////////////////////////////////////
void CChildView::OnPaint() 
{
	// Device context for painting
	CPaintDC dc(this); 
	OnDraw(&dc);
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::OnDraw(CDC* aDC)
{
	int iHScrollPos = GetScrollPos(SB_HORZ);
	int iVScrollPos = GetScrollPos(SB_VERT);
	
	// Offset point
	CPoint offset(MARGIN_X - iHScrollPos, MARGIN_Y - iVScrollPos);

	for (int i = 0, nTables = m_apTables.size(); i < nTables; i++)
	{
		SIZE tableSize = m_apTables[i]->GetSize();

		CRect rectGraph(
			offset.x, 
			offset.y + tableSize.cy - m_apTables[i]->GetCellHeight() * m_apTables[i]->GetNumberOfCells(), 
			offset.x + GRAPH_WIDTH, 
			offset.y + tableSize.cy);

		CPoint pntTable(
			//(i == nTables - 1) ? offset.x : offset.x + GRAPH_WIDTH + MARGIN_X, 
			offset.x + GRAPH_WIDTH + MARGIN_X,
			offset.y);

		if (i < m_apGraphs.size())
		{
			m_apGraphs[i]->OnDraw(aDC, rectGraph);
		}

		m_apTables[i]->OnDraw(aDC, &pntTable);

		// Set the offset point for the next table / graph
		offset.y += (tableSize.cy + MARGIN_Y);
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::OnSize(UINT nType, int cx, int cy) 
{
	CWnd ::OnSize(nType, cx, cy);
	
	if (m_sizeClient.cx <= cx)
	{
		SetScrollPos(SB_HORZ, 0, TRUE);
		ShowScrollBar(SB_HORZ, FALSE);
	}
	else
	{
		ShowScrollBar(SB_HORZ, TRUE);
		int iVBarWidth = GetSystemMetrics(SM_CYVTHUMB);
		SetScrollRange(SB_HORZ, 0, m_sizeClient.cx - cx + iVBarWidth, FALSE);	
	}

	if (m_sizeClient.cy <= cy)
	{
		SetScrollPos(SB_VERT, 0, TRUE);
		ShowScrollBar(SB_VERT, FALSE);
	}
	else
	{
		ShowScrollBar(SB_VERT, TRUE);
		int iHBarHeight = GetSystemMetrics(SM_CXHTHUMB);
		SetScrollRange(SB_VERT, 0, m_sizeClient.cy - cy + iHBarHeight, FALSE);	
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int iNewPos = nPos;

	CRect rect;
	GetClientRect(rect);
	int iVBarWidth = GetSystemMetrics(SM_CYVTHUMB);

	switch (nSBCode)
	{
	case SB_LINEDOWN: // Scroll one line down 
		iNewPos = min(GetScrollPos(SB_HORZ) + 30, 
			          m_sizeClient.cx - rect.Width() + iVBarWidth);
		break;

	case SB_LINEUP: // Scroll one line up 
		iNewPos = max(GetScrollPos(SB_HORZ) - 30, 0);
		break;

	case SB_PAGEDOWN: // Scroll one page down 
		iNewPos = min(GetScrollPos(SB_HORZ) + rect.Width(), 
			          m_sizeClient.cx - rect.Width() + iVBarWidth);
		break;

	case SB_PAGEUP: // Scroll one page up 
		iNewPos = max(GetScrollPos(SB_HORZ) - rect.Width(), 0);
		break;

	case SB_THUMBPOSITION: // Scroll to the absolute position
	case SB_THUMBTRACK:    // Drag scroll box to specified position
		break; // The current position is provided in nPos

	case SB_TOP:       // Scroll to top 
	case SB_BOTTOM:    // Scroll to bottom 
	case SB_ENDSCROLL: // End scroll 
	default:
		return;
	}

	SetScrollPos(SB_HORZ, iNewPos, TRUE);
	InvalidateRect(NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int iNewPos = nPos;

	CRect rect;
	GetClientRect(rect);
	int iHBarHeight = GetSystemMetrics(SM_CXHTHUMB);

	switch (nSBCode)
	{
	case SB_LINEDOWN: // Scroll one line down 
		iNewPos = min(GetScrollPos(SB_VERT) + 30, 
			          m_sizeClient.cy - rect.Height() + iHBarHeight);
		break;

	case SB_LINEUP: // Scroll one line up 
		iNewPos = max(GetScrollPos(SB_VERT) - 30, 0);
		break;

	case SB_PAGEDOWN: // Scroll one page down 
		iNewPos = min(GetScrollPos(SB_VERT) + rect.Height(), 
			          m_sizeClient.cy - rect.Height() + iHBarHeight);
		break;

	case SB_PAGEUP: // Scroll one page up 
		iNewPos = max(GetScrollPos(SB_VERT) - rect.Height(), 0);
		break;

	case SB_THUMBPOSITION: // Scroll to the absolute position
	case SB_THUMBTRACK:    // Drag scroll box to specified position
		break; // The current position is provided in nPos

	case SB_TOP:       // Scroll to top 
	case SB_BOTTOM:    // Scroll to bottom 
	case SB_ENDSCROLL: // End scroll 
	default:
		return;
	}

	SetScrollPos(SB_VERT, iNewPos, TRUE);
	InvalidateRect(NULL);
}

/////////////////////////////////////////////////////////////////////////////
BOOL CChildView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	CRect rect;
	GetClientRect(rect);
	int iHBarHeight = GetSystemMetrics(SM_CXHTHUMB);

	int iNewPos = GetScrollPos(SB_VERT) - zDelta / 4;
	iNewPos = min(iNewPos, m_sizeClient.cy - rect.Height() + iHBarHeight);
	iNewPos = max(iNewPos, 0);

	SetScrollPos(SB_VERT, iNewPos, TRUE);
	InvalidateRect(NULL);

	return TRUE;
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
void CChildView::UpdateFromProject()
{
	// Clear data -----------------------------------------------------------

	for (int i = 0, nItems = m_apTables.size(); i < nItems; i++)
	{
		delete m_apTables[i];
	}

	m_apTables.clear();

	for (i = 0, nItems = m_apGraphs.size(); i < nItems; i++)
	{
		delete m_apGraphs[i];
	}

	m_apGraphs.clear();

	m_dMaxExtentsX = 0.0;

	// Setup snapshot tables ------------------------------------------------

	const CBuilding* pBldg = m_Project.GetBuilding();
	ASSERT(pBldg);
	int nFloors = pBldg->GetNumberOfFloors();

	vector<const CIsoSystem*> v;
	m_Project.GetIsoSystems(v);

	for (i = 0; i < v.size(); i++)
	{
		m_apTables.push_back(new CTable());
		CTable* pTable = m_apTables.back();

		CString strIsoSys = CIsoSystem::GetNameByType(v[i]->GetType());
		double dTime = m_Project.GetMaxRelDispTime_SEC(v[i], m_iSnapshotFloor);

		CString strTitle;
		strTitle.Format(
			"%s. Snapshot for REL. V%u = %.2f cm, TIME = %.2f sec", 
			strIsoSys, 
			m_iSnapshotFloor, 
			fabs(m_Project.GetRelDisp_CM(v[i], m_iSnapshotFloor, dTime)),
			dTime);

		pTable->AddTitleLine(strTitle);

		SetSnapshotColumns(pTable);
		pTable->Resize(nFloors);
		SetSnapshotRows(pTable, pBldg, v[i]);

		m_apGraphs.push_back(new CGraphSnapshot(RGB(230, 230, 200)));
		SetSnapshotGraph(m_apGraphs.back(), pBldg, v[i]);
	}

	for (i = 0; i < v.size(); i++)
		m_apGraphs[i]->SetExtentsX(2.0 * m_dMaxExtentsX);

	// Setup performance evaluation table -----------------------------------

	for (i = 0; i < v.size(); i++)
	{
		m_apTables.push_back(new CTable());
		CTable* pTable = m_apTables.back();

		CString strTitle;
		strTitle.Format("Performance evaluation for %s", CIsoSystem::GetNameByType(v[i]->GetType()));
		pTable->AddTitleLine(strTitle);

		SetPerformanceEvalColumns(pTable);
		pTable->Resize(nFloors);
		SetPerformanceEvalRows(pTable, pBldg, v[i]);
	}

	// Setup joint table ----------------------------------------------------

	m_apTables.push_back(new CTable());
	CTable* pTable = m_apTables.back();

	pTable->AddTitleLine("Relative displacement summary");

	SetJointColumns(pTable, pBldg, v);
	pTable->Resize(pBldg->GetNumberOfFloors());

	SetJointRows(pTable, pBldg, v);

	// Eq table -------------------------------------------------------------

	if (m_Project.GetEqCone())
	{
		m_apTables.push_back(new CTable());
		pTable = m_apTables.back();
		SetEarthquakeConeTable(pTable, m_Project.GetEqCone());
	}

	// Building table -------------------------------------------------------
	
	m_apTables.push_back(new CTable());
	pTable = m_apTables.back();
	SetBuildingTable(pTable, pBldg);

	// Sliding system table -------------------------------------------------

	for (i = 0; i < v.size(); i++)
	{
		if (v[i]->GetType() == CIsoSystem::TYPE_SLIDING)
		{
			m_apTables.push_back(new CTable());
			pTable = m_apTables.back();
			SetIsoSysSlidingTable(pTable, static_cast<const CIsoSystemSliding*>(v[i]));
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetSnapshotColumns(CTable* aTable)
{
	aTable->AddColumn("Floor",          80, CTable::JUSTIFY_RIGHT);
	aTable->AddColumn("u (cm)",        150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("v (cm)",        150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("u\" (cm/sec2)", 150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("m (kg)",        150, CTable::JUSTIFY_RIGHT);
	aTable->AddColumn("F=mu\" (kN)",   150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("V=Kv (kN)",     150, CTable::JUSTIFY_CENTER_DECIMAL);
	//aTable->AddColumn("K (kT/sec2)",   145, CTable::JUSTIFY_CENTER_DECIMAL);
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetSnapshotRows(CTable* aTable, 
								 const CBuilding* aBuilding, 
								 const CIsoSystem* aIsoSys)
{
	double dTime = m_Project.GetMaxRelDispTime_SEC(aIsoSys, m_iSnapshotFloor);

	for (int iFloor = 0, nFloors = aBuilding->GetNumberOfFloors(); iFloor < nFloors; iFloor++)
	{
		int iRowIndex = nFloors - iFloor - 1;

		CString str;
		str.Format("%d",    iFloor);
		aTable->SetSell(0,  iRowIndex, str);

		str.Format("%+.2f", m_Project.GetAbsDisp_CM(aIsoSys, iFloor, dTime));
		aTable->SetSell(1,  iRowIndex, str);

		str.Format("%+.2f", m_Project.GetRelDisp_CM(aIsoSys, iFloor, dTime));
		aTable->SetSell(2,  iRowIndex, str);

		str.Format("%+.1f", m_Project.GetAbsAccel_CM_SEC2(aIsoSys, iFloor, dTime));
		aTable->SetSell(3,  iRowIndex, str);

		str.Format("%u",    (UINT)aBuilding->GetFloorMass_KG(iFloor));
		aTable->SetSell(4,  iRowIndex, str);

		str.Format("%+.1f", m_Project.GetForce_KN(aIsoSys, iFloor, dTime));
		aTable->SetSell(5,  iRowIndex, str);

		str.Format("%.1f",  fabs(m_Project.GetShear_KN(aIsoSys, iFloor, dTime)));
		aTable->SetSell(6,  iRowIndex, str);

		//str.Format("%.1f",  fabs(m_Project.GetK_KT_SEC2(aIsoSys, iFloor, dTime)));    
		//aTable->SetSell(7,  iRowIndex, str);
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetPerformanceEvalColumns(CTable* aTable)
{
	aTable->AddColumn("Floor",          80, CTable::JUSTIFY_RIGHT);
	aTable->AddColumn("Max v (cm)",    150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("t (sec)",       150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("u\" (cm/sec2)", 150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("V=Kv (kN)",     150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("SPR=v/vmax",    150, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("Performance",   150, CTable::JUSTIFY_RIGHT);
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetPerformanceEvalRows(CTable* aTable, 
										const CBuilding* aBuilding, 
										const CIsoSystem* aIsoSys)
{
	for (int iFloor = 0, nFloors = aBuilding->GetNumberOfFloors(); iFloor < nFloors; iFloor++)
	{
		double dTime = m_Project.GetMaxRelDispTime_SEC(aIsoSys, iFloor);
		int iRowIndex = nFloors - iFloor - 1;

		// Floor number
		CString str;
		str.Format("%d", iFloor);
		aTable->SetSell(0, iRowIndex, str);

		// Relative displacement
		double dRelDisp = m_Project.GetRelDisp_CM(aIsoSys, iFloor, dTime);
		str.Format("%+.2f", dRelDisp);
		aTable->SetSell(1, iRowIndex, str);

		// Time
		str.Format("%.2f", dTime);
		aTable->SetSell(2, iRowIndex, str);

		// Acceleration
		str.Format("%+.1f", m_Project.GetAbsAccel_CM_SEC2(aIsoSys, iFloor, dTime));
		aTable->SetSell(3, iRowIndex, str);

		// Shear
		str.Format("%.1f", fabs(m_Project.GetShear_KN(aIsoSys, iFloor, dTime)));
		aTable->SetSell(4, iRowIndex, str);

		// Seismic Performance Ratio
		double dSPR = fabs(dRelDisp) / (double(aBuilding->GetQualityFactor(iFloor)) * aBuilding->GetElasticLimit_CM(iFloor));
		str.Format("%.2f", dSPR);
		aTable->SetSell(5, iRowIndex, str);

		if (iFloor)
		{
			// Meaning of SPR
			if (dSPR <= 1.0)
			{
				str = "GOOD";
			}
			else if (dSPR <= 1.5)
			{
				str = "FAILURE";
			}
			else
			{
				str = "OLLAPSE";
			}

			aTable->SetSell(6, iRowIndex, str);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetSnapshotGraph(CGraphSnapshot* aGraph, 
								  const CBuilding* aBuilding, 
								  const CIsoSystem* aIsoSys)
{
	double dTime = m_Project.GetMaxRelDispTime_SEC(aIsoSys, m_iSnapshotFloor);

	vector<double> floors;
	vector<double> disps;

	floors.push_back(-1);
	disps.push_back(m_Project.GetEq()->GetAbsDisp_CM(dTime));

	for (int iFloor = 0, nFloors = aBuilding->GetNumberOfFloors(); iFloor < nFloors; iFloor++)
	{
		double dDisp = m_Project.GetAbsDisp_CM(aIsoSys, iFloor, dTime);
		m_dMaxExtentsX = max(m_dMaxExtentsX, fabs(dDisp));

		floors.push_back(iFloor);
		disps.push_back(dDisp);
	}

	aGraph->AddGraph(floors, disps, RGB(0, 0, 255), 3);
	aGraph->ShowFrame(true);
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetJointColumns(CTable* aTable, 
								 const CBuilding* aBuilding,
								 vector<const CIsoSystem*>& aIsoSys)
{
	aTable->AddColumn("Vmax (cm)", 120, CTable::JUSTIFY_RIGHT);

	for (int iIsoSys = 0, nIsoSystems = aIsoSys.size(); iIsoSys < nIsoSystems; iIsoSys++)
	{
		aTable->AddColumn(
			CIsoSystem::GetNameByType(aIsoSys[iIsoSys]->GetType()), 
			250, 
			CTable::JUSTIFY_CENTER_DECIMAL);
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetJointRows(CTable* aTable, 
							  const CBuilding* aBuilding, 
							  vector<const CIsoSystem*>& aIsoSys)
{
	for (int iFloor = 0, nFloors = aBuilding->GetNumberOfFloors(); iFloor < nFloors; iFloor++)
	{
		int iRowIndex = nFloors - iFloor - 1;

		for (int iIsoSys = 0, nIsoSystems = aIsoSys.size(); iIsoSys < nIsoSystems; iIsoSys++)
		{
			// Floor number
			CString str;
			str.Format("V%d", iFloor);
			aTable->SetSell(0, iRowIndex, str);

			double dTime = m_Project.GetMaxRelDispTime_SEC(aIsoSys[iIsoSys], iFloor);
			str.Format("%.2f", fabs(m_Project.GetRelDisp_CM(aIsoSys[iIsoSys], iFloor, dTime)));
			aTable->SetSell(iIsoSys + 1, iRowIndex, str);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetEarthquakeConeTable(CTable* aTable, 
										const CEarthquakeCone* aCone)
{
	aTable->AddColumn("Param. name", 400, CTable::JUSTIFY_LEFT);
	aTable->AddColumn("Param. value", 400, CTable::JUSTIFY_CENTER_DECIMAL);

	CString str;
	str.Format("Cone Earthquake %s", aCone->GetDescription());
	aTable->AddTitleLine(str);

	aTable->Resize(5);
	int iRowIndex = 0;

	str.Format("%.2f", aCone->GetTotalTime_SEC());
	aTable->SetSell(0, iRowIndex, "Total time (sec)");
	aTable->SetSell(1, iRowIndex++, str);

	str.Format("%.2f", aCone->GetStep_SEC());
	aTable->SetSell(0, iRowIndex, "Step (sec)");
	aTable->SetSell(1, iRowIndex++, str);

	str.Format("%.2f", aCone->GetMaxVelocity_CM_SEC());
	aTable->SetSell(0, iRowIndex, "Max. velocity (cm/sec)");
	aTable->SetSell(1, iRowIndex++, str);

	str.Format("%.2f", aCone->GetMinPeriod_SEC());
	aTable->SetSell(0, iRowIndex, "Min. period (sec)");
	aTable->SetSell(1, iRowIndex++, str);

	str.Format("%.2f", aCone->GetMaxPeriod_SEC());
	aTable->SetSell(0, iRowIndex, "Max. period (sec)");
	aTable->SetSell(1, iRowIndex++, str);
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetBuildingTable(CTable* aTable, const CBuilding* aBuilding)
{
	int nFloors = aBuilding->GetNumberOfFloors();

	CString str;
	str.Format(
		"Building %s, %d floors, mass = %d kg ", 
		aBuilding->GetDescription(),
		nFloors - 1,
		(int)aBuilding->GetTotalMass_KG());

	aTable->AddTitleLine(str);

	aTable->AddColumn("Floor",          100, CTable::JUSTIFY_RIGHT);
	aTable->AddColumn("Mass (kg)",      200, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("Quality Factor", 200, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("El. Limit (cm)", 200, CTable::JUSTIFY_CENTER_DECIMAL);
	aTable->AddColumn("Rigidity Coeff", 200, CTable::JUSTIFY_CENTER_DECIMAL);

	aTable->Resize(nFloors);

	for (int iFloor = 0; iFloor < nFloors; iFloor++)
	{
		int iRowIndex = nFloors - iFloor - 1;

		// Floor number
		CString str;
		iFloor ? str.Format("%d", iFloor) : str = "Bsmt";
		aTable->SetSell(0, iRowIndex, str);

		str.Format("%d", (int)aBuilding->GetFloorMass_KG(iFloor));
		aTable->SetSell(1, iRowIndex, str);

		iFloor ? str.Format("%d", aBuilding->GetQualityFactor(iFloor)) : str = "N/A";
		aTable->SetSell(2, iRowIndex, str);

		iFloor ? str.Format("%.2f", aBuilding->GetElasticLimit_CM(iFloor)) : str = "N/A";
		aTable->SetSell(3, iRowIndex, str);

		iFloor ? str.Format("%d", aBuilding->GetRigidityCoeff(iFloor)) : str = "N/A";
		aTable->SetSell(4, iRowIndex, str);
	}
}

/////////////////////////////////////////////////////////////////////////////
void CChildView::SetIsoSysSlidingTable(CTable* aTable, 
									   const CIsoSystemSliding* aIsoSys)
{
	aTable->AddColumn("Param. name", 400, CTable::JUSTIFY_LEFT);
	aTable->AddColumn("Param. value", 400, CTable::JUSTIFY_CENTER_DECIMAL);

	CString str;
	str.Format("Earthquake Protector %s", aIsoSys->GetDescription());
	aTable->AddTitleLine(str);

	aTable->Resize(2);
	int iRowIndex = 0;

	str.Format("%.2f", aIsoSys->GetNaturalFriction());
	aTable->SetSell(0, iRowIndex, "Natural friction coeff.");
	aTable->SetSell(1, iRowIndex++, str);

	str.Format("%.2f", aIsoSys->GetRadius_CM());
	aTable->SetSell(0, iRowIndex, "Radius (cm)");
	aTable->SetSell(1, iRowIndex++, str);
}

//=================================== EOF ===================================
