/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			EPET.cpp
//		DESCRIPTION:	implementation of the CEPETApp class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "OpenDatabaseDlg.h"

#include "StdArray_i.c"
#include "Graphics_i.c"
#include "Space3d_i.c"
#include "Database_i.c"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// The one and only CEPETApp object
CEPETApp g_Application;


//***************************************************************************
//                             CAboutDlg CLASS
//***************************************************************************

class CAboutDlg : public CDialog
{
public:

	CAboutDlg();

	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

protected:

	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}


//***************************************************************************
//                             CEPETApp CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CEPETApp, CWinApp)
	//{{AFX_MSG_MAP(CEPETApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	//}}AFX_MSG_MAP
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

//****************************** Setup/cleanup ******************************
/////////////////////////////////////////////////////////////////////////////
CEPETApp::CEPETApp() :
m_pDBase(NULL),
m_pProject(NULL),
m_pSelIsoSystem(NULL),
m_iSnapshotFloor(1),
m_bExitInstance(true)
{
}

/////////////////////////////////////////////////////////////////////////////
BOOL CEPETApp::InitInstance()
{
	CoInitializeEx(NULL, COINIT_MULTITHREADED);
	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(_T(""));

	// Restore the database info
	COpenDatabaseDlg::SetDatabaseName(GetProfileString("Database", "Database Name"));
	COpenDatabaseDlg::SetUserName    (GetProfileString("Database", "User Name"));
	COpenDatabaseDlg::SetPassword    (GetProfileString("Database", "Password"));
	COpenDatabaseDlg::SetReadOnly    (GetProfileInt   ("Database", "Read Only", FALSE) == TRUE);

	// To create the main window, this code creates a new frame window
	// object and then sets it as the application's main window object.
	CMDIFrameWnd* pFrame = new CMainFrame;
	m_pMainWnd = pFrame;

	// create main MDI frame window
	if (!pFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;

	// Try to load shared MDI menus and accelerator table
	// TODO: add additional member variables and load calls for
	// additional menu types your application may need. 

	HINSTANCE hInst = AfxGetResourceHandle();
	m_hMDIMenu  = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_MAINFRAME));
	m_hMDIAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_MAINFRAME));

	// The main window has been initialized, so show and update it.
	m_pMainWnd->ShowWindow(m_nCmdShow);
	m_pMainWnd->UpdateWindow();

	m_pProject = new CProject;

	if (COpenDatabaseDlg::GetDatabaseName())
	{
		// Re-open the database
		OpenDatabase(
			COpenDatabaseDlg::GetDatabaseName(),
			COpenDatabaseDlg::GetUserName(),
			COpenDatabaseDlg::GetPassword(),
			COpenDatabaseDlg::IsReadOnly());
	}

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
int CEPETApp::ExitInstance() 
{
	if (!m_bExitInstance)
	{
		return 0;
	}

	m_bExitInstance = false;

	// Save the database info
	WriteProfileString("Database", "Database Name", COpenDatabaseDlg::GetDatabaseName());
	WriteProfileString("Database", "User Name",     COpenDatabaseDlg::GetUserName());
	WriteProfileString("Database", "Password",      COpenDatabaseDlg::GetPassword());
	WriteProfileInt   ("Database", "Read Only",     COpenDatabaseDlg::IsReadOnly());
	
	//TODO: handle additional resources you may have added
	if (m_hMDIMenu != NULL)
	{
		FreeResource(m_hMDIMenu);
	}

	if (m_hMDIAccel != NULL)
	{
		FreeResource(m_hMDIAccel);
	}

	if (m_pProject)
	{
		delete m_pProject;
	}

	if (m_pDBase)
	{
		delete m_pDBase;
	}

	CoUninitialize();

	return CWinApp::ExitInstance();
}

/////////////////////////////////////////////////////////////////////////////
CEPETApp::~CEPETApp()
{
}

/////////////////////////////////////////////////////////////////////////////
bool CEPETApp::OpenDatabase(const char* aDatabaseName, 
							const char* aUserName, 
							const char* aPassword,
							bool aReadOnly)
{
	// Create the dbase specified by user
	CEPETDatabase* pDBase = new CEPETDatabase(
		aDatabaseName, 
		aUserName, 
		aPassword, 
		aReadOnly);

	if (pDBase->IsValid())
	{
		if (m_pDBase)
		{
			// Delete old dbase
			delete m_pDBase;
		}

		// Point to new dbase and return success
		m_pDBase = pDBase;

		m_pMainWnd->PostMessage(WM_UPDATE_MAINFRAME_UI);
		return true;
	}

	// Cleanup and return failure
	delete pDBase;
	return false;
}

//******************************** Commands *********************************
/////////////////////////////////////////////////////////////////////////////
void CEPETApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

//=================================== EOF ===================================
