/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			EPETDatabase.h
//		DESCRIPTION:	interface for the CEPETDatabase class
//
/////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_EPETDATABASE_H__BA8E1140_C8B0_4536_8874_0FFA54AD2E93__INCLUDED_)
#define AFX_EPETDATABASE_H__BA8E1140_C8B0_4536_8874_0FFA54AD2E93__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define ID_NEW_RECORD 0
#define MAX_NUMBER_OF_FLOORS 21


//***************************************************************************
//                          CEPETDatabase CLASS
//***************************************************************************

class CEPETDatabase  
{
private:

	IDBase* m_pDBase;

	IDbTable* m_pBuildingsTable;
	IDbTable* m_pSecSystemsTable;
	IDbTable* m_pIsoSysAFMSTable;
	IDbTable* m_pIsoSysSlidingTable;
	IDbTable* m_pIsoSysShearTable;
	IDbTable* m_pEqRealTable;
	IDbTable* m_pEqTimeHistoryTable;
	IDbTable* m_pEqConeTable;
	IDbTable* m_pProjectsTable;

protected:

	void UpdateLayout();

public:

	CEPETDatabase(
		const char* aDatabaseName, 
		const char* aUserName, 
		const char* aPassword,
		bool aReadOnly = false);

	~CEPETDatabase();

	bool IsValid() const { return m_pDBase != NULL; }

	inline IDbTable* GetBuildingsTable()     { return m_pBuildingsTable; }
	inline IDbTable* GetSecSystemsTable()    { return m_pSecSystemsTable; }
	inline IDbTable* GetIsoSysAFMSTable()    { return m_pIsoSysAFMSTable; }
	inline IDbTable* GetIsoSysSlidingTable() { return m_pIsoSysSlidingTable; }
	inline IDbTable* GetIsoSysShearTable()   { return m_pIsoSysShearTable; }
	inline IDbTable* GetEqRealTable()        { return m_pEqRealTable; }
	inline IDbTable* GetEqTimeHistoryTable() { return m_pEqTimeHistoryTable; }
	inline IDbTable* GetEqConeTable()        { return m_pEqConeTable; }
	inline IDbTable* GetProjectsTable()      { return m_pProjectsTable; }

	IDbRecordSet* TableToRecordSet(IDbTable* aTable);

	CString GetFieldAsString(IDbTable* aTable, const char* aFieldName, long aRecordID) const;
	float GetFieldAsFloat(IDbTable* aTable, const char* aFieldName, long aRecordID) const;
	long GetFieldAsInteger(IDbTable* aTable, const char* aFieldName, long aRecordID) const;

	bool SetFieldToString(IDbTable* aTable, const char* aFieldName, long aRecordID, const char* aValue) const;
	bool SetFieldToFloat(IDbTable* aTable, const char* aFieldName, long aRecordID, double aValue) const;
	bool SetFieldToInteger(IDbTable* aTable, const char* aFieldName, long aRecordID, long aValue) const;

	inline void Commit() { m_pDBase->Commit(); }
};

#endif // !defined(AFX_EPETDATABASE_H__BA8E1140_C8B0_4536_8874_0FFA54AD2E93__INCLUDED_)

//=================================== EOF ===================================
