/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			Earthquake.h
//		DESCRIPTION:	interface for the CEarthquake class
//
/////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_EARTHQUAKE_H__75B6B803_9329_4388_801D_A1D020890704__INCLUDED_)
#define AFX_EARTHQUAKE_H__75B6B803_9329_4388_801D_A1D020890704__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


//***************************************************************************
//                            CEarthquake CLASS
//***************************************************************************

class CEarthquake  
{
public:

	enum EType
	{
		TYPE_UNDEFINED = 0,
		TYPE_REAL      = 1,
		TYPE_CONE      = 2,
	};

protected:

	long m_lID;
	CString m_strDescription;

	double m_dStep_SEC;
	double m_dTotalTime_SEC;

	vector<double> m_adAbsDisp;
	vector<double> m_adAbsVel;
	vector<double> m_adAbsAccel;
	vector<double> m_adAbsAccel2;

	bool m_bCalculated;

public:

	CEarthquake(long aID = ID_NEW_RECORD);
	CEarthquake(const CEarthquake& aEq);
	const CEarthquake& operator = (const CEarthquake& aEq);
	virtual ~CEarthquake();

	virtual bool Save() = 0;
	virtual bool IsValid() const = 0;
	virtual CString GetParamDesc() const = 0;

	// Get ------------------------------------------------------------------

	inline long GetID() const { return m_lID; }
	inline const char* GetDescription() const { return m_strDescription; }

	inline double GetStep_SEC() const { return m_dStep_SEC; }
	inline double GetTotalTime_SEC() const { return m_dTotalTime_SEC; }

	double GetAbsDisp_CM(double aTime) const;
	double GetAbsVel_CM_SEC(double aTime) const;
	double GetAbsAccel_CM_SEC2(double aTime) const;
	double GetAbsAccel2_CM_SEC3(double aTime) const;

protected:

	int TimeToIndex(double aTime) const;

	// Set ------------------------------------------------------------------

public:

	inline bool SetDescription(const char* aValue) { m_strDescription = aValue; return true; }

	bool SetStep_SEC(double aValue);
	bool SetTotalTime_SEC(double aValue);

	// Calculus -------------------------------------------------------------

	inline bool IsCalculated() const { return m_bCalculated; }
	virtual bool Calculate() = 0;
};


//***************************************************************************
//                          CEarthquakeCone CLASS
//***************************************************************************

class CEarthquakeCone : public CEarthquake
{
	double m_dMaxVelocity_CM_SEC;
	double m_dMinPeriod_SEC;
	double m_dMaxPeriod_SEC;

public:

	CEarthquakeCone(long aID = ID_NEW_RECORD);

	// Archiving ------------------------------------------------------------

	virtual bool Save();
	virtual bool IsValid() const;
	virtual CString GetParamDesc() const;

	// Get ------------------------------------------------------------------

	inline double GetMaxVelocity_CM_SEC() const { return m_dMaxVelocity_CM_SEC; }
	inline double GetMinPeriod_SEC() const { return m_dMinPeriod_SEC; }
	inline double GetMaxPeriod_SEC() const { return m_dMaxPeriod_SEC; }

	bool SetMaxVelocity_CM_SEC(double aValue);
	bool SetMinPeriod_SEC(double aValue);
	bool SetMaxPeriod_SEC(double aValue);

	// Calculus -------------------------------------------------------------

	virtual bool Calculate();
};


//***************************************************************************
//                          CEarthquakeReal CLASS
//***************************************************************************

class CEarthquakeReal : public CEarthquake
{
	CString m_strName;
	long m_lLocalTime;
	double m_dLatitude;
	double m_dLongitude;
	double m_dDepth_KM;
	double m_dMagnitude_ML;
	double m_dTransNaturalPeriod_SEC;
	double m_dDamping;
	double m_dSensitivity_CM_G;
	CString m_strStationName;
	CString m_strStationID;
	CString m_strAccelID;

public:

	CEarthquakeReal(long aID = ID_NEW_RECORD);

	// Archiving ------------------------------------------------------------

	virtual bool Save();
	virtual bool IsValid() const;
	virtual CString GetParamDesc() const;

	// Get ------------------------------------------------------------------

	inline const char* GetName                  () const { return m_strName; }
	inline long        GetLocalTime             () const { return m_lLocalTime; }
	inline double      GetLatitude              () const { return m_dLatitude; }
	inline double      GetLongitude             () const { return m_dLongitude; }
	inline double      GetDepth_KM              () const { return m_dDepth_KM; }
	inline double      GetMagnitude_ML          () const { return m_dMagnitude_ML; }
	inline double      GetTransNaturalPeriod_SEC() const { return m_dTransNaturalPeriod_SEC; }
	inline double      GetDamping               () const { return m_dDamping; }
	inline double      GetSensitivity_CM_G      () const { return m_dSensitivity_CM_G; }
	inline const char* GetStationName           () const { return m_strStationName; }
	inline const char* GetStationID             () const { return m_strStationID; }
	inline const char* GetAccelID               () const { return m_strAccelID; }

	// Set ------------------------------------------------------------------

	bool SetName                  (const char* aValue);
	bool SetLocalTime             (long aValue);
	bool SetLatitude              (double aValue);
	bool SetLongitude             (double aValue);
	bool SetDepth_KM              (double aValue);
	bool SetMagnitude_ML          (double aValue);
	bool SetTransNaturalPeriod_SEC(double aValue);
	bool SetDamping               (double aValue);
	bool SetSensitivity_CM_G      (double aValue);
	bool SetStationName           (const char* aValue);
	bool SetStationID             (const char* aValue);
	bool SetAccelID               (const char* aValue);

	// Calculus -------------------------------------------------------------

	virtual bool Calculate();
};


#endif // !defined(AFX_EARTHQUAKE_H__75B6B803_9329_4388_801D_A1D020890704__INCLUDED_)

//=================================== EOF ===================================
