/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			EditBuildingDlg.cpp
//		DESCRIPTION:	implementation of the CEditBuildingDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "EditBuildingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                        CEditBuildingDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CEditBuildingDlg, CDialog)
	//{{AFX_MSG_MAP(CEditBuildingDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnButtonAdd)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE, OnButtonRemove)
	ON_LBN_DBLCLK(IDC_LIST_FLOORS, OnDblclkListFloors)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CEditBuildingDlg::CEditBuildingDlg(long aBuildingID, CWnd* pParent) : 
CDialog(CEditBuildingDlg::IDD, pParent),
CBuilding(aBuildingID)
{
	//{{AFX_DATA_INIT(CEditBuildingDlg)
	m_strDescription = GetDescription();
	m_dElasticLimit = DEFAULT_ELASTIC_LIMIT;
	m_iBasementMass = GetFloorMass_KG(0);
	m_iFloorMass = DEFAULT_FLOOR_MASS;
	m_iRigidityCoeff = DEFAULT_RIGIDITY_COEFF;
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CEditBuildingDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// Setup Quality Factor Combo -------------------------------------------

	int iSelIndex = 0;
	
	for (int i = MIN_QUALITY_FACTOR; i <= MAX_QUALITY_FACTOR; i++)
	{
		CString str;
		str.Format("%d", i);
		int iIndex = m_cboxQualityFactor.AddString(str);
		m_cboxQualityFactor.SetItemData(iIndex, i);

		if (i == DEFAULT_QUALITY_FACTOR)
		{
			iSelIndex = iIndex;
		}
	}

	m_cboxQualityFactor.SetCurSel(iSelIndex);

	// Setup Floors ---------------------------------------------------------

	int nFloors = GetNumberOfFloors();

	for (i = 1; i < nFloors; i++)
	{
		FLOOR_DATA fd;
		fd.m_dMass = GetFloorMass_KG(i);
		fd.m_iQualityFactor = GetQualityFactor(i);
		fd.m_dElasticLimit = GetElasticLimit_CM(i);
		fd.m_iRigidityCoeff = GetRigidityCoeff(i);

		ASSERT(fd.m_dMass > 0.0);
		m_aFloors.push_back(fd);
	}

	UpdateCtrlState();
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
void CEditBuildingDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEditBuildingDlg)
	DDX_Control(pDX, IDC_LIST_FLOORS, m_listFloors);
	DDX_Control(pDX, IDC_COMBO_QUALITY_FACTOR, m_cboxQualityFactor);
	DDX_Control(pDX, IDOK, m_buttonOK);
	DDX_Control(pDX, IDC_BUTTON_REMOVE, m_buttonRemove);
	DDX_Control(pDX, IDC_BUTTON_ADD, m_buttonAdd);
	DDX_Text(pDX, IDC_EDIT_DESCRIPTION, m_strDescription);
	DDX_Text(pDX, IDC_EDIT_ELASTIC_LIMIT, m_dElasticLimit);
	DDV_MinMaxDouble(pDX, m_dElasticLimit, 0.5, 1.);
	DDX_Text(pDX, IDC_EDIT_BSMT_MASS, m_iBasementMass);
	DDV_MinMaxInt(pDX, m_iBasementMass, 100, 10000000);
	DDX_Text(pDX, IDC_EDIT_FLOOR_MASS, m_iFloorMass);
	DDV_MinMaxInt(pDX, m_iFloorMass, 100, 10000000);
	DDX_Text(pDX, IDC_EDIT_RIGIDITY_COEFF, m_iRigidityCoeff);
	DDV_MinMaxInt(pDX, m_iRigidityCoeff, 3000, 10000);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
void CEditBuildingDlg::OnButtonAdd() 
{
	UpdateData();
	
	if (!m_iFloorMass || m_listFloors.GetCount() == MAX_NUMBER_OF_FLOORS - 1)
		return;

	FLOOR_DATA fd;
	fd.m_dMass = m_iFloorMass;
	fd.m_iQualityFactor = m_cboxQualityFactor.GetItemData(m_cboxQualityFactor.GetCurSel());
	fd.m_dElasticLimit = m_dElasticLimit;
	fd.m_iRigidityCoeff = m_iRigidityCoeff;

	int iSelIndex = m_listFloors.GetCurSel();
	m_aFloors.insert(m_aFloors.begin() + iSelIndex + 1, fd);
	UpdateCtrlState(iSelIndex + 1);
}

/////////////////////////////////////////////////////////////////////////////
void CEditBuildingDlg::OnButtonRemove() 
{
	int iSelIndex = m_listFloors.GetCurSel();

	if (iSelIndex != LB_ERR)
	{
		m_aFloors.erase(m_aFloors.begin() + iSelIndex);
		UpdateCtrlState(iSelIndex);
	}
}

/////////////////////////////////////////////////////////////////////////////
void CEditBuildingDlg::OnDblclkListFloors() 
{
	OnButtonRemove();
}

/////////////////////////////////////////////////////////////////////////////
void CEditBuildingDlg::OnOK() 
{
	UpdateData();

	SetDescription(m_strDescription);
	SetFloorMass_KG(0, double(m_iBasementMass));

	for (int i = 0, nCount = m_aFloors.size(); i < nCount; i++)
	{
		SetFloorMass_KG   (i + 1, m_aFloors[i].m_dMass);
		SetQualityFactor  (i + 1, m_aFloors[i].m_iQualityFactor);
		SetElasticLimit_CM(i + 1, m_aFloors[i].m_dElasticLimit);
		SetRigidityCoeff  (i + 1, m_aFloors[i].m_iRigidityCoeff);
	}

	for (i++; i < MAX_NUMBER_OF_FLOORS; i++)
	{
		SetFloorMass_KG   (i, 0.0);
		SetQualityFactor  (i, 0);
		SetElasticLimit_CM(i, 0.0);
		SetRigidityCoeff  (i, 0);
	}

	if (Save())
	{
		CDialog::OnOK();
	}
	else
	{
		MessageBox("Invalid data");
	}
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
CString CEditBuildingDlg::ComposeFloorString(int aFloor, const FLOOR_DATA* aFD) const
{
	CString str;
	str.Format(
		"Floor: %u Mass: %u QF: %u EL: %.2f K: %u", 
		(UINT)aFloor,
		(UINT)aFD->m_dMass, 
		(UINT)aFD->m_iQualityFactor, 
		(double)aFD->m_dElasticLimit,
		(UINT)aFD->m_iRigidityCoeff);

	return str;
}

/////////////////////////////////////////////////////////////////////////////
void CEditBuildingDlg::UpdateCtrlState(int iSelFloor)
{
	m_listFloors.ResetContent();

	for (int i = 0, nCount = m_aFloors.size(); i < nCount; i++)
	{
		CString str = ComposeFloorString(i + 1, m_aFloors.begin() + i);
		m_listFloors.AddString(str);
	}

	iSelFloor = min(iSelFloor, nCount - 1);
	iSelFloor = max(iSelFloor, 0);

	m_listFloors.SetCurSel(iSelFloor);

	m_buttonAdd.EnableWindow(m_listFloors.GetCount() < MAX_NUMBER_OF_FLOORS - 1);
	m_buttonRemove.EnableWindow(m_listFloors.GetCurSel() != LB_ERR);
	m_buttonOK.EnableWindow(IsValid());
}

//=================================== EOF ===================================
