/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			EditEqRealDlg.cpp
//		DESCRIPTION:	implementation of the CEditEqRealDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "EditEqRealDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                           CEditEqRealDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CEditEqRealDlg, CDialog)
	//{{AFX_MSG_MAP(CEditEqRealDlg)
	ON_NOTIFY(DTN_CLOSEUP, IDC_DATETIMEPICKER, OnCloseupDatetimepicker)
	ON_BN_CLICKED(IDC_BUTTON_TIME_HISTORY, OnButtonTimeHistory)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
CEditEqRealDlg::CEditEqRealDlg(long aID, CWnd* pParent) : 
CDialog(CEditEqRealDlg::IDD, pParent),
CEarthquakeReal(aID)
{
	//{{AFX_DATA_INIT(CEditEqRealDlg)
	m_strDescription = GetDescription();
	m_strEqName = GetName();
	m_strStationName = GetStationName();
	m_strStationID = GetStationID();
	m_strAccelID = GetAccelID();
	m_dDamping = GetDamping();
	m_dDepth = GetDepth_KM();
	m_dLatitude = GetLatitude();
	m_dLongitude = GetLongitude();
	m_dMagnitude = GetMagnitude_ML();
	m_dSensitivity = GetSensitivity_CM_G();
	m_dTotalTime = GetTotalTime_SEC();
	m_dTransNaturalPeriod = GetTransNaturalPeriod_SEC();
	m_timeCtrl = GetLocalTime();
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
void CEditEqRealDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEditEqRealDlg)
	DDX_Text(pDX, IDC_EDIT_DESCRIPTION, m_strDescription);
	DDX_Text(pDX, IDC_EDIT_EQ_NAME, m_strEqName);
	DDX_Text(pDX, IDC_EDIT_STATION_NAME, m_strStationName);
	DDX_Text(pDX, IDC_EDIT_STATION_ID, m_strStationID);
	DDX_Text(pDX, IDC_EDIT_ACCEL_ID, m_strAccelID);
	DDX_Text(pDX, IDC_EDIT_DAMPING, m_dDamping);
	DDX_Text(pDX, IDC_EDIT_DEPTH, m_dDepth);
	DDX_Text(pDX, IDC_EDIT_LATITUDE, m_dLatitude);
	DDX_Text(pDX, IDC_EDIT_LONGITUDE, m_dLongitude);
	DDX_Text(pDX, IDC_EDIT_MAGNITUDE, m_dMagnitude);
	DDX_Text(pDX, IDC_EDIT_SENSITIVITY, m_dSensitivity);
	DDX_Text(pDX, IDC_EDIT_TOTAL_TIME, m_dTotalTime);
	DDX_Text(pDX, IDC_EDIT_TRANS_NATURAL_PERIOD, m_dTransNaturalPeriod);
	DDX_DateTimeCtrl(pDX, IDC_DATETIMEPICKER, m_timeCtrl);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
void CEditEqRealDlg::OnCloseupDatetimepicker(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
void CEditEqRealDlg::OnButtonTimeHistory() 
{
	static char BASED_CODE szFilter[] = "Text Files (*.txt)|*.txt|All Files (*.*)|*.*||";

	char szDir[MAX_PATH];
	GetCurrentDirectory(sizeof(szDir), szDir);
	strcat(szDir, "\\*.txt");

	CFileDialog dlg(TRUE, NULL, szDir, OFN_HIDEREADONLY | OFN_FILEMUSTEXIST, szFilter, this);

	if (dlg.DoModal() == IDOK)
	{
		TRY
		{
			CFile file(dlg.GetPathName(), CFile::modeRead | CFile::shareExclusive);
			DWORD dwBytes = file.GetLength();

			if (dwBytes)
			{
				char* pBuffer = new char[dwBytes + 1];
				file.Read(pBuffer, dwBytes);
				pBuffer[dwBytes] = 0;

				vector<char*> lines;
				char* pLine = strtok(pBuffer, "\n");

				while (pLine)
				{
					lines.push_back(pLine);
					pLine = strtok(NULL, "\n");
				}

				vector<double> time;
				vector<double> disp;

				vector<char*>::iterator it = lines.begin();

				while (it != lines.end())
				{
					time.push_back(atof(strtok(*it,  ",")));
					disp.push_back(atof(strtok(NULL, ",")));

					it++;
				}

				m_adAbsDisp = disp;

				if (time.size() > 1)
				{
					m_dTotalTime_SEC = m_dTotalTime = time.back();
					m_dStep_SEC = m_dTotalTime_SEC / double(time.size() - 1);

					UpdateData(FALSE);
				}

				delete pBuffer;
			}
		}
		CATCH( CFileException, e )
		{
		}
		END_CATCH
	}
}

/////////////////////////////////////////////////////////////////////////////
void CEditEqRealDlg::OnOK() 
{
	UpdateData();

	SetDescription(m_strDescription);
	SetName(m_strEqName);
	SetLocalTime(long(m_timeCtrl.GetTime()));
	SetLatitude(m_dLatitude);
	SetLongitude(m_dLongitude);
	SetDepth_KM(m_dDepth);
	SetMagnitude_ML(m_dMagnitude);
	SetTransNaturalPeriod_SEC(m_dTransNaturalPeriod);
	SetDamping(m_dDamping);
	SetSensitivity_CM_G(m_dSensitivity);
	SetTotalTime_SEC(m_dTotalTime);
	SetStationName(m_strStationName);
	SetStationID(m_strStationID);
	SetAccelID(m_strAccelID);

	if (Save())
	{
		CDialog::OnOK();
	}
	else
	{
		MessageBox("Invalid data");
	}
}

//=================================== EOF ===================================
