/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			EditProjectDlg.cpp
//		DESCRIPTION:	implementation of the CEditProjectDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "EditProjectDlg.h"

#include "Earthquake.h"
#include "Building.h"
#include "SecSystem.h"
#include "IsoSystem.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                          CEditProjectDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CEditProjectDlg, CDialog)
	//{{AFX_MSG_MAP(CEditProjectDlg)
	ON_LBN_SELCHANGE(IDC_LIST_AVAIL_ISO_SYSTEMS, OnSelchangeListAvailIsoSystems)
	ON_LBN_DBLCLK(IDC_LIST_AVAIL_ISO_SYSTEMS, OnDblclkListAvailIsoSystems)
	ON_LBN_SELCHANGE(IDC_LIST_USED_ISO_SYSTEMS, OnSelchangeListUsedIsoSystems)
	ON_LBN_DBLCLK(IDC_LIST_USED_ISO_SYSTEMS, OnDblclkListUsedIsoSystems)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnButtonAdd)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE, OnButtonRemove)
	ON_CBN_SELCHANGE(IDC_COMBO_BUILDINGS, OnSelchangeComboBuildings)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CEditProjectDlg::CEditProjectDlg(long aProjectID, bool aOpenProject, CWnd* pParent) : 
CDialog(CEditProjectDlg::IDD, pParent),
CProject(aProjectID),
m_bOpenProject(aOpenProject),
m_nUsedSysAFMS(0),
m_nUsedSysSliding(0),
m_nUsedSysShear(0)
{
	//{{AFX_DATA_INIT(CEditProjectDlg)
	m_strDescription = GetDescription();
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CEditProjectDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Enumerate REAL EQ. records -------------------------------------------

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	IDbTable* pTable = g_Application.GetDatabase()->GetEqRealTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	long nIDs;
	pArray->GetSize(&nIDs);

	for (int i = 0, iCurSel = -1; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		if (GetEqReal() && GetEqReal()->GetID() == lID)
		{
			iCurSel = i;
		}

		// Compose the description
		CEarthquakeReal eq(lID);
		CString str;
		str.Format("REAL EQ #%d %s %s", eq.GetID(), eq.GetDescription(), eq.GetParamDesc());

		// Insert item and remember the record ID
		int nItem = m_cboxEarthquakes.AddString(str);
		m_cboxEarthquakes.SetItemData(nItem, PackItemData(lID, CEarthquake::TYPE_REAL));
	}

	if (nIDs && iCurSel > -1)
	{
		m_cboxEarthquakes.SetCurSel(iCurSel);
	}
	
	// Enumerate CONE EQ. records -------------------------------------------

	pArray->Empty();
	pTable = g_Application.GetDatabase()->GetEqConeTable();
	pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	int iOffset = nIDs;
	pArray->GetSize(&nIDs);

	for (i = 0, iCurSel = -1; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		if (GetEqCone() && GetEqCone()->GetID() == lID)
		{
			iCurSel = i + iOffset;
		}

		// Compose the description
		CEarthquakeCone eq(lID);
		CString str;
		str.Format("CONE EQ #%d %s %s", eq.GetID(), eq.GetDescription(), eq.GetParamDesc());

		// Insert item and remember the record ID
		int nItem = m_cboxEarthquakes.AddString(str);
		m_cboxEarthquakes.SetItemData(nItem, PackItemData(lID, CEarthquake::TYPE_CONE));
	}

	if (m_cboxEarthquakes.GetCount())
	{
		m_cboxEarthquakes.SetCurSel(iCurSel == -1 ? 0 : iCurSel);
	}
	
	// Enumerate BUILDING records -------------------------------------------

	pArray->Empty();
	pTable = g_Application.GetDatabase()->GetBuildingsTable();
	pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	pArray->GetSize(&nIDs);

	for (i = 0, iCurSel = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		if (GetBuilding() && GetBuilding()->GetID() == lID)
		{
			iCurSel = i;
		}

		// Compose the description
		CBuilding bldg(lID);
		CString str;
		str.Format("BLDG #%d %s %s", bldg.GetID(), bldg.GetDescription(), bldg.GetParamDesc());

		// Insert item and remember the record ID
		int nItem = m_cboxBuildings.AddString(str);
		m_cboxBuildings.SetItemData(nItem, DWORD(lID));
	}

	if (nIDs)
	{
		m_cboxBuildings.SetCurSel(iCurSel);

		// Enumerate SEC SYSTEM records
		UpdateSecSysCombo(m_cboxBuildings.GetItemData(iCurSel));
	}
	
	// Enumerate ISO SYSTEM AFMS records ------------------------------------

	pArray->Empty();
	pTable = g_Application.GetDatabase()->GetIsoSysAFMSTable();
	pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	pArray->GetSize(&nIDs);

	for (i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Establish the listbox to put the item to
		CListBox& cbox = HasIsoSysAFMSID(lID) ? m_lboxUsedIsoSystems : m_lboxAvailIsoSystems;

		// Compose the description
		CIsoSystemAFMS sys(lID);
		CString str;
		str.Format("SYS #%d %s %s", sys.GetID(), sys.GetDescription(), sys.GetParamDesc());

		// Insert item and remember the record ID
		int nItem = cbox.AddString(str);
		cbox.SetItemData(nItem, PackItemData(lID, CIsoSystem::TYPE_AFMS));
	}
	
	// Enumerate SEC SYSTEM SLIDING records ---------------------------------

	pArray->Empty();
	pTable = g_Application.GetDatabase()->GetIsoSysSlidingTable();
	pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	pArray->GetSize(&nIDs);

	for (i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Establish the listbox to put the item to
		CListBox& cbox = HasIsoSysSlidingID(lID) ? m_lboxUsedIsoSystems : m_lboxAvailIsoSystems;

		// Compose the description
		CIsoSystemSliding sys(lID);
		CString str;
		str.Format("SYS #%d %s %s", sys.GetID(), sys.GetDescription(), sys.GetParamDesc());

		// Insert item and remember the record ID
		int nItem = cbox.AddString(str);
		cbox.SetItemData(nItem, PackItemData(lID, CIsoSystem::TYPE_SLIDING));
	}
	
	// Enumerate ISO SYSTEM SHEAR records -----------------------------------------

	pArray->Empty();
	pTable = g_Application.GetDatabase()->GetIsoSysShearTable();
	pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	pArray->GetSize(&nIDs);

	for (i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Establish the listbox to put the item to
		CListBox& cbox = HasIsoSysShearID(lID) ? m_lboxUsedIsoSystems : m_lboxAvailIsoSystems;

		// Compose the description
		CIsoSystemShear sys(lID);
		CString str;
		str.Format("SYS #%d %s %s", sys.GetID(), sys.GetDescription(), sys.GetParamDesc());

		// Insert item and remember the record ID
		int nItem = cbox.AddString(str);
		cbox.SetItemData(nItem, PackItemData(lID, CIsoSystem::TYPE_SHEAR_HYS));
	}

	if (m_lboxAvailIsoSystems.GetCount())
	{
		m_lboxAvailIsoSystems.SetCurSel(0);
	}

	if (m_lboxUsedIsoSystems.GetCount())
	{
		m_lboxUsedIsoSystems.SetCurSel(0);
	}

	m_buttonOk.SetWindowText(m_bOpenProject ? "Open" : "OK");
	UpdateCtrlState();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

//********************************** DDX ************************************
/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEditProjectDlg)
	DDX_Control(pDX, IDOK, m_buttonOk);
	DDX_Text(pDX, IDC_EDIT_DESCRIPTION, m_strDescription);
	DDX_Control(pDX, IDC_COMBO_EARTHQUAKES, m_cboxEarthquakes);
	DDX_Control(pDX, IDC_COMBO_BUILDINGS, m_cboxBuildings);
	DDX_Control(pDX, IDC_COMBO_SEC_SYSTEMS, m_cboxSecSystems);
	DDX_Control(pDX, IDC_LIST_AVAIL_ISO_SYSTEMS, m_lboxAvailIsoSystems);
	DDX_Control(pDX, IDC_LIST_USED_ISO_SYSTEMS, m_lboxUsedIsoSystems);
	DDX_Control(pDX, IDC_BUTTON_ADD, m_buttonAdd);
	DDX_Control(pDX, IDC_BUTTON_REMOVE, m_buttonRemove);
	//}}AFX_DATA_MAP
}

//******************************** Controls *********************************
/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::OnSelchangeComboBuildings() 
{
	UpdateSecSysCombo(m_cboxBuildings.GetItemData(m_cboxBuildings.GetCurSel()));
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::OnSelchangeListAvailIsoSystems() 
{
	UpdateCtrlState();
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::OnDblclkListAvailIsoSystems() 
{
	OnButtonAdd();
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::OnSelchangeListUsedIsoSystems() 
{
	UpdateCtrlState();
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::OnDblclkListUsedIsoSystems() 
{
	OnButtonRemove();
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::OnButtonAdd() 
{
	int nItem = m_lboxAvailIsoSystems.GetCurSel();

	if (nItem > -1)
	{
		CString str;
		m_lboxAvailIsoSystems.GetText(nItem, str);
		DWORD dwData = m_lboxAvailIsoSystems.GetItemData(nItem);

		long lID;
		char cType;
		UnpackItemData(dwData, lID, cType);

		switch (cType)
		{
		case CIsoSystem::TYPE_AFMS:

			if (m_nUsedSysAFMS == 4)
			{
				MessageBox("Maximum number of AFMS systems is 4");
				return;
			}
			
			m_nUsedSysAFMS++;
			break;

		case CIsoSystem::TYPE_SLIDING:

			if (m_nUsedSysSliding == 4)
			{
				MessageBox("Maximum number of Earthquake Protector systems is 4");
				return;
			}
			
			m_nUsedSysSliding++;
			break;

		case CIsoSystem::TYPE_SHEAR_HYS:

			if (m_nUsedSysShear == 4)
			{
				MessageBox("Maximum number of shear systems is 4");
				return;
			}
			
			m_nUsedSysShear++;
			break;

		default:
			break;
		}

		m_lboxAvailIsoSystems.DeleteString(nItem);
		m_lboxAvailIsoSystems.SetCurSel(nItem == m_lboxAvailIsoSystems.GetCount() ? nItem - 1 : nItem);

		nItem = m_lboxUsedIsoSystems.AddString(str);
		m_lboxUsedIsoSystems.SetItemData(nItem, dwData);
		m_lboxUsedIsoSystems.SetCurSel(nItem);

		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::OnButtonRemove() 
{
	int nItem = m_lboxUsedIsoSystems.GetCurSel();

	if (nItem > -1)
	{
		CString str;
		m_lboxUsedIsoSystems.GetText(nItem, str);
		DWORD dwData = m_lboxUsedIsoSystems.GetItemData(nItem);

		long lID;
		char cType;
		UnpackItemData(dwData, lID, cType);

		switch (cType)
		{
		case CIsoSystem::TYPE_AFMS: m_nUsedSysAFMS--; break;
		case CIsoSystem::TYPE_SLIDING: m_nUsedSysSliding--; break;
		case CIsoSystem::TYPE_SHEAR_HYS: m_nUsedSysShear--; break;
		default: ASSERT(FALSE);
		}

		m_lboxUsedIsoSystems.DeleteString(nItem);
		m_lboxUsedIsoSystems.SetCurSel(nItem == m_lboxUsedIsoSystems.GetCount() ? nItem - 1 : nItem);

		nItem = m_lboxAvailIsoSystems.AddString(str);
		m_lboxAvailIsoSystems.SetItemData(nItem, dwData);
		m_lboxAvailIsoSystems.SetCurSel(nItem);

		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::OnOK() 
{
	UpdateData();

	SetDescription(m_strDescription);
	SetBuildingID(m_cboxBuildings.GetCurSel() > -1 ? m_cboxBuildings.GetItemData(m_cboxBuildings.GetCurSel()) : 0);
	SetSecSystemID(m_cboxSecSystems.GetCurSel() > -1 ? m_cboxSecSystems.GetItemData(m_cboxSecSystems.GetCurSel()) : 0);

	// Set EQ ID ------------------------------------------------------------

	int iCurSel = m_cboxEarthquakes.GetCurSel();

	if (iCurSel > -1)
	{
		long lID;
		char cType;
		UnpackItemData(m_cboxEarthquakes.GetItemData(iCurSel), lID, cType);

		switch (cType)
		{
		case CEarthquake::TYPE_REAL: 
			SetEqRealID(lID);
			SetEqConeID(0);
			break;

		case CEarthquake::TYPE_CONE: 
			SetEqRealID(0);
			SetEqConeID(lID);
			break;

		default: 
			ASSERT(FALSE); 
			break;
		}
	}
	else
	{
		SetEqRealID(0);
		SetEqConeID(0);
	}

	// Set ISO SYS ----------------------------------------------------------

	ClearIsoSystems();

	int nUsedSysAFMS = 0;
	int nUsedSysSliding = 0;
	int nUsedSysShear = 0;

	for (int i = 0, nItems = m_lboxUsedIsoSystems.GetCount(); i < nItems; i++)
	{
		long lID;
		char cType;
		UnpackItemData(m_lboxUsedIsoSystems.GetItemData(i), lID, cType);

		switch (cType)
		{
		case CIsoSystem::TYPE_AFMS:
			SetIsoSysAFMSID(lID);
			nUsedSysAFMS++;
			break;

		case CIsoSystem::TYPE_SLIDING:
			SetIsoSysSlidingID(lID);
			nUsedSysSliding++;
			break;

		case CIsoSystem::TYPE_SHEAR_HYS:
			SetIsoSysShearID(lID);
			nUsedSysShear++;
			break;

		default: 
			ASSERT(FALSE); 
			break;
		}
	}

	// Save -----------------------------------------------------------------

	if (Save())
	{
		g_Application.GetMainWnd()->PostMessage(WM_UPDATE_MAINFRAME_UI);
		CDialog::OnOK();
	}
	else
	{
		MessageBox("Invalid data");
	}
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::UpdateSecSysCombo(long aBuildingID)
{
	ASSERT(aBuildingID);

	m_cboxSecSystems.ResetContent();
	m_cboxSecSystems.AddString("NONE");

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	IDbTable* pTable = g_Application.GetDatabase()->GetSecSystemsTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	long nIDs;
	pArray->GetSize(&nIDs);

	for (int i = 0, iCurSel = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		CSecSystem sys(lID);

		if (sys.GetBuildingID() != aBuildingID)
		{
			continue;
		}

		if (GetSecSystem() && GetSecSystem()->GetID() == lID)
		{
			iCurSel = i;
		}

		// Compose the description
		CString str;
		str.Format("SEC SYS #%d %s %s", sys.GetID(), sys.GetDescription(), sys.GetParamDesc());

		// Insert item and remember the record ID
		int nItem = m_cboxSecSystems.AddString(str);
		m_cboxSecSystems.SetItemData(nItem, DWORD(lID));
	}

	if (nIDs)
	{
		m_cboxSecSystems.SetCurSel(iCurSel);
	}
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::UpdateCtrlState()
{
	m_buttonAdd.EnableWindow(m_lboxAvailIsoSystems.GetCurSel() > -1);
	m_buttonRemove.EnableWindow(m_lboxUsedIsoSystems.GetCurSel() > -1);
	m_buttonOk.EnableWindow(m_cboxBuildings.GetCurSel() > -1 && m_cboxEarthquakes.GetCurSel() > -1);
}

/////////////////////////////////////////////////////////////////////////////
DWORD CEditProjectDlg::PackItemData(long aID, char aFlag)
{
	DWORD dwItemData = (aID << 8);
	return (dwItemData | DWORD(aFlag));
}

/////////////////////////////////////////////////////////////////////////////
void CEditProjectDlg::UnpackItemData(DWORD aItemData, long& aID, char& aFlag)
{
	aFlag = (aItemData & 0xFF);
	aID = (aItemData >> 8);
}

//=================================== EOF ===================================
