/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			EditSecondarySystemDlg.cpp
//		DESCRIPTION:	implementation of the CEditSecondarySystemDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "EditSecondarySystemDlg.h"
#include "Building.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                     CEditSecondarySystemDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CEditSecondarySystemDlg, CDialog)
	//{{AFX_MSG_MAP(CEditSecondarySystemDlg)
	ON_CBN_SELCHANGE(IDC_COMBO_BUILDING, OnSelchangeComboBuilding)
	ON_CBN_SELCHANGE(IDC_COMBO_FLOOR, OnSelchangeComboFloor)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CEditSecondarySystemDlg::CEditSecondarySystemDlg(long aSecSystemID, CWnd* pParent) : 
CDialog(CEditSecondarySystemDlg::IDD, pParent),
CSecSystem(aSecSystemID)
{
	//{{AFX_DATA_INIT(CEditSecondarySystemDlg)
	m_strDescription = GetDescription();
	m_dMass = GetMass_KG();
	m_dDampingRatio = GetDampingRatio();
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CEditSecondarySystemDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Enumerate building records -------------------------------------------

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	IDbTable* pTable = g_Application.GetDatabase()->GetBuildingsTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	// Populate combo -------------------------------------------------------

	long iCurSel = 0;

	long nIDs;
	pArray->GetSize(&nIDs);

	for (int i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		CBuilding bldg(lID);
		CString str;
		str.Format("BLDG #%u", lID);

		if (strlen(bldg.GetDescription()))
		{
			str += " (";
			str += bldg.GetDescription();
			str += ")";
		}

		int iIndex = m_cboxBuildings.AddString(str);
		m_cboxBuildings.SetItemData(iIndex, DWORD(lID));

		if (lID == GetBuildingID())
		{
			// Remember to select the index
			iCurSel = iIndex;
		}
	}

	if (nIDs)
	{
		// Select the building
		m_cboxBuildings.SetCurSel(iCurSel);
		UpdateFloorCombo(m_cboxBuildings.GetItemData(iCurSel));
	}
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
void CEditSecondarySystemDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEditSecondarySystemDlg)
	DDX_Control(pDX, IDC_COMBO_FLOOR, m_cboxFloors);
	DDX_Control(pDX, IDC_COMBO_BUILDING, m_cboxBuildings);
	DDX_Text(pDX, IDC_EDIT_DESCRIPTION, m_strDescription);
	DDX_Text(pDX, IDC_EDIT_MASS, m_dMass);
	DDX_Text(pDX, IDC_EDIT_DAMPING_RATIO, m_dDampingRatio);
	//}}AFX_DATA_MAP
}

//******************************** Controls *********************************
/////////////////////////////////////////////////////////////////////////////
void CEditSecondarySystemDlg::OnSelchangeComboBuilding() 
{
	long iIndex = m_cboxBuildings.GetCurSel();

	if (iIndex > -1)
	{
		long lBuildingID = m_cboxBuildings.GetItemData(iIndex);
		UpdateFloorCombo(lBuildingID);
	}
}

/////////////////////////////////////////////////////////////////////////////
void CEditSecondarySystemDlg::OnSelchangeComboFloor() 
{
}

/////////////////////////////////////////////////////////////////////////////
void CEditSecondarySystemDlg::OnOK() 
{
	UpdateData();

	SetDescription(m_strDescription);
	SetBuildingID(m_cboxBuildings.GetItemData(m_cboxBuildings.GetCurSel()));
	SetFloorNumber(m_cboxFloors.GetCurSel());
	SetMass_KG(m_dMass);
	SetDampingRatio(m_dDampingRatio);

	if (Save())
	{
		CDialog::OnOK();
	}
	else
	{
		MessageBox("Invalid data");
	}
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
void CEditSecondarySystemDlg::UpdateFloorCombo(long aBuildingID)
{
	m_cboxFloors.ResetContent();
	CBuilding bldg(aBuildingID);
	int nFloors = bldg.GetNumberOfFloors();

	for (int i = 0; i < nFloors; i++)
	{
		CString str;
		str.Format("%u", (UINT)i);
		m_cboxFloors.AddString(str);
	}

	if (nFloors)
	{
		long lCurSel = min(GetFloorNumber(), nFloors - 1);
		m_cboxFloors.SetCurSel(lCurSel);
	}
}

//=================================== EOF ===================================
