/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			EqTimeHistoryView.cpp
//		DESCRIPTION:	implementation of the CEqTimeHistoryView class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "EqTimeHistoryView.h"
#include "Project.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define GRAPH_LINE_COLOR RGB(0,0,0)
#define GRAPH_LINE_WIDTH 2
#define FONT_HEIGHT 20
#define MARGIN 10


//***************************************************************************
//                         CEqTimeHistoryView CLASS
//***************************************************************************

CFont CEqTimeHistoryView::m_Font;

BEGIN_MESSAGE_MAP(CEqTimeHistoryView, CWnd)
	//{{AFX_MSG_MAP(CEqTimeHistoryView)
	ON_WM_PAINT()
	ON_COMMAND(ID_PROJECT_PRINT, OnPrint)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_PRINT, OnUpdatePrint)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//**************************** Setup/cleanup ********************************
/////////////////////////////////////////////////////////////////////////////
CEqTimeHistoryView::CEqTimeHistoryView() :
m_Graph(RGB(230, 230, 200))
{
	if (!HFONT(m_Font))
	{
		m_Font.CreateFont( 
			FONT_HEIGHT, 
			0, 
			0, 
			0, 
			FW_BOLD, 
			0, 
			0, 
			0, 
			DEFAULT_CHARSET, 
			OUT_DEFAULT_PRECIS, 
			CLIP_DEFAULT_PRECIS, 
			DEFAULT_QUALITY, 
			DEFAULT_PITCH, 
			"Times New Roman");
	}

	CProject& project = g_Application.GetProject();
	m_pEarthquake = (CEarthquake*)project.GetEq();
	ASSERT(m_pEarthquake);

	m_pEarthquake->Calculate();

	double dStep = m_pEarthquake->GetStep_SEC();
	double dTotalTime = m_pEarthquake->GetTotalTime_SEC();

	vector<double> times;
	vector<double> disp;

	for (double dTime = 0.0; dTime <= dTotalTime; dTime += dStep)
	{
		times.push_back(dTime);
		disp.push_back(m_pEarthquake->GetAbsDisp_CM(dTime));
	}

	m_Graph.AddGraph(times, disp, GRAPH_LINE_COLOR, GRAPH_LINE_WIDTH);
	m_Graph.ShowTickmark(true);
}

/////////////////////////////////////////////////////////////////////////////
BOOL CEqTimeHistoryView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
	{
		return FALSE;
	}

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(
		CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), 
		HBRUSH(COLOR_WINDOW + 1), 
		NULL);

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
CEqTimeHistoryView::~CEqTimeHistoryView()
{
}

//******************************* Commands **********************************
/////////////////////////////////////////////////////////////////////////////
void CEqTimeHistoryView::OnPrint() 
{
	CPrintDialog dlg(false);

	if (dlg.DoModal() == IDOK)
	{
		// Create printer DC, start printing document

		CDC dc;
		dc.Attach(dlg.GetPrinterDC());

		DOCINFO di;
		di.cbSize = sizeof(di);
		di.lpszDocName = "Time History View";
		di.lpszOutput = NULL;
		di.lpszDatatype = NULL;
		di.fwType = 0;

		dc.StartDoc(&di);
		dc.StartPage();

		// Create window-compatible DC and draw on it

		CRect rect;
		GetClientRect(rect);
		CPaintDC winDc(this); 

		CDC memDc; 
		memDc.CreateCompatibleDC(&winDc);

		CBitmap memBitmap;
		memBitmap.CreateCompatibleBitmap(&winDc, rect.Width(), rect.Height());
		CBitmap* pOldBitmap = memDc.SelectObject(&memBitmap);
		memDc.PatBlt(0, 0, rect.Width(), rect.Height(), WHITENESS);

		OnDraw(&memDc);

		// Stretch bitmap onto printer DC

		int iMargins = int(0.1 * double(dc.GetDeviceCaps(HORZRES)));
		int iTargetPixelsX = dc.GetDeviceCaps(HORZRES) - iMargins;
		float fScale = float(iTargetPixelsX) / float(rect.Width());
		int iTargetPixelsY = int(float(rect.Height()) * fScale);

		dc.StretchBlt(
			iMargins / 2, 
			iMargins / 2, 
			iTargetPixelsX, 
			iTargetPixelsY, 
			&memDc, 
			0, 
			0, 
			rect.Width(), 
			rect.Height(), 
			SRCCOPY);

		// End printing

		dc.EndPage();
		dc.EndDoc();

		// Cleanup

		memDc.SelectObject(pOldBitmap);
		memDc.DeleteDC();
		memBitmap.DeleteObject();

		dc.DeleteDC();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CEqTimeHistoryView::OnUpdatePrint(CCmdUI* pCmdUI) 
{
}

//******************************* Messages **********************************
/////////////////////////////////////////////////////////////////////////////
void CEqTimeHistoryView::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	OnDraw(&dc);
}

/////////////////////////////////////////////////////////////////////////////
void CEqTimeHistoryView::OnDraw(CDC* aDC)
{
	CRect rect;
	GetClientRect(rect);

	// Display earthquake info ----------------------------------------------

	CFont* pOldFont = aDC->SelectObject(&m_Font);
	aDC->SetBkMode(TRANSPARENT);

	rect.top += MARGIN;
	rect.bottom -= (FONT_HEIGHT + MARGIN);
	rect.left += MARGIN;
	rect.right -= MARGIN;

	aDC->TextOut(rect.left, rect.bottom + MARGIN / 2, m_pEarthquake->GetParamDesc());

	aDC->SelectObject(pOldFont);

	// Display graphics -----------------------------------------------------

	m_Graph.OnDraw(aDC, rect);
}

//=================================== EOF ===================================
