/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  SERGEY SHUSTOV 2001. All Rights Reserved.
//
//		FILE:			GLPrims.h
//		DESCRIPTION:	wrapper classes for GL structures
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _GLPRIMS_H_
#define _GLPRIMS_H_

#include "Graphics.h"
#include "math.h"

#ifndef PI
#define PI 3.1415926536
#endif


//***************************************************************************
//                             GlPoint CLASS
//***************************************************************************

class GlPoint
{
public:

	GLPOINT m;

public:

	GlPoint( double aX = 0.0, double aY = 0.0, double aZ = 0.0 ) 
	{ 
		m.x = aX; 
		m.y = aY; 
		m.z = aZ; 
	}

	inline operator GLPOINT* () 
	{ 
		return &m; 
	}

	inline void SetMagnitude( double aMagnitude )
	{
		double dMag = sqrt( pow( m.x, 2 ) + pow( m.y, 2 ) + pow( m.z, 2 ) );

		if ( aMagnitude )
		{
			dMag = aMagnitude / dMag;

			m.x *= dMag;
			m.y *= dMag;
			m.z *= dMag;
		}
	}
};


//***************************************************************************
//                             GlPlane CLASS
//***************************************************************************

class GlPlane
{
public:

	GlPoint m_A;
	GlPoint m_B;
	GlPoint m_C;

public:

	GlPlane()
	{
	}

	GlPlane( const GlPoint& aA, const GlPoint& aB, const GlPoint& aC ) :
	m_A( aA ),
	m_B( aB ),
	m_C( aC )
	{
	}

	GlPoint GetNormal() const
	{
		GlPoint p1( m_A.m.x - m_B.m.x, m_A.m.y - m_B.m.y, m_A.m.z - m_B.m.z );
		GlPoint p2( m_B.m.x - m_C.m.x, m_B.m.y - m_C.m.y, m_B.m.z - m_C.m.z );

		GlPoint p3( p1.m.y * p2.m.z - p1.m.z * p2.m.y,
			        p1.m.z * p2.m.x - p1.m.x * p2.m.z,
					p1.m.x * p2.m.y - p1.m.y * p2.m.x );

		double dMagnitude = sqrt( p3.m.x * p3.m.x + p3.m.y * p3.m.y + p3.m.z * p3.m.z );

		p3.m.x /= dMagnitude;
		p3.m.y /= dMagnitude;
		p3.m.z /= dMagnitude;
		
		return p3;
	}
};


//***************************************************************************
//                             GlColor CLASS
//***************************************************************************

class GlColor
{
	GLCOLOR4F m;

public:

	GlColor( float aRed, float aGreen, float aBlue, float aAlpha = 1.0f )
	{
		m.m_fRed   = aRed;
		m.m_fGreen = aGreen;
		m.m_fBlue  = aBlue;
		m.m_fAlpha = aAlpha;
	}

	inline operator GLCOLOR4F* () 
	{ 
		return &m; 
	}
};


//***************************************************************************
//                            GlColor4b CLASS
//***************************************************************************

class GlColor4b
{
	GLCOLOR4B m;

public:

	GlColor4b( BYTE aRed, BYTE aGreen, BYTE aBlue, BYTE aAlpha = 255 )
	{
		m.m_bRed   = aRed;
		m.m_bGreen = aGreen;
		m.m_bBlue  = aBlue;
		m.m_bAlpha = aAlpha;
	}

	inline operator GLCOLOR4B* () 
	{ 
		return &m; 
	}

	inline operator GLCOLOR4B () 
	{ 
		return m; 
	}
};


#endif // _GLPRIMS_H_

//=================================== EOF ===================================

