/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			GlViewerGauge.cpp
//		DESCRIPTION:	implementation of the GlViewerGauge class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "GLViewerGauge.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//***************************************************************************
//                            GlViewerGauge CLASS
//***************************************************************************

BSTR GlViewerGauge::m_strMode(NULL);
BSTR GlViewerGauge::m_strTotalTime_SEC(NULL);
BSTR GlViewerGauge::m_strTime_SEC(NULL);

/////////////////////////////////////////////////////////////////////////////
GlViewerGauge::GlViewerGauge() :
m_pFont(NULL),
m_bShow(false),
m_strTitle(NULL)
{
}

/////////////////////////////////////////////////////////////////////////////
GlViewerGauge::~GlViewerGauge()
{
	CScopeLock sl(&m_Lock);

	if (m_strTitle)
	{
		SysFreeString(m_strTitle);
		m_strTitle = NULL;
	}

	/*
	if (m_strMode)
	{
		SysFreeString(m_strMode);
		m_strMode = NULL;
	}

	if (m_strTotalTime_SEC)
	{
		SysFreeString(m_strTotalTime_SEC);
		m_strTotalTime_SEC = NULL;
	}

	if (m_strTime_SEC)
	{
		SysFreeString(m_strTime_SEC);
		m_strTime_SEC = NULL;
	}
	*/
}

/////////////////////////////////////////////////////////////////////////////
void GlViewerGauge::SetTitle(const char* aValue)
{
	CScopeLock sl(&m_Lock);

	if (m_strTitle)
	{
		SysFreeString(m_strTitle);
		m_strTitle = NULL;
	}

	if (aValue)
	{
		m_strTitle = A2BSTR(aValue);
	}
}

/////////////////////////////////////////////////////////////////////////////
void GlViewerGauge::SetMode(const char* aValue)
{
	CScopeLock sl(&m_Lock);

	if (m_strMode)
	{
		SysFreeString(m_strMode);
		m_strMode = NULL;
	}

	if (aValue)
	{
		m_strMode = A2BSTR(aValue);
	}
}

/////////////////////////////////////////////////////////////////////////////
void GlViewerGauge::SetTotalTime_SEC(const char* aValue)
{
	CScopeLock sl(&m_Lock);

	if (m_strTotalTime_SEC)
	{
		SysFreeString(m_strTotalTime_SEC);
		m_strTotalTime_SEC = NULL;
	}

	if (aValue)
	{
		m_strTotalTime_SEC = A2BSTR(aValue);
	}
}

/////////////////////////////////////////////////////////////////////////////
void GlViewerGauge::SetTime_SEC(const char* aValue)
{
	CScopeLock sl(&m_Lock);

	if (m_strTime_SEC)
	{
		SysFreeString(m_strTime_SEC);
		m_strTime_SEC = NULL;
	}

	if (aValue)
	{
		m_strTime_SEC = A2BSTR(aValue);
	}
}

/////////////////////////////////////////////////////////////////////////////
STDMETHODIMP GlViewerGauge::Show(boolean aShow)
{
	m_bShow = (aShow != FALSE);
	return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
STDMETHODIMP GlViewerGauge::IsVisible(boolean* aVisible)
{
	*aVisible = m_bShow == true;
	return S_OK;
}

//******************************** Draw *************************************
/////////////////////////////////////////////////////////////////////////////
STDMETHODIMP GlViewerGauge::Draw(IGlCanvas* aCanvas,
								 IGlContext* aContext, 
								 IGlViewer*, 
								 IUnknown*)
{
	if (m_pFont)
	{
		USES_CONVERSION;

		CScopeLock sl(&m_Lock);

		aCanvas->SetVertexColor(GlColor(0.7f, 0.3f, 0.0f));

		GlPoint refPnt;
		aContext->DeviceToSCS(20, 20, &refPnt.m.x, &refPnt.m.y);

		if (m_strTitle)
		{
			m_pFont->Print(m_strTitle, refPnt);
			refPnt.m.y -= 0.12;
		}

		if (m_strMode)
		{
			m_pFont->Print(m_strMode, refPnt);
			refPnt.m.y -= 0.12;
		}

		if (m_strTotalTime_SEC)
		{
			m_pFont->Print(m_strTotalTime_SEC, refPnt);
			refPnt.m.y -= 0.12;
		}

		if (m_strTime_SEC)
		{
			m_pFont->Print(m_strTime_SEC, refPnt);
			refPnt.m.y -= 0.12;
		}
	}

	return S_OK;
}

//=================================== EOF ===================================
