/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			GLIsoSystem.cpp
//		DESCRIPTION:	implementation of the GLIsoSystem class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "GLIsoSystem.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//***************************************************************************
//                             GlIsoSystem CLASS
//***************************************************************************

long GlIsoSystem::m_iList = 0;
CLock GlIsoSystem::m_Lock;

//**************************** Setup/cleanup ********************************
/////////////////////////////////////////////////////////////////////////////
GlIsoSystem::GlIsoSystem(IVolSpace* aSpace, const GlColor& aColor, double aWidth) :
m_pCoord    (NULL),
m_pVolObject(NULL),
m_Color     (aColor),
m_dWidth    (aWidth)
{
	// Create 3D object

	HRESULT hr = CoCreateInstance(
		CLSID_VolObject, 
		NULL, 
		CLSCTX_INPROC_SERVER, 
		IID_IVolObject, 
		(void**)&m_pVolObject);

	if (FAILED(hr))
	{
		ASSERT(FALSE);
		return;
	}

	// Create coordinate system

	hr = CoCreateInstance(
		CLSID_GlCoord, 
		NULL, 
		CLSCTX_INPROC_SERVER, 
		IID_IGlCoord, 
		(void**)&m_pCoord);

	if (FAILED(hr))
	{
		ASSERT(FALSE);
		return;
	}

	m_pCoord->SetOrigin3d(0.0, -3.0, 0.0);
	m_pVolObject->AttachCoord(m_pCoord);

	m_pVolObject->AttachPainter(this);

	CComQIPtr<IStdNode>(aSpace)->AddChild(CComQIPtr<IStdNode>(m_pVolObject));
	m_pVolObject->Release();
}

/////////////////////////////////////////////////////////////////////////////
GlIsoSystem::~GlIsoSystem()
{
	CScopeLock sl(&m_Lock);

	m_pCoord->Release();
}

//******************************** Draw *************************************
/////////////////////////////////////////////////////////////////////////////
STDMETHODIMP GlIsoSystem::Draw(IGlCanvas* aCanvas,
						       IGlContext* aContext, 
						       IGlViewer* aViewer,
							   IUnknown*)
{
	boolean bValidList;
	aCanvas->IsCompiledList(m_iList, &bValidList);

	if (!bValidList)
	{
		aCanvas->BeginCompileList(&m_iList);

		//    +Y
		//     |
		//   F--------G
		//  /|       /|
		// B-|- - - C |
		// | |      | |
		// | |        |-- +X
		// | E--------H
		// |/       |/
		// A--------D

		GlPoint A(-0.5, +0.5, -0.5);
		GlPoint B(+0.5, +0.5, -0.5);
		GlPoint C(+0.5, -0.5, -0.5);
		GlPoint D(-0.5, -0.5, -0.5);
		GlPoint E(-0.5, +0.5, +0.5);
		GlPoint F(+0.5, +0.5, +0.5);
		GlPoint G(+0.5, -0.5, +0.5);
		GlPoint H(-0.5, -0.5, +0.5);

		// Side #1
		aCanvas->BeginDrawPolygon();
		{
			aCanvas->SetVertexNormal(GlPoint(-1.0, 0.0, 0.0));

			aCanvas->Vertex(A);
			aCanvas->Vertex(D);
			aCanvas->Vertex(H);
			aCanvas->Vertex(E);
		}
		aCanvas->EndDrawVertices();

		// Side #2
		aCanvas->BeginDrawPolygon();
		{
			aCanvas->SetVertexNormal(GlPoint(0.0, +1.0, 0.0));

			aCanvas->Vertex(B);
			aCanvas->Vertex(A);
			aCanvas->Vertex(E);
			aCanvas->Vertex(F);
		}
		aCanvas->EndDrawVertices();

		// Side #3
		aCanvas->BeginDrawPolygon();
		{
			aCanvas->SetVertexNormal(GlPoint(+1.0, 0.0, 0.0));

			aCanvas->Vertex(C);
			aCanvas->Vertex(B);
			aCanvas->Vertex(F);
			aCanvas->Vertex(G);
		}
		aCanvas->EndDrawVertices();

		// Side #4
		aCanvas->BeginDrawPolygon();
		{
			aCanvas->SetVertexNormal(GlPoint(0.0, -1.0, 0.0));

			aCanvas->Vertex(D);
			aCanvas->Vertex(C);
			aCanvas->Vertex(G);
			aCanvas->Vertex(H);
		}
		aCanvas->EndDrawVertices();

		// Top
		aCanvas->BeginDrawPolygon();
		{
			aCanvas->SetVertexNormal(GlPoint(0.0, 0.0, +1.0));

			aCanvas->Vertex(E);
			aCanvas->Vertex(H);
			aCanvas->Vertex(G);
			aCanvas->Vertex(F);
		}
		aCanvas->EndDrawVertices();

		// Base
		aCanvas->BeginDrawPolygon();
		{
			aCanvas->SetVertexNormal(GlPoint(0.0, 0.0, -1.0));

			aCanvas->Vertex(A);
			aCanvas->Vertex(B);
			aCanvas->Vertex(C);
			aCanvas->Vertex(D);
		}
		aCanvas->EndDrawVertices();

		aCanvas->EndCompileList();
	}

	CScopeLock sl(&m_Lock);

	aCanvas->PushModelviewMatrix(FALSE);

	aCanvas->SetVertexColor(m_Color);
	aCanvas->Scale(GlPoint(m_dWidth, 1.0, 1.0));
	aCanvas->DrawCompiledList(m_iList);

	aCanvas->PopModelviewMatrix();

	return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
void GlIsoSystem::SetAbsDisp(double aDisp)
{
	CScopeLock sl(&m_Lock);
	m_pCoord->SetOrigin3d(aDisp, -3.0, 0.0);
}

//=================================== EOF ===================================
