/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			Graph.cpp
//		DESCRIPTION:	implementation of the CGraph class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "Graph.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define FONT_HEIGHT 10


//***************************************************************************
//                               CGraph CLASS
//***************************************************************************

CFont CGraph::m_Font;

/////////////////////////////////////////////////////////////////////////////
CGraph::CGraph(COLORREF aRgbBkgd) :
m_rgbBkgd(aRgbBkgd),
m_dMinX(0.0), 
m_dMaxX(0.0), 
m_dMinY(0.0), 
m_dMaxY(0.0),
m_bShowAxis(true),
m_bShowFrame(false),
m_bShowTickmark(false)
{
	if (!HFONT(m_Font))
	{
		m_Font.CreateFont( 
			FONT_HEIGHT, 
			0, 
			0, 
			0, 
			FW_NORMAL, 
			0, 
			0, 
			0, 
			DEFAULT_CHARSET, 
			OUT_DEFAULT_PRECIS, 
			CLIP_DEFAULT_PRECIS, 
			DEFAULT_QUALITY, 
			DEFAULT_PITCH, 
			"Courier");
	}
}

/////////////////////////////////////////////////////////////////////////////
CGraph::~CGraph()
{
}

/////////////////////////////////////////////////////////////////////////////
void CGraph::AddGraph(const vector<double>& aX, 
					  const vector<double>& aY, 
					  COLORREF aRgbPen,
					  int aPenWidth)
{
	for (int i = 0, nSize = min(aX.size(), aY.size()); i < nSize; i++)
	{
		if (i == 0 && m_Graphs.empty())
		{
			m_dMinX = m_dMaxX = aX[i];
		}
		else if (m_dMinX > aX[i])
		{
			m_dMinX = aX[i];
		}
		else if (m_dMaxX < aX[i])
		{
			m_dMaxX = aX[i];
		}

		if (i == 0 && m_Graphs.empty())
		{
			m_dMinY = m_dMaxY = aY[i];
		}
		else if (m_dMinY > aY[i])
		{
			m_dMinY = aY[i];
		}
		else if (m_dMaxY < aY[i])
		{
			m_dMaxY = aY[i];
		}
	}

	m_Graphs.push_back(GRAPH_DATA());
	m_Graphs.back().m_aX = aX;
	m_Graphs.back().m_aY = aY;
	m_Graphs.back().m_rgbPen = aRgbPen;
	m_Graphs.back().m_iPenWidth = aPenWidth;
}

/////////////////////////////////////////////////////////////////////////////
void CGraph::OnDraw(CDC* aDC, const RECT* aRect)
{
	ASSERT(aDC && aRect);

	CRgn rgn;
	rgn.CreateRectRgn(aRect->left, aRect->top, aRect->right, aRect->bottom);
	aDC->SelectClipRgn(&rgn, RGN_AND);

	CFont* pOldFont = aDC->SelectObject(&m_Font);

	aDC->SetBkMode(TRANSPARENT);

	// Draw bkgd and frame --------------------------------------------------

	CPen pen(PS_SOLID, 1, m_bShowFrame ? RGB(0, 0, 0) : m_rgbBkgd);
	CPen* pOldPen = aDC->SelectObject(&pen);

	CBrush brush(m_rgbBkgd);
	CBrush* pOldBrush = aDC->SelectObject(&brush);

	aDC->Rectangle(aRect->left, aRect->top, aRect->right, aRect->bottom);

	aDC->SelectObject(pOldPen);
	aDC->SelectObject(pOldBrush);

	// Calculate scale ------------------------------------------------------

	CRect rect(aRect);
	rect.DeflateRect(5, 5);
	rect.left += 60;

	double dWidth  = double(rect.right - rect.left);
	double dHeight = double(rect.bottom - rect.top);

	double dMinX = m_dMinX;
	double dMaxX = m_dMaxX;
	double dMinY = m_dMinY;
	double dMaxY = m_dMaxY;

	if (dMinX == dMaxX)
	{
		dMinX = -0.5;
		dMaxX = +0.5;
	}

	if (dMinY == dMaxY)
	{
		dMinY = -0.5;
		dMaxY = +0.5;
	}

	double dScaleX = dWidth  / (dMaxX - dMinX);
	double dScaleY = dHeight / (dMaxY - dMinY);

	if (m_bShowTickmark)
	{
		CPen pen(PS_DOT, 1, RGB(127, 127, 127));
		CPen* pOldPen = aDC->SelectObject(&pen);

		// VERTICAL AXIS

		int nSteps = rect.Width() / 120;
		nSteps = max(nSteps, 1);

		double dStep = (dMaxX - dMinX) / double(nSteps);
		double dLog = -log10(dStep) + 1;
		double dCoeff = pow(10.0, dLog);
		dStep = floor((dCoeff + 1) * dStep / dCoeff);
		
		if (dStep)
		{
			CString str;
			int iOldTextAlign = aDC->SetTextAlign(TA_CENTER);

			double dStart(dMinX);

			if (m_bShowAxis && dMinX <= 0.0 && dMaxX > 0.0)
			{
				dStart = dStep * floor(dMinX / dStep + 1.0);
			}

			for (double x = dStart; x <= dMaxX; x += dStep)
			{
				int i = rect.left + dScaleX * (x - dMinX);

				if (i > rect.left + 30)
				{
					aDC->MoveTo(i, rect.top);
					aDC->LineTo(i, rect.bottom);

					str.Format("%.2f", x);
					aDC->TextOut(i, rect.top + 2, str);
				}
			}

			aDC->SetTextAlign(iOldTextAlign);
		}

		// HORIZONTAL AXIS

		nSteps = rect.Height() / 60;
		nSteps = max(nSteps, 1);

		dStep = (dMaxY - dMinY) / double(nSteps);
		dLog = -log10(dStep) + 1;
		dCoeff = pow(10.0, dLog);
		dStep = floor((dCoeff + 1) * dStep / dCoeff);

		if (dStep)
		{
			CString str;
			int iOldTextAlign = aDC->SetTextAlign(TA_RIGHT);

			double dStart(dMinY);

			if (m_bShowAxis && dMinY <= 0.0 && dMaxY > 0.0)
			{
				dStart = dStep * floor(dMinY / dStep + 1.0);
			}

			for (double y = dStart; y <= dMaxY; y += dStep)
			{
				int i = rect.bottom - dScaleY * (y - dMinY);

				if (i > rect.top + 30)
				{
					aDC->MoveTo(rect.left, i);
					aDC->LineTo(rect.right, i);

					str.Format("%.2f", y);
					aDC->TextOut(rect.left - 2, i - 10, str);
				}
			}

			aDC->SetTextAlign(iOldTextAlign);
		}

		aDC->SelectObject(pOldPen);
	}

	// Draw axes ------------------------------------------------------------

	if (m_bShowAxis)
	{
		if (dMinX <= 0.0 && dMaxX > 0.0)
		{
			int i = rect.left - dScaleX * dMinX;

			aDC->MoveTo(i, rect.top);
			aDC->LineTo(i, rect.bottom);
		}

		if (dMinY <= 0.0 && dMaxY > 0.0)
		{
			int i = rect.bottom + dScaleY * dMinY;
			aDC->MoveTo(rect.left, i);
			aDC->LineTo(rect.right, i);
		}
	}

	// Draw graphs ----------------------------------------------------------

	for (int iGraph = 0, nGraphs = m_Graphs.size(); iGraph < nGraphs; iGraph++)
	{
		int nSize = min(m_Graphs[iGraph].m_aX.size(), m_Graphs[iGraph].m_aY.size());

		if (nSize && (dMaxX - dMinX) && (dMaxY - dMinY))
		{
			CPen pen(PS_SOLID, m_Graphs[iGraph].m_iPenWidth, m_Graphs[iGraph].m_rgbPen);
			CPen* pOldPen = aDC->SelectObject(&pen);

			double x = rect.left   + dScaleX * (m_Graphs[iGraph].m_aX[0] - dMinX);
			double y = rect.bottom - dScaleY * (m_Graphs[iGraph].m_aY[0] - dMinY);

			aDC->MoveTo(int(x), int(y));

			for (int i = 1; i < nSize; i++)
			{
				x = double(rect.left)   + dScaleX * (m_Graphs[iGraph].m_aX[i] - dMinX);
				y = double(rect.bottom) - dScaleY * (m_Graphs[iGraph].m_aY[i] - dMinY);

				aDC->LineTo(int(x), int(y));
			}

			aDC->SelectObject(pOldPen);
		}
	}

	aDC->SelectObject(pOldFont);
	aDC->SelectClipRgn(NULL);
}


//***************************************************************************
//                         CGraphSnapshot CLASS
//***************************************************************************

/////////////////////////////////////////////////////////////////////////////
CGraphSnapshot::CGraphSnapshot(COLORREF aRgbBkgd) :
CGraph(aRgbBkgd)
{
}

/////////////////////////////////////////////////////////////////////////////
void CGraphSnapshot::OnDraw(CDC* aDC, const RECT* aRect)
{
	if (!m_Graphs.size() || !m_dExtentsX)
		return;

	ASSERT(aDC && aRect);

	CRgn rgn;
	rgn.CreateRectRgn(aRect->left, aRect->top, aRect->right, aRect->bottom);
	aDC->SelectClipRgn(&rgn, RGN_AND);

	CFont* pOldFont = aDC->SelectObject(&m_Font);

	aDC->SetBkMode(TRANSPARENT);

	// Draw bkgd and frame --------------------------------------------------

	CPen pen(PS_SOLID, 1, m_bShowFrame ? RGB(0, 0, 0) : m_rgbBkgd);
	CPen* pOldPen = aDC->SelectObject(&pen);

	CBrush brush(m_rgbBkgd);
	CBrush* pOldBrush = aDC->SelectObject(&brush);

	aDC->Rectangle(aRect->left, aRect->top, aRect->right, aRect->bottom);

	aDC->SelectObject(pOldPen);
	aDC->SelectObject(pOldBrush);

	// Draw floor net -------------------------------------------------------

	CPen penNet(PS_DOT, 1, RGB(127, 127, 127));
	pOldPen = aDC->SelectObject(&penNet);

	CRect rect(aRect);
	rect.left += 2;
	rect.right -= 2;

	int nFloors = m_Graphs[0].m_aX.size() - 1;
	int nStep = rect.Height() / nFloors;
	int iBasementY = rect.bottom - nStep / 2;

	for (int iFloor = 0, iY = iBasementY; iFloor < nFloors; iFloor++, iY -= nStep)
	{
		aDC->MoveTo(rect.left, iY);
		aDC->LineTo(rect.right, iY);

		CString str;
		str.Format("M%u", iFloor);

		aDC->TextOut(rect.left, iY - 10, str);
	}

	// Draw vertical axis and iso system ------------------------------------

	rect.left += 20;
	int iCenterX = rect.left + rect.Width() / 2;
	double dScaleX = double(rect.Width()) / m_dExtentsX;

	aDC->MoveTo(iCenterX, rect.top + nStep / 2);
	aDC->LineTo(iCenterX, rect.bottom - nStep / 2);

	aDC->SelectObject(pOldPen);

	// Draw graph -----------------------------------------------------------

	CPen penGraph(PS_SOLID, 1, m_Graphs[0].m_rgbPen);
	pOldPen = aDC->SelectObject(&penGraph);

	for (iFloor = 1, iY = iBasementY; iFloor < nFloors; iFloor++, iY -= nStep)
	{
		int iX1 = iCenterX + int(dScaleX * m_Graphs[0].m_aY[iFloor]);
		int iX2 = iCenterX + int(dScaleX * m_Graphs[0].m_aY[iFloor + 1]);
		int iY2 = iY - nStep;

		aDC->MoveTo(iX1, iY);
		aDC->LineTo(iX2, iY2);

		aDC->Ellipse(iX1 - 3, iY - 3, iX1 + 3, iY + 3);

		if (iFloor == nFloors - 1)
			aDC->Ellipse(iX2 - 3, iY2 - 3, iX2 + 3, iY2 + 3);
	}

	aDC->SelectObject(pOldPen);

	// Draw iso system ------------------------------------------------------

	POINT pnts[4];

	pnts[0].x = iCenterX + int(dScaleX * m_Graphs[0].m_aY[0]);
	pnts[0].y = iBasementY;
	pnts[1].x = pnts[0].x + 10;
	pnts[1].y = rect.bottom - 3;
	pnts[2].x = pnts[0].x - 10;
	pnts[2].y = rect.bottom - 3;
	pnts[3].x = pnts[0].x;
	pnts[3].y = pnts[0].y;

	CBrush brushIsoSys(RGB(0, 127, 127));
	pOldBrush = aDC->SelectObject(&brushIsoSys);

	aDC->Polygon(pnts, 4);

	aDC->SelectObject(pOldBrush);

	aDC->SelectObject(pOldFont);
	aDC->SelectClipRgn(NULL);
}

//=================================== EOF ===================================
