/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			ListBuildingsDlg.cpp
//		DESCRIPTION:	implementation of the CListBuildingsDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "ListBuildingsDlg.h"
#include "EditBuildingDlg.h"
#include "Building.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                        CListBuildingsDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CListBuildingsDlg, CDialog)
	//{{AFX_MSG_MAP(CListBuildingsDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnButtonAdd)
	ON_BN_CLICKED(IDC_BUTTON_EDIT, OnButtonEdit)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnButtonDelete)
	ON_NOTIFY(NM_CLICK, IDC_LIST_BUILDINGS, OnClickListBuildings)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_BUILDINGS, OnDblclkListBuildings)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CListBuildingsDlg::CListBuildingsDlg(CWnd* pParent) : 
CDialog(CListBuildingsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CListBuildingsDlg)
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CListBuildingsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// Create column headers ------------------------------------------------

	m_listCtrl.InsertColumn(0,  "#",                LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(1,  "Description",      LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(2,  "Number of floors", LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(3,  "Total mass (kg)",  LVCFMT_LEFT, 120);

	// Enumerate building records -------------------------------------------

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	IDbTable* pTable = g_Application.GetDatabase()->GetBuildingsTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	// Populate table -------------------------------------------------------

	long nIDs;
	pArray->GetSize(&nIDs);

	for (int i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(i, "");
		m_listCtrl.SetItemData(nItem, DWORD(lID));

		// Setup item in the list control
		SetListCtrlItem(nItem);
	}

	if (nIDs)
	{
		m_listCtrl.SetSelectionMark(0);
	}

	UpdateCtrlState();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

//********************************** DDX ************************************
/////////////////////////////////////////////////////////////////////////////
void CListBuildingsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListBuildingsDlg)
	DDX_Control(pDX, IDC_BUTTON_EDIT, m_buttonEdit);
	DDX_Control(pDX, IDC_BUTTON_DELETE, m_buttonDelete);
	DDX_Control(pDX, IDC_LIST_BUILDINGS, m_listCtrl);
	//}}AFX_DATA_MAP
}

//******************************** Controls *********************************
/////////////////////////////////////////////////////////////////////////////
void CListBuildingsDlg::OnButtonAdd() 
{
	CEditBuildingDlg dlg(ID_NEW_RECORD, this);
	
	if (dlg.DoModal() == IDOK)
	{
		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(m_listCtrl.GetItemCount(), "");
		m_listCtrl.SetItemData(nItem, dlg.GetID());

		// Setup item in the list control
		SetListCtrlItem(nItem);

		// Select the item
		m_listCtrl.SetSelectionMark(nItem);
		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListBuildingsDlg::OnButtonEdit() 
{
	// Get selected item
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		// Edit building data
		CEditBuildingDlg dlg(long(m_listCtrl.GetItemData(nItem)), this);
		
		if (dlg.DoModal() == IDOK)
		{
			// Update item data
			SetListCtrlItem(nItem);

			// Select the item
			m_listCtrl.SetSelectionMark(nItem);
			UpdateCtrlState();
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListBuildingsDlg::OnButtonDelete() 
{
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		DWORD dwID = m_listCtrl.GetItemData(nItem);

		IDbTable* pTable = g_Application.GetDatabase()->GetBuildingsTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		int iRet = ::MessageBox(
			this->m_hWnd,
			"NOTE: all the records associated with the selected building will be deleted!\n"
			"Do you want to continue deleting the building?",
			"Confirm deletion",
			MB_ICONEXCLAMATION | MB_YESNO);

		if (iRet == IDYES)
		{
			if (SUCCEEDED(pRS->DeleteRecord(dwID, TRUE)))
			{
				g_Application.GetDatabase()->Commit();
				m_listCtrl.DeleteItem(nItem);

				int nItems = m_listCtrl.GetItemCount();

				if (nItems)
				{
					m_listCtrl.SetSelectionMark(min(nItem, nItems - 1));
				}

				UpdateCtrlState();
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListBuildingsDlg::OnClickListBuildings(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
void CListBuildingsDlg::OnDblclkListBuildings(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnButtonEdit();
	*pResult = 0;
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
void CListBuildingsDlg::SetListCtrlItem(long aItem)
{
	ASSERT(aItem > -1);

	DWORD dwID = m_listCtrl.GetItemData(aItem);
	CBuilding bldg(dwID);

	CString str;
	str.Format("%u", dwID);
	m_listCtrl.SetItemText(aItem, 0, str);

	str = bldg.GetDescription();

	if (str.IsEmpty())
	{
		str.Format("BLDG #%u", (UINT)bldg.GetID());
	}

	m_listCtrl.SetItemText(aItem, 1, str);

	str.Format("%u", bldg.GetNumberOfFloors() - 1);
	m_listCtrl.SetItemText(aItem, 2, str);

	str.Format("%u", (UINT)bldg.GetTotalMass_KG());
	m_listCtrl.SetItemText(aItem, 3, str);
}

/////////////////////////////////////////////////////////////////////////////
void CListBuildingsDlg::UpdateCtrlState()
{
	BOOL bEnable = m_listCtrl.GetSelectionMark() > -1;
	m_buttonEdit.EnableWindow(bEnable);
	m_buttonDelete.EnableWindow(bEnable);
}

//=================================== EOF ===================================
