/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			ListEarthquakeDlg.cpp
//		DESCRIPTION:	implementation the CListEarthquakesDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "ListEarthquakesDlg.h"
#include "EditEqConeDlg.h"
#include "EditEqRealDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                       CListEarthquakesDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CListEarthquakesDlg, CDialog)
	//{{AFX_MSG_MAP(CListEarthquakesDlg)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnButtonDelete)
	ON_NOTIFY(NM_CLICK, IDC_LIST_EARHQUAKES, OnClickListEarhquakes)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_EARHQUAKES, OnDblclkListEarhquakes)
	ON_BN_CLICKED(IDC_BUTTON_ADD_CONE, OnButtonAddCone)
	ON_BN_CLICKED(IDC_BUTTON_ADD_REAL, OnButtonAddReal)
	ON_BN_CLICKED(IDC_BUTTON_EDIT, OnButtonEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CListEarthquakesDlg::CListEarthquakesDlg(CWnd* pParent) : 
CDialog(CListEarthquakesDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CListEarthquakesDlg)
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CListEarthquakesDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Create column headers ------------------------------------------------

	m_listCtrl.InsertColumn(0,  "Type",        LVCFMT_LEFT, 80);
	m_listCtrl.InsertColumn(1,  "Record #",    LVCFMT_LEFT, 80);
	m_listCtrl.InsertColumn(2,  "Description", LVCFMT_LEFT, 100);
	m_listCtrl.InsertColumn(3,  "Parameters",  LVCFMT_LEFT, 290);
	
	// Enumerate CONE EARTHQUAKE records ------------------------------------

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	IDbTable* pTable = g_Application.GetDatabase()->GetEqConeTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	long nIDs;
	pArray->GetSize(&nIDs);

	for (int i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(i, "Cone Eq.");
		m_listCtrl.SetItemData(nItem, DWORD(lID));

		// Setup item in the list control
		SetListCtrlItem(nItem);
	}

	// Enumerate REAL EARTHQUAKE records ------------------------------------

	long iItemOffset = nIDs;

	pTable = g_Application.GetDatabase()->GetEqRealTable();
	pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	pArray->GetSize(&nIDs);

	for (i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(i + iItemOffset, "Real Eq.");
		m_listCtrl.SetItemData(nItem, DWORD(lID));

		// Setup item in the list control
		SetListCtrlItem(nItem);
	}

	// Update state ---------------------------------------------------------

	if (m_listCtrl.GetItemCount() > 0)
	{
		m_listCtrl.SetSelectionMark(0);
	}

	UpdateCtrlState();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

//********************************** DDX ************************************
/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListEarthquakesDlg)
	DDX_Control(pDX, IDC_BUTTON_EDIT, m_buttonEdit);
	DDX_Control(pDX, IDC_BUTTON_DELETE, m_buttonDelete);
	DDX_Control(pDX, IDC_LIST_EARHQUAKES, m_listCtrl);
	//}}AFX_DATA_MAP
}

//******************************** Controls *********************************
/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::OnButtonAddCone() 
{
	CEditEqConeDlg dlg(ID_NEW_RECORD, this);

	if (dlg.DoModal() == IDOK)
	{
		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(m_listCtrl.GetItemCount(), "Cone Eq.");
		m_listCtrl.SetItemData(nItem, dlg.GetID());

		// Setup item in the list control
		SetListCtrlItem(nItem);

		// Select the item
		m_listCtrl.SetSelectionMark(nItem);
		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::OnButtonAddReal() 
{
	CEditEqRealDlg dlg(ID_NEW_RECORD, this);

	if (dlg.DoModal() == IDOK)
	{
		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(m_listCtrl.GetItemCount(), "Real Eq.");
		m_listCtrl.SetItemData(nItem, dlg.GetID());

		// Setup item in the list control
		SetListCtrlItem(nItem);

		// Select the item
		m_listCtrl.SetSelectionMark(nItem);
		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::OnButtonEdit() 
{
	// Get selected item
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		// Get the type string
		CString strType = m_listCtrl.GetItemText(nItem, 0);
		CDialog* pDlg = NULL;

		if (strType == "Cone Eq.")
		{
			pDlg = new CEditEqConeDlg(long(m_listCtrl.GetItemData(nItem)), this);
		}
		else if (strType == "Real Eq.")
		{
			pDlg = new CEditEqRealDlg(long(m_listCtrl.GetItemData(nItem)), this);
		}

		if (pDlg)
		{
			if (pDlg->DoModal() == IDOK)
			{
				// Update item data
				SetListCtrlItem(nItem);

				// Select the item
				m_listCtrl.SetSelectionMark(nItem);
				UpdateCtrlState();
			}

			delete pDlg;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::OnButtonDelete() 
{
	// Get selected item
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		int iRet = ::MessageBox(
			this->m_hWnd,
			"NOTE: all the records associated with the selected earthquake will be deleted!\n"
			"Do you want to continue deleting the earthquake?",
			"Confirm deletion",
			MB_ICONEXCLAMATION | MB_YESNO);

		if (iRet == IDYES)
		{
			// Get the type string
			CString strType = m_listCtrl.GetItemText(nItem, 0);

			// Point to appropriate table -----------------------------------

			IDbTable* pTable = NULL;

			if (strType == "Cone Eq.")
			{
				pTable = g_Application.GetDatabase()->GetEqConeTable();
			}
			else if (strType == "Real Eq.")
			{
				pTable = g_Application.GetDatabase()->GetEqRealTable();
			}

			// Delete record ------------------------------------------------

			if (pTable)
			{
				// Get the recordset interface
				IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

				// Get the record ID
				DWORD dwID = m_listCtrl.GetItemData(nItem);

				if (SUCCEEDED(pRS->DeleteRecord(dwID, TRUE)))
				{
					g_Application.GetDatabase()->Commit();
					m_listCtrl.DeleteItem(nItem);

					int nItems = m_listCtrl.GetItemCount();

					if (nItems)
					{
						m_listCtrl.SetSelectionMark(min(nItem, nItems - 1));
					}

					UpdateCtrlState();
				}
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::OnClickListEarhquakes(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::OnDblclkListEarhquakes(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnButtonEdit();
	*pResult = 0;
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::SetListCtrlItem(long aItem)
{
	ASSERT(aItem > -1);

	// Get the type/record ID
	DWORD dwID = m_listCtrl.GetItemData(aItem);
	CString strType = m_listCtrl.GetItemText(aItem, 0);

	CString strNumber, strDesc, strParams;

	if (strType == "Cone Eq.")
	{
		CEarthquakeCone eq(dwID);

		strNumber.Format("%u", eq.GetID());
		strDesc = eq.GetDescription();
		strParams = eq.GetParamDesc();
	}
	else if (strType == "Real Eq.")
	{
		CEarthquakeReal eq(dwID);
		strNumber.Format("%u", eq.GetID());
		strDesc = eq.GetDescription();
		strParams = eq.GetParamDesc();
	}

	if (strDesc.IsEmpty())
	{
		strDesc.Format("EQ #%u", dwID);
	}

	m_listCtrl.SetItemText(aItem, 0, strType);
	m_listCtrl.SetItemText(aItem, 1, strNumber);
	m_listCtrl.SetItemText(aItem, 2, strDesc);
	m_listCtrl.SetItemText(aItem, 3, strParams);
}

/////////////////////////////////////////////////////////////////////////////
void CListEarthquakesDlg::UpdateCtrlState()
{
	BOOL bEnable = m_listCtrl.GetSelectionMark() > -1;
	m_buttonEdit.EnableWindow(bEnable);
	m_buttonDelete.EnableWindow(bEnable);
}

//=================================== EOF ===================================
