/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			ListIsolationSystemsDlg.cpp
//		DESCRIPTION:	implementation of the CListIsolationSystemsDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "ListIsolationSystemsDlg.h"
#include "EditIsoSystemAFMSDlg.h"
#include "EditIsoSystemShearDlg.h"
#include "EditIsoSystemSlidingDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                     CListIsolationSystemsDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CListIsolationSystemsDlg, CDialog)
	//{{AFX_MSG_MAP(CListIsolationSystemsDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD_AFMS, OnButtonAddAfms)
	ON_BN_CLICKED(IDC_BUTTON_ADD_SHEAR, OnButtonAddShear)
	ON_BN_CLICKED(IDC_BUTTON_ADD_SLIDING, OnButtonAddSliding)
	ON_BN_CLICKED(IDC_BUTTON_EDIT, OnButtonEdit)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnButtonDelete)
	ON_NOTIFY(NM_CLICK, IDC_LIST_ISO_SYSTEMS, OnClickListIsoSystems)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_ISO_SYSTEMS, OnDblclkListIsoSystems)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CListIsolationSystemsDlg::CListIsolationSystemsDlg(CWnd* pParent) : 
CDialog(CListIsolationSystemsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CListIsolationSystemsDlg)
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CListIsolationSystemsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Create column headers ------------------------------------------------

	m_listCtrl.InsertColumn(0,  "Type",        LVCFMT_LEFT, 150);
	m_listCtrl.InsertColumn(1,  "Record #",    LVCFMT_LEFT, 80);
	m_listCtrl.InsertColumn(2,  "Description", LVCFMT_LEFT, 100);
	m_listCtrl.InsertColumn(3,  "Parameters",  LVCFMT_LEFT, 290);
	
	// Enumerate AFMS records -----------------------------------------------

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	IDbTable* pTable = g_Application.GetDatabase()->GetIsoSysAFMSTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	long nIDs;
	pArray->GetSize(&nIDs);

	for (int i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(i, "AFMS");
		m_listCtrl.SetItemData(nItem, DWORD(lID));

		// Setup item in the list control
		SetListCtrlItem(nItem);
	}

	// Enumerate Shear records ----------------------------------------------

	long iItemOffset = nIDs;

	pTable = g_Application.GetDatabase()->GetIsoSysShearTable();
	pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	pArray->GetSize(&nIDs);

	for (i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(i + iItemOffset, "Shear-Hys");
		m_listCtrl.SetItemData(nItem, DWORD(lID));

		// Setup item in the list control
		SetListCtrlItem(nItem);
	}

	// Enumerate Sliding records --------------------------------------------

	iItemOffset += nIDs;

	pTable = g_Application.GetDatabase()->GetIsoSysSlidingTable();
	pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	pArray->GetSize(&nIDs);

	for (i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(i + iItemOffset, "Earthquake Protector");
		m_listCtrl.SetItemData(nItem, DWORD(lID));

		// Setup item in the list control
		SetListCtrlItem(nItem);
	}

	// Update state ---------------------------------------------------------

	if (m_listCtrl.GetItemCount() > 0)
	{
		m_listCtrl.SetSelectionMark(0);
	}

	UpdateCtrlState();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

//********************************** DDX ************************************
/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListIsolationSystemsDlg)
	DDX_Control(pDX, IDC_BUTTON_DELETE, m_buttonDelete);
	DDX_Control(pDX, IDC_BUTTON_EDIT, m_buttonEdit);
	DDX_Control(pDX, IDC_LIST_ISO_SYSTEMS, m_listCtrl);
	//}}AFX_DATA_MAP
}

//******************************** Controls *********************************
/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::OnButtonAddAfms() 
{
	CEditIsoSystemAFMSDlg dlg(ID_NEW_RECORD, this);
	
	if (dlg.DoModal() == IDOK)
	{
		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(m_listCtrl.GetItemCount(), "AFMS");
		m_listCtrl.SetItemData(nItem, dlg.GetID());

		// Setup item in the list control
		SetListCtrlItem(nItem);

		// Select the item
		m_listCtrl.SetSelectionMark(nItem);
		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::OnButtonAddShear() 
{
	CEditIsoSystemShearDlg dlg(ID_NEW_RECORD, this);
	
	if (dlg.DoModal() == IDOK)
	{
		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(m_listCtrl.GetItemCount(), "Shear-Hys");
		m_listCtrl.SetItemData(nItem, dlg.GetID());

		// Setup item in the list control
		SetListCtrlItem(nItem);

		// Select the item
		m_listCtrl.SetSelectionMark(nItem);
		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::OnButtonAddSliding() 
{
	CEditIsoSystemSlidingDlg dlg(ID_NEW_RECORD, this);
	
	if (dlg.DoModal() == IDOK)
	{
		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(m_listCtrl.GetItemCount(), "Earthquake Protector");
		m_listCtrl.SetItemData(nItem, dlg.GetID());

		// Setup item in the list control
		SetListCtrlItem(nItem);

		// Select the item
		m_listCtrl.SetSelectionMark(nItem);
		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::OnButtonEdit() 
{
	// Get selected item
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		// Get the type string
		CString strType = m_listCtrl.GetItemText(nItem, 0);
		CDialog* pDlg = NULL;

		if (strType == "AFMS" || strType == "AFMS-CD")
		{
			pDlg = new CEditIsoSystemAFMSDlg(long(m_listCtrl.GetItemData(nItem)), this);
		}
		else if (strType == "Shear-Hys" || strType == "Shear-Vis")
		{
			pDlg = new CEditIsoSystemShearDlg(long(m_listCtrl.GetItemData(nItem)), this);
		}
		else if (strType == "Earthquake Protector")
		{
			pDlg = new CEditIsoSystemSlidingDlg(long(m_listCtrl.GetItemData(nItem)), this);
		}

		if (pDlg)
		{
			if (pDlg->DoModal() == IDOK)
			{
				// Update item data
				SetListCtrlItem(nItem);

				// Select the item
				m_listCtrl.SetSelectionMark(nItem);
				UpdateCtrlState();
			}

			delete pDlg;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::OnButtonDelete() 
{
	// Get selected item
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		int iRet = ::MessageBox(
			this->m_hWnd,
			"NOTE: all the records associated with the selected isolation will be deleted!\n"
			"Do you want to continue deleting the isolation?",
			"Confirm deletion",
			MB_ICONEXCLAMATION | MB_YESNO);

		if (iRet == IDYES)
		{
			// Get the type string
			CString strType = m_listCtrl.GetItemText(nItem, 0);

			// Point to appropriate table -----------------------------------

			IDbTable* pTable = NULL;

			if (strType == "AFMS" || strType == "AFMS-CD")
			{
				pTable = g_Application.GetDatabase()->GetIsoSysAFMSTable();
			}
			else if (strType == "Shear-Hys" || strType == "Shear-Vis")
			{
				pTable = g_Application.GetDatabase()->GetIsoSysShearTable();
			}
			else if (strType == "Earthquake Protector")
			{
				pTable = g_Application.GetDatabase()->GetIsoSysSlidingTable();
			}

			// Delete record ------------------------------------------------

			if (pTable)
			{
				// Get the recordset interface
				IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

				// Get the record ID
				DWORD dwID = m_listCtrl.GetItemData(nItem);

				if (SUCCEEDED(pRS->DeleteRecord(dwID, TRUE)))
				{
					g_Application.GetDatabase()->Commit();
					m_listCtrl.DeleteItem(nItem);

					int nItems = m_listCtrl.GetItemCount();

					if (nItems)
					{
						m_listCtrl.SetSelectionMark(min(nItem, nItems - 1));
					}

					UpdateCtrlState();
				}
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::OnClickListIsoSystems(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::OnDblclkListIsoSystems(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnButtonEdit();
	*pResult = 0;
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::SetListCtrlItem(long aItem)
{
	ASSERT(aItem > -1);

	// Get the type/record ID
	DWORD dwID = m_listCtrl.GetItemData(aItem);
	CString strType = m_listCtrl.GetItemText(aItem, 0);

	CString strNumber, strDesc, strParams;

	if (strType == "AFMS" || strType == "AFMS-CD")
	{
		CIsoSystemAFMS sys(dwID);

		strType = (sys.GetType() == CIsoSystem::TYPE_AFMS) ? "AFMS" : "AFMS-CD";
		strNumber.Format("%u", sys.GetID());
		strDesc = sys.GetDescription();
		strParams = sys.GetParamDesc();
	}
	else if (strType == "Shear-Hys" || strType == "Shear-Vis")
	{
		CIsoSystemShear sys(dwID);

		strType = (sys.GetType() == CIsoSystem::TYPE_SHEAR_HYS) ? "Shear-Hys" : "Shear-Vis";
		strNumber.Format("%u", sys.GetID());
		strDesc = sys.GetDescription();
		strParams = sys.GetParamDesc();
	}
	else if (strType == "Earthquake Protector")
	{
		CIsoSystemSliding sys(dwID);
		strNumber.Format("%u", sys.GetID());
		strDesc = sys.GetDescription();
		strParams = sys.GetParamDesc();
	}

	if (strDesc.IsEmpty())
	{
		strDesc.Format("SYS #%u", dwID);
	}

	m_listCtrl.SetItemText(aItem, 0, strType);
	m_listCtrl.SetItemText(aItem, 1, strNumber);
	m_listCtrl.SetItemText(aItem, 2, strDesc);
	m_listCtrl.SetItemText(aItem, 3, strParams);
}

/////////////////////////////////////////////////////////////////////////////
void CListIsolationSystemsDlg::UpdateCtrlState()
{
	BOOL bEnable = m_listCtrl.GetSelectionMark() > -1;
	m_buttonEdit.EnableWindow(bEnable);
	m_buttonDelete.EnableWindow(bEnable);
}

//=================================== EOF ===================================
