/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			ListProjects.cpp
//		DESCRIPTION:	implementation of the CListProjectsDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "ListProjectsDlg.h"
#include "EditProjectDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                         CListProjectsDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CListProjectsDlg, CDialog)
	//{{AFX_MSG_MAP(CListProjectsDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnButtonAdd)
	ON_BN_CLICKED(IDC_BUTTON_EDIT, OnButtonEdit)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnButtonDelete)
	ON_NOTIFY(NM_CLICK, IDC_LIST_PROJECTS, OnClickListProjects)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_PROJECTS, OnDblclkListProjects)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CListProjectsDlg::CListProjectsDlg(bool aOpenProject, CWnd* pParent) : 
CDialog(CListProjectsDlg::IDD, pParent),
m_bOpenProject(aOpenProject),
m_lSelProjectID(0)
{
	//{{AFX_DATA_INIT(CListProjectsDlg)
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CListProjectsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Create column headers ------------------------------------------------

	m_listCtrl.InsertColumn(0,  "#",                 LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(1,  "Description",       LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(2,  "Earthquake",        LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(3,  "Building",          LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(4,  "Sec. System",       LVCFMT_LEFT, 120);
	//m_listCtrl.InsertColumn(5,  "AFMS Iso. Sys.",    LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(5,  "Eq. Protector", LVCFMT_LEFT, 120);
	//m_listCtrl.InsertColumn(7,  "Shear Iso. Sys.",   LVCFMT_LEFT, 120);
	
	// Enumerate project records --------------------------------------------

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	IDbTable* pTable = g_Application.GetDatabase()->GetProjectsTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	// Populate table -------------------------------------------------------

	long nIDs;
	pArray->GetSize(&nIDs);

	for (int i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(i, "");
		m_listCtrl.SetItemData(nItem, DWORD(lID));

		// Setup item in the list control
		SetListCtrlItem(nItem);
	}

	if (nIDs)
	{
		m_listCtrl.SetSelectionMark(0);
	}

	if (m_bOpenProject)
	{
		m_buttonAdd.ShowWindow(SW_HIDE);
		m_buttonEdit.ShowWindow(SW_HIDE);
		m_buttonDelete.ShowWindow(SW_HIDE);
	}

	m_buttonOk.SetWindowText(m_bOpenProject ? "Open" : "OK");
	UpdateCtrlState();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

//********************************** DDX ************************************
/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListProjectsDlg)
	DDX_Control(pDX, IDC_BUTTON_ADD, m_buttonAdd);
	DDX_Control(pDX, IDC_LIST_PROJECTS, m_listCtrl);
	DDX_Control(pDX, IDC_BUTTON_DELETE, m_buttonDelete);
	DDX_Control(pDX, IDC_BUTTON_EDIT, m_buttonEdit);
	DDX_Control(pDX, IDOK, m_buttonOk);
	//}}AFX_DATA_MAP
}

//******************************** Controls *********************************
/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::OnClickListProjects(NMHDR* pNMHDR, LRESULT* pResult) 
{	
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::OnDblclkListProjects(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if (m_bOpenProject)
	{
		OnOK();
	}
	else
	{
		OnButtonEdit();
	}

	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::OnButtonAdd() 
{
	CEditProjectDlg dlg(ID_NEW_RECORD, false, this);

	if (dlg.DoModal() == IDOK)
	{
		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(m_listCtrl.GetItemCount(), "");
		m_listCtrl.SetItemData(nItem, dlg.GetID());

		// Setup item in the list control
		SetListCtrlItem(nItem);

		// Select the item
		m_listCtrl.SetSelectionMark(nItem);
		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::OnButtonEdit() 
{
	// Get selected item
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		CEditProjectDlg dlg(long(m_listCtrl.GetItemData(nItem)), false, this);

		if (dlg.DoModal() == IDOK)
		{
			// Update item data
			SetListCtrlItem(nItem);

			// Select the item
			m_listCtrl.SetSelectionMark(nItem);
			UpdateCtrlState();
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::OnButtonDelete() 
{
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		DWORD dwID = m_listCtrl.GetItemData(nItem);

		IDbTable* pTable = g_Application.GetDatabase()->GetProjectsTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		int iRet = ::MessageBox(
			this->m_hWnd,
			"NOTE: all the records associated with the selected project will be deleted!\n"
			"Do you want to continue deleting the project?",
			"Confirm deletion",
			MB_ICONEXCLAMATION | MB_YESNO);

		if (iRet == IDYES)
		{
			if (SUCCEEDED(pRS->DeleteRecord(dwID, TRUE)))
			{
				g_Application.GetDatabase()->Commit();
				m_listCtrl.DeleteItem(nItem);

				int nItems = m_listCtrl.GetItemCount();

				if (nItems)
				{
					m_listCtrl.SetSelectionMark(min(nItem, nItems - 1));
				}

				UpdateCtrlState();
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::OnOK() 
{
	int nItem = m_listCtrl.GetSelectionMark();
	m_lSelProjectID = (nItem > -1) ? m_listCtrl.GetItemData(nItem) : 0;
	
	g_Application.GetMainWnd()->PostMessage(WM_UPDATE_MAINFRAME_UI);

	CDialog::OnOK();
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::SetListCtrlItem(long aItem)
{
	ASSERT(aItem > -1);

	DWORD dwID = m_listCtrl.GetItemData(aItem);
	CProject project(dwID);

	// RECORD #

	CString str;
	str.Format("%u", dwID);
	m_listCtrl.SetItemText(aItem, 0, str);

	// DESCRIPTION

	str = project.GetDescription();

	if (str.IsEmpty())
		str.Format("PROJECT #%u", (UINT)project.GetID());

	m_listCtrl.SetItemText(aItem, 1, str);

	// EARTHQUAKE

	if (project.GetEqReal())
		str.Format("REAL EQ #%u", (UINT)project.GetEqReal()->GetID());
	else if (project.GetEqCone())
		str.Format("CONE EQ #%u", (UINT)project.GetEqCone()->GetID());
	else
		str = "N/A";

	m_listCtrl.SetItemText(aItem, 2, str);

	// BUILDING

	if (project.GetBuilding())
		str.Format("BLDG #%u", (UINT)project.GetBuilding()->GetID());
	else
		str = "N/A";

	m_listCtrl.SetItemText(aItem, 3, str);

	// SEC SYSTEM

	if (project.GetSecSystem())
		str.Format("SEC SYS #%u", (UINT)project.GetSecSystem()->GetID());
	else
		str = "N/A";

	m_listCtrl.SetItemText(aItem, 4, str);

	// ISO SYSTEMS

	// int nIsoSystems = project.GetIsoSysAFMS().size();
	// str.Format("%u", nIsoSystems);
	// m_listCtrl.SetItemText(aItem, 5, str);

	int nIsoSystems = project.GetIsoSysSliding().size();
	str.Format("%u", nIsoSystems);
	m_listCtrl.SetItemText(aItem, 5, str);

	// nIsoSystems = project.GetIsoSysShear().size();
	// str.Format("%u", nIsoSystems);
	// m_listCtrl.SetItemText(aItem, 7, str);
}

/////////////////////////////////////////////////////////////////////////////
void CListProjectsDlg::UpdateCtrlState()
{
	BOOL bEnable = m_listCtrl.GetSelectionMark() > -1;
	m_buttonEdit.EnableWindow(bEnable);
	m_buttonDelete.EnableWindow(bEnable);
	m_buttonOk.EnableWindow(bEnable);
}

//=================================== EOF ===================================
