/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			ListSecondarySystemsDlg.cpp
//		DESCRIPTION:	implementation of the CListSecondarySystemsDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "ListSecondarySystemsDlg.h"
#include "EditSecondarySystemDlg.h"
#include "SecSystem.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                     CListSecondarySystemsDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CListSecondarySystemsDlg, CDialog)
	//{{AFX_MSG_MAP(CListSecondarySystemsDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnButtonAdd)
	ON_BN_CLICKED(IDC_BUTTON_EDIT, OnButtonEdit)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnButtonDelete)
	ON_NOTIFY(NM_CLICK, IDC_LIST_SEC_SYSTEMS, OnClickListSecSystems)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_SEC_SYSTEMS, OnDblclkListSecSystems)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CListSecondarySystemsDlg::CListSecondarySystemsDlg(CWnd* pParent) : 
CDialog(CListSecondarySystemsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CListSecondarySystemsDlg)
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CListSecondarySystemsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Create column headers ------------------------------------------------

	m_listCtrl.InsertColumn(0, "#",             LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(1, "Description",   LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(2, "Building",      LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(3, "Floor",         LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(4, "Mass (kg)",     LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(5, "Damping Ratio", LVCFMT_LEFT, 120);
	
	// Enumerate building records -------------------------------------------

	CComPtr<IStdIntArray> pArray;
	pArray.CoCreateInstance(CLSID_StdIntArray);

	IDbTable* pTable = g_Application.GetDatabase()->GetSecSystemsTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

	// Populate table -------------------------------------------------------

	long nIDs;
	pArray->GetSize(&nIDs);

	for (int i = 0; i < nIDs; i++)
	{
		long lID;
		pArray->Get(i, &lID);

		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(i, "");
		m_listCtrl.SetItemData(nItem, DWORD(lID));

		// Setup item in the list control
		SetListCtrlItem(nItem);
	}

	if (nIDs)
	{
		m_listCtrl.SetSelectionMark(0);
	}

	UpdateCtrlState();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
void CListSecondarySystemsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListSecondarySystemsDlg)
	DDX_Control(pDX, IDC_BUTTON_DELETE, m_buttonDelete);
	DDX_Control(pDX, IDC_LIST_SEC_SYSTEMS, m_listCtrl);
	DDX_Control(pDX, IDC_BUTTON_EDIT, m_buttonEdit);
	//}}AFX_DATA_MAP
}

//******************************** Controls *********************************
/////////////////////////////////////////////////////////////////////////////
void CListSecondarySystemsDlg::OnButtonAdd() 
{
	CEditSecondarySystemDlg dlg(ID_NEW_RECORD, this);
	
	if (dlg.DoModal() == IDOK)
	{
		// Insert item and remember the record ID
		int nItem = m_listCtrl.InsertItem(m_listCtrl.GetItemCount(), "");
		m_listCtrl.SetItemData(nItem, dlg.GetID());

		// Setup item in the list control
		SetListCtrlItem(nItem);

		// Select the item
		m_listCtrl.SetSelectionMark(nItem);
		UpdateCtrlState();
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListSecondarySystemsDlg::OnButtonEdit() 
{
	// Get selected item
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		// Edit building data
		CEditSecondarySystemDlg dlg(long(m_listCtrl.GetItemData(nItem)), this);
		
		if (dlg.DoModal() == IDOK)
		{
			// Update item data
			SetListCtrlItem(nItem);

			// Select the item
			m_listCtrl.SetSelectionMark(nItem);
			UpdateCtrlState();
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListSecondarySystemsDlg::OnButtonDelete() 
{
	int nItem = m_listCtrl.GetSelectionMark();

	if (nItem > -1)
	{
		DWORD dwID = m_listCtrl.GetItemData(nItem);

		IDbTable* pTable = g_Application.GetDatabase()->GetSecSystemsTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		int iRet = ::MessageBox(
			this->m_hWnd,
			"NOTE: all the records associated with the selected system will be deleted!\n"
			"Do you want to continue deleting the system?",
			"Confirm deletion",
			MB_ICONEXCLAMATION | MB_YESNO);

		if (iRet == IDYES)
		{
			if (SUCCEEDED(pRS->DeleteRecord(dwID, TRUE)))
			{
				g_Application.GetDatabase()->Commit();
				m_listCtrl.DeleteItem(nItem);

				int nItems = m_listCtrl.GetItemCount();

				if (nItems)
				{
					m_listCtrl.SetSelectionMark(min(nItem, nItems - 1));
				}

				UpdateCtrlState();
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CListSecondarySystemsDlg::OnClickListSecSystems(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;
}

/////////////////////////////////////////////////////////////////////////////
void CListSecondarySystemsDlg::OnDblclkListSecSystems(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnButtonEdit();
	*pResult = 0;
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
void CListSecondarySystemsDlg::SetListCtrlItem(long aItem)
{
	ASSERT(aItem > -1);

	DWORD dwID = m_listCtrl.GetItemData(aItem);

	CSecSystem secSystem(dwID);

	CString str;
	str.Format("%u", dwID);
	m_listCtrl.SetItemText(aItem, 0, str);

	str = secSystem.GetDescription();

	if (str.IsEmpty())
	{
		str.Format("SYS #%u", (UINT)secSystem.GetID());
	}

	m_listCtrl.SetItemText(aItem, 1, str);

	str.Format("%u", secSystem.GetBuildingID());
	m_listCtrl.SetItemText(aItem, 2, str);

	str.Format("%u", secSystem.GetFloorNumber());
	m_listCtrl.SetItemText(aItem, 3, str);

	str.Format("%.2f", secSystem.GetMass_KG());
	m_listCtrl.SetItemText(aItem, 4, str);

	str.Format("%.4f", secSystem.GetDampingRatio());
	m_listCtrl.SetItemText(aItem, 5, str);
}

/////////////////////////////////////////////////////////////////////////////
void CListSecondarySystemsDlg::UpdateCtrlState()
{
	BOOL bEnable = m_listCtrl.GetSelectionMark() > -1;
	m_buttonEdit.EnableWindow(bEnable);
	m_buttonDelete.EnableWindow(bEnable);
}

//=================================== EOF ===================================
