/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  RAM 2002 International. All Rights Reserved.
//
//		FILE:			Lock.h
//		DESCRIPTION:	definition of the CLock and CScopeLock classes.
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _LOCK_H_
#define _LOCK_H_


//***************************************************************************
//                              CLock CLASS
//***************************************************************************

class CLock
{
	CRITICAL_SECTION m_CriticalSection;

public:
	
	CLock()
	{
		InitializeCriticalSection (&m_CriticalSection); 
	}

	~CLock()
	{
		EnterCriticalSection (&m_CriticalSection); 
		DeleteCriticalSection (&m_CriticalSection); 
	}

	void Lock()
	{
		EnterCriticalSection (&m_CriticalSection); 
	}

	void Unlock()
	{
		LeaveCriticalSection (&m_CriticalSection); 
	}
};


//***************************************************************************
//                           CScopeLock CLASS
//***************************************************************************

class CScopeLock
{
	CLock* m_pLock;

public:

	CScopeLock( CLock* aLock ) :
	m_pLock( aLock )
	{
		m_pLock->Lock();
	}

	~CScopeLock()
	{
		m_pLock->Unlock();
	}
};


#endif // _LOCK_H_

//=================================== EOF ===================================


