/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			MainFrm.cpp
//		DESCRIPTION:	implementation of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "EPET.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "OpenDatabaseDlg.h"
#include "EditProjectDlg.h"
#include "ListProjectsDlg.h"
#include "ListBuildingsDlg.h"
#include "ListSecondarySystemsDlg.h"
#include "ListIsolationSystemsDlg.h"
#include "ListEarthquakesDlg.h"
#include "EqTimeHistoryFrame.h"
#include "AnimateFrame.h"
#include "IsoSystem.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                             CMainFrame CLASS
//***************************************************************************

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_DATABASE_OPEN, OnDatabaseOpen)
	ON_COMMAND(ID_PROJECT_NEW, OnProjectNew)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_NEW, OnUpdateProjectNew)
	ON_COMMAND(ID_PROJECT_OPEN, OnProjectOpen)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_OPEN, OnUpdateProjectOpen)
	ON_COMMAND(ID_EDIT_BUILDINGS, OnEditBuildings)
	ON_UPDATE_COMMAND_UI(ID_EDIT_BUILDINGS, OnUpdateEditBuildings)
	ON_COMMAND(ID_EDIT_SECONDARY_SYSTEMS, OnEditSecondarySystems)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SECONDARY_SYSTEMS, OnUpdateEditSecondarySystems)
	ON_COMMAND(ID_EDIT_ISOLATION_SYSTEMS, OnEditIsolationSystems)
	ON_UPDATE_COMMAND_UI(ID_EDIT_ISOLATION_SYSTEMS, OnUpdateEditIsolationSystems)
	ON_COMMAND(ID_EDIT_EARTHQUAKES, OnEditEarthquakes)
	ON_UPDATE_COMMAND_UI(ID_EDIT_EARTHQUAKES, OnUpdateEditEarthquakes)
	ON_COMMAND(ID_VIEW_EARTHQUAKE_HISTORY, OnViewEarthquakeHistory)
	ON_UPDATE_COMMAND_UI(ID_VIEW_EARTHQUAKE_HISTORY, OnUpdateViewEarthquakeHistory)
	ON_BN_CLICKED(IDC_DIALOGBAR_GO, OnDialogbarGo)
	ON_BN_CLICKED(IDC_DIALOGBAR_SHOW_SNAPSHOT, OnDialogbarShowSnapshot)
	ON_WM_DESTROY()
	ON_COMMAND(ID_PROJECT_EDIT, OnProjectEdit)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_EDIT, OnUpdateProjectEdit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

//**************************** Setup/cleanup ********************************
/////////////////////////////////////////////////////////////////////////////
CMainFrame::CMainFrame()
{
}

/////////////////////////////////////////////////////////////////////////////
BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if(!CMDIFrameWnd::PreCreateWindow(cs))
	{
		return FALSE;
	}

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
	{
		return -1;
	}

	SetIcon(g_Application.LoadIcon(IDR_MAINFRAME), TRUE);
	
	if (!m_wndToolBar.CreateEx(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndChildViewToolBar.CreateEx(this) ||
		!m_wndChildViewToolBar.LoadToolBar(IDR_EPETTYPE))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndAnimateViewToolBar.CreateEx(this) ||
		!m_wndAnimateViewToolBar.LoadToolBar(IDR_ANIMATION))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndDlgBar.Create(this, IDR_MAINFRAME, 
		CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR))
	{
		TRACE0("Failed to create dialogbar\n");
		return -1;		// fail to create
	}

	if (!m_wndReBar.Create(this) ||
		!m_wndReBar.AddBar(&m_wndToolBar) ||
		!m_wndReBar.AddBar(&m_wndChildViewToolBar) ||
		!m_wndReBar.AddBar(&m_wndAnimateViewToolBar) ||
		!m_wndReBar.AddBar(&m_wndDlgBar))
	{
		TRACE0("Failed to create rebar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Remove this if you don't want tool tips
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY);

	// Initialize the dialog bar combobox
	CWnd* pWnd = m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_COMBO);
	m_cboxDlgBar.Attach(pWnd->m_hWnd);

	pWnd = m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_COMBO_FLOORS);
	m_cboxDlgBarFloors.Attach(pWnd->m_hWnd);

	m_wndChildViewToolBar.ShowWindow(SW_HIDE);
	m_wndAnimateViewToolBar.ShowWindow(SW_HIDE);

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnDestroy() 
{
	m_cboxDlgBar.Detach();
	CMDIFrameWnd::OnDestroy();	
}

/////////////////////////////////////////////////////////////////////////////
CMainFrame::~CMainFrame()
{
	g_Application.ExitInstance();
}

//******************************** Commands *********************************
/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnDatabaseOpen() 
{
	COpenDatabaseDlg dlg(this);
	
	if (dlg.DoModal() == IDOK)
	{
		g_Application.OpenDatabase(
			dlg.GetDatabaseName(),
			dlg.GetUserName(),
			dlg.GetPassword(),
			dlg.IsReadOnly());
	}
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnProjectNew() 
{
	CEditProjectDlg dlg(0, true, this);

	if (dlg.DoModal() == IDOK)
	{
		// Create the new project
		g_Application.CreateProject(dlg.GetID());

		// Open child frame
		CreateChildFrame();
	}
}

void CMainFrame::OnUpdateProjectNew(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_Application.GetDatabase() != NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnProjectOpen() 
{
	CListProjectsDlg dlg(true, this);

	if (dlg.DoModal() == IDOK)
	{
		// Remember the new project
		g_Application.CreateProject(dlg.GetSelProjectID());

		// Open child frame
		CreateChildFrame();
	}
}

void CMainFrame::OnUpdateProjectOpen(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_Application.GetDatabase() != NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnProjectEdit() 
{
	CListProjectsDlg dlg(false, this);

	if (dlg.DoModal() == IDOK)
	{
		// TODO
	}
}

void CMainFrame::OnUpdateProjectEdit(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_Application.GetDatabase() != NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnEditBuildings() 
{
	CListBuildingsDlg dlg(this);

	if (dlg.DoModal() == IDOK)
	{
		// TODO
	}
}

void CMainFrame::OnUpdateEditBuildings(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_Application.GetDatabase() != NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnEditSecondarySystems() 
{
	CListSecondarySystemsDlg dlg(this);

	if (dlg.DoModal() == IDOK)
	{
		// TODO
	}
}

void CMainFrame::OnUpdateEditSecondarySystems(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_Application.GetDatabase() != NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnEditIsolationSystems() 
{
	CListIsolationSystemsDlg dlg(this);

	if (dlg.DoModal() == IDOK)
	{
		// TODO
	}
}

void CMainFrame::OnUpdateEditIsolationSystems(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_Application.GetDatabase() != NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnEditEarthquakes() 
{
	CListEarthquakesDlg dlg(this);

	if (dlg.DoModal() == IDOK)
	{
		// TODO
	}
}

void CMainFrame::OnUpdateEditEarthquakes(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_Application.GetDatabase() != NULL);
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnViewEarthquakeHistory() 
{
	if (!g_Application.GetProject().IsCalculated())
		g_Application.GetProject().Calculate();

	// Create a new MDI child window
	CreateNewChild(
		RUNTIME_CLASS(CEqTimeHistoryFrame), 
		IDR_TIMEHISTORY, 
		::LoadMenu(g_Application.m_hInstance, MAKEINTRESOURCE(IDR_EPETTYPE)), 
		::LoadAccelerators(g_Application.m_hInstance, MAKEINTRESOURCE(IDR_MAINFRAME)));
}

void CMainFrame::OnUpdateViewEarthquakeHistory(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(g_Application.GetDatabase() != NULL);
}

//**************************** WindowProc ***********************************
/////////////////////////////////////////////////////////////////////////////
LRESULT CMainFrame::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	if (message == WM_UPDATE_MAINFRAME_UI)
	{
		UpdateDialogBar(wParam, lParam != 0);
	}
	
	return CMDIFrameWnd::WindowProc(message, wParam, lParam);
}

//****************************** Dialog bar *********************************
/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnDialogbarGo() 
{
	if (m_cboxDlgBar.GetCount())
	{
		int iSelIndex = m_cboxDlgBar.GetCurSel();

		if (iSelIndex > -1)
		{
			DWORD dwData = m_cboxDlgBar.GetItemData(iSelIndex);

			if (!MDIGetActive())
			{
				ASSERT(g_Application.GetDatabase());

				// Remember the selected project
				g_Application.CreateProject(dwData);

				// Create child frame
				CreateChildFrame();
			}
			else if (g_Application.GetProject().IsCalculated())
			{
				CIsoSystem* pIsoSys = (CIsoSystem*)m_cboxDlgBar.GetItemData(iSelIndex);
				g_Application.SetSelIsoSystem(pIsoSys);

				// Create a new MDI child window
				CreateNewChild(
					RUNTIME_CLASS(CAnimateFrame), 
					IDR_ANIMATION, 
					::LoadMenu(g_Application.m_hInstance, MAKEINTRESOURCE(IDR_ANIMATION)), 
					::LoadAccelerators(g_Application.m_hInstance, MAKEINTRESOURCE(IDR_MAINFRAME)));
			}
			else
			{
				// Calculate the project
				g_Application.GetProject().Calculate();

				// Change the button's text
				CWnd* pWnd = m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_GO);
				pWnd->SetWindowText("Animate");
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::OnDialogbarShowSnapshot() 
{
	if (m_cboxDlgBarFloors.GetCount())
	{
		int iSelIndex = m_cboxDlgBarFloors.GetCurSel();

		if (iSelIndex > -1)
		{
			DWORD dwData = m_cboxDlgBarFloors.GetItemData(iSelIndex);
			g_Application.SetSnapshotFloor(dwData);

			// Create child frame
			CreateChildFrame();
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::UpdateDialogBar(int aWinResID, bool aSetFocus)
{
	if (aSetFocus) // Message from child view -------------------------------
	{
		switch (aWinResID)
		{
		case IDR_EPETTYPE:
		case IDR_TIMEHISTORY:
			{
				m_wndToolBar.ShowWindow(SW_HIDE);
				m_wndChildViewToolBar.ShowWindow(SW_SHOWNA);
				m_wndAnimateViewToolBar.ShowWindow(SW_HIDE);

				m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_COMBO_FLOORS)->ShowWindow(SW_SHOWNA);
				m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_SHOW_SNAPSHOT)->ShowWindow(SW_SHOWNA);

				RecalcLayout();

				UpdateDialogBarCombo(aWinResID);
				UpdateDialogBarFloorList();

				CWnd* pWnd = m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_GO);
				pWnd->SetWindowText(g_Application.GetProject().IsCalculated() ? "Animate" : "Calculate");
			}
			break;

		case IDR_ANIMATION:
			{
				m_wndToolBar.ShowWindow(SW_HIDE);
				m_wndChildViewToolBar.ShowWindow(SW_HIDE);
				m_wndAnimateViewToolBar.ShowWindow(SW_SHOWNA);

				m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_COMBO_FLOORS)->ShowWindow(SW_SHOWNA);
				m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_SHOW_SNAPSHOT)->ShowWindow(SW_SHOWNA);

				RecalcLayout();

				UpdateDialogBarCombo(aWinResID);
				UpdateDialogBarFloorList();

				CWnd* pWnd = m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_GO);
				pWnd->SetWindowText("Animate");
			}
			break;

		default:
			break;
		}
	}
	else if (!MDIGetActive()) // No views -----------------------------------
	{
		m_wndToolBar.ShowWindow(SW_SHOWNA);
		m_wndChildViewToolBar.ShowWindow(SW_HIDE);
		m_wndAnimateViewToolBar.ShowWindow(SW_HIDE);

		m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_COMBO_FLOORS)->ShowWindow(SW_HIDE);
		m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_SHOW_SNAPSHOT)->ShowWindow(SW_HIDE);

		RecalcLayout();

		UpdateDialogBarCombo(IDR_MAINFRAME);

		CWnd* pWnd = m_wndDlgBar.GetDlgItem(IDC_DIALOGBAR_GO);
		pWnd->SetWindowText("Open Project");
	}
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::UpdateDialogBarCombo(int aWinResID)
{
	m_cboxDlgBar.ResetContent();

	switch (aWinResID)
	{
	case IDR_EPETTYPE:
	case IDR_TIMEHISTORY:
	case IDR_ANIMATION:
		{
			CProject& project = g_Application.GetProject();

			vector<const CIsoSystem*> v;
			project.GetIsoSystems(v);

			int iCurSel = 0;

			for (int i = 0, nCount = v.size(); i < nCount; i++)
			{
				int iIndex = m_cboxDlgBar.AddString(v[i]->GetParamDesc());
				m_cboxDlgBar.SetItemData(iIndex, DWORD(v[i]));

				if (v[i] == g_Application.GetSelIsoSystem())
				{
					iCurSel = i;
				}
			}

			m_cboxDlgBar.SetCurSel(iCurSel);
		}
		break;

	case IDR_MAINFRAME:
	default:

		if (g_Application.GetDatabase())
		{
			CComPtr<IStdIntArray> pArray;
			pArray.CoCreateInstance(CLSID_StdIntArray);

			IDbTable* pTable = g_Application.GetDatabase()->GetProjectsTable();
			IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);
			pRS->EnumRecords(NULL, DB_PRIMARY_KEY_FIELD_ID, pArray);

			long nIDs;
			pArray->GetSize(&nIDs);

			int iSelIndex = 0;

			for (int i = 0; i < nIDs; i++)
			{
				long lID;
				pArray->Get(i, &lID);

				CProject project(lID);
				const char* pszDesc = project.GetDescription();
				CString str;

				if (strlen(pszDesc))
				{
					str.Format("PROJECT #%d [%s]", lID, pszDesc);
				}
				else
				{
					str.Format("PROJECT #%d", lID);
				}

				int iIndex = m_cboxDlgBar.AddString(str);
				m_cboxDlgBar.SetItemData(iIndex, lID);

				if (lID == g_Application.GetProject().GetID())
				{
					iSelIndex = iIndex;
				}
			}

			m_cboxDlgBar.SetCurSel(iSelIndex);
		}
		break;
	}
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::UpdateDialogBarFloorList()
{
	const CBuilding* pBldg = g_Application.GetProject().GetBuilding();
	ASSERT(pBldg);

	m_cboxDlgBarFloors.ResetContent();

	for (int iFloor = 0, nFloors = pBldg->GetNumberOfFloors(); iFloor < nFloors; iFloor++)
	{
		CString str;

		if (iFloor)
			str.Format("%u-floor", iFloor);
		else
			str = "Basement";

		int iIndex = m_cboxDlgBarFloors.AddString(str);
		m_cboxDlgBarFloors.SetItemData(iIndex, iFloor);
	}

	int iSnapshotFloor = min(g_Application.GetSnapshotFloor(), nFloors - 1);
	g_Application.SetSnapshotFloor(iSnapshotFloor);
	m_cboxDlgBarFloors.SetCurSel(iSnapshotFloor);
}

// Creating children ********************************************************
/////////////////////////////////////////////////////////////////////////////
void CMainFrame::CreateChildFrame()
{
	UpdateDialogBarFloorList();

	if (!g_Application.GetProject().IsCalculated())
		g_Application.GetProject().Calculate();

	// Create a new MDI child window
	CreateNewChild(
		RUNTIME_CLASS(CChildFrame), 
		IDR_EPETTYPE, 
		::LoadMenu(g_Application.m_hInstance, MAKEINTRESOURCE(IDR_EPETTYPE)), 
		::LoadAccelerators(g_Application.m_hInstance, MAKEINTRESOURCE(IDR_MAINFRAME)));
}

#ifdef _DEBUG

//********************************* Debug ***********************************
/////////////////////////////////////////////////////////////////////////////
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

/////////////////////////////////////////////////////////////////////////////
void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

//=================================== EOF ===================================
