/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			Project.cpp
//		DESCRIPTION:	implementation of the CProject class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "Project.h"
#include "Earthquake.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MAX_ISOSYS 4


//***************************************************************************
//                             CProject CLASS
//***************************************************************************

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CProject::CProject(long aID) :
m_lID(aID),
m_strDescription(""),
m_pBuilding(NULL),
m_pSecSystem(NULL),
m_pEqReal(NULL),
m_pEqCone(NULL),
m_bCalculated(false)
{
	if (m_lID)
	{
		IDbTable* pTable = DB_PTR->GetProjectsTable();

		m_strDescription = DB_PTR->GetFieldAsString(pTable, "DESCRIPTION", m_lID);

		// BUILDING

		long lID = DB_PTR->GetFieldAsInteger(pTable, "ID_BUILDING", m_lID);

		if (lID) 
			m_pBuilding = new CBuilding(lID);

		// SEC SYSTEM

		lID = DB_PTR->GetFieldAsInteger(pTable, "ID_SEC_SYSTEM", m_lID);

		if (lID) 
			m_pSecSystem = new CSecSystem(lID);

		// EARTHQUAKES

		lID = DB_PTR->GetFieldAsInteger(pTable, "ID_EQ_REAL", m_lID);

		if (lID) 
			m_pEqReal = new CEarthquakeReal(lID);

		lID = DB_PTR->GetFieldAsInteger(pTable, "ID_EQ_CONE", m_lID);

		if (lID) 
			m_pEqCone = new CEarthquakeCone(lID);

		// ISO SYS AFMS

		for (int i = 0; i < MAX_ISOSYS; i++)
		{
			CString str;
			str.Format("ID_ISO_SYS_AFMS%u", i);
			lID = DB_PTR->GetFieldAsInteger(pTable, str, m_lID);

			if (lID) 
				m_apIsoSysAFMS.push_back(new CIsoSystemAFMS(lID));
		}

		// ISO SYS  SLIDING

		for (i = 0; i < MAX_ISOSYS; i++)
		{
			CString str;
			str.Format("ID_ISO_SYS_SLIDING%u", i);
			lID = DB_PTR->GetFieldAsInteger(pTable, str, m_lID);

			if (lID) 
				m_apIsoSysSliding.push_back(new CIsoSystemSliding(lID));
		}

		// ISO SYS SHEAR

		for (i = 0; i < MAX_ISOSYS; i++)
		{
			CString str;
			str.Format("ID_ISO_SYS_SHEAR%u", i);
			lID = DB_PTR->GetFieldAsInteger(pTable, str, m_lID);

			if (lID) 
				m_apIsoSysShear.push_back(new CIsoSystemShear(lID));
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
CProject::CProject(const CProject& aProject)
{
	m_lID = aProject.m_lID;
	m_strDescription = aProject.m_strDescription;
	m_bCalculated = aProject.m_bCalculated;

	m_pBuilding = aProject.GetBuilding() ? new CBuilding(*aProject.GetBuilding()) : NULL;
	m_pSecSystem = aProject.GetSecSystem() ? new CSecSystem(*aProject.GetSecSystem()) : NULL;
	m_pEqReal = aProject.GetEqReal() ? new CEarthquakeReal(*aProject.GetEqReal()) : NULL;
	m_pEqCone = aProject.GetEqCone() ? new CEarthquakeCone(*aProject.GetEqCone()) : NULL;

	m_IsoSysFixed = *aProject.GetIsoSystemFixed();

	for (int i = 0, nCount = aProject.m_apIsoSysAFMS.size(); i < nCount; i++)
		m_apIsoSysAFMS.push_back(new CIsoSystemAFMS(*aProject.m_apIsoSysAFMS[i]));

	for (i = 0, nCount = aProject.m_apIsoSysSliding.size(); i < nCount; i++)
		m_apIsoSysSliding.push_back(new CIsoSystemSliding(*aProject.m_apIsoSysSliding[i]));

	for (i = 0, nCount = aProject.m_apIsoSysShear.size(); i < nCount; i++)
		m_apIsoSysShear.push_back(new CIsoSystemShear(*aProject.m_apIsoSysShear[i]));

	for (i = 0, nCount = aProject.m_apIsoSysData.size(); i < nCount; i++)
		m_apIsoSysData.push_back(new CCalculusIsoSysData(*aProject.m_apIsoSysData[i]));
}

/////////////////////////////////////////////////////////////////////////////
const CProject& CProject::operator = (const CProject& aProject)
{
	// Clear object ---------------------------------------------------------

	if (m_pBuilding) 
		delete m_pBuilding;

	if (m_pSecSystem) 
		delete m_pSecSystem;

	if (m_pEqReal) 
		delete m_pEqReal;

	if (m_pEqCone) 
		delete m_pEqCone;

	ClearIsoSystems();

	// Update object --------------------------------------------------------

	m_lID = aProject.m_lID;
	m_strDescription = aProject.m_strDescription;
	m_bCalculated = aProject.m_bCalculated;

	m_pBuilding = aProject.GetBuilding() ? new CBuilding(*aProject.GetBuilding()) : NULL;
	m_pSecSystem = aProject.GetSecSystem() ? new CSecSystem(*aProject.GetSecSystem()) : NULL;
	m_pEqReal = aProject.GetEqReal() ? new CEarthquakeReal(*aProject.GetEqReal()) : NULL;
	m_pEqCone = aProject.GetEqCone() ? new CEarthquakeCone(*aProject.GetEqCone()) : NULL;

	m_IsoSysFixed = *aProject.GetIsoSystemFixed();

	for (int i = 0, nCount = aProject.m_apIsoSysAFMS.size(); i < nCount; i++)
		m_apIsoSysAFMS.push_back(new CIsoSystemAFMS(*aProject.m_apIsoSysAFMS[i]));

	for (i = 0, nCount = aProject.m_apIsoSysSliding.size(); i < nCount; i++)
		m_apIsoSysSliding.push_back(new CIsoSystemSliding(*aProject.m_apIsoSysSliding[i]));

	for (i = 0, nCount = aProject.m_apIsoSysShear.size(); i < nCount; i++)
		m_apIsoSysShear.push_back(new CIsoSystemShear(*aProject.m_apIsoSysShear[i]));

	for (i = 0, nCount = aProject.m_apIsoSysData.size(); i < nCount; i++)
		m_apIsoSysData.push_back(new CCalculusIsoSysData(*aProject.m_apIsoSysData[i]));

	return *this;
}

/////////////////////////////////////////////////////////////////////////////
CProject::~CProject()
{
	if (m_pBuilding) 
		delete m_pBuilding;

	if (m_pSecSystem) 
		delete m_pSecSystem;

	if (m_pEqReal) 
		delete m_pEqReal;

	if (m_pEqCone) 
		delete m_pEqCone;

	ClearIsoSystems();
}

//******************************* Archiving *********************************
/////////////////////////////////////////////////////////////////////////////
bool CProject::Save()
{
	if (!IsValid())
	{
		return false;
	}

	IDbTable* pTable = DB_PTR->GetProjectsTable();

	if (!m_lID)
	{
		// Create record
		IDbRecordSet* pRS = DB_PTR->TableToRecordSet(pTable);
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &m_lID);
		ASSERT(m_lID);
	}

	DB_PTR->SetFieldToString(pTable, "DESCRIPTION", m_lID, m_strDescription);

	DB_PTR->SetFieldToInteger(pTable, "ID_BUILDING", m_lID, m_pBuilding ? m_pBuilding->GetID() : 0);
	DB_PTR->SetFieldToInteger(pTable, "ID_SEC_SYSTEM", m_lID, m_pSecSystem ? m_pSecSystem->GetID() : 0);
	DB_PTR->SetFieldToInteger(pTable, "ID_EQ_REAL", m_lID, m_pEqReal ? m_pEqReal->GetID() : 0);
	DB_PTR->SetFieldToInteger(pTable, "ID_EQ_CONE", m_lID, m_pEqCone ? m_pEqCone->GetID() : 0);

	for (int i = 0, nCount = m_apIsoSysAFMS.size(); i < MAX_ISOSYS; i++)
	{
		CString str;
		str.Format("ID_ISO_SYS_AFMS%u", i);
		long lID = (i < nCount) ? m_apIsoSysAFMS[i]->GetID() : 0;

		DB_PTR->SetFieldToInteger(pTable, str, m_lID, lID);
	}

	for (i = 0, nCount = m_apIsoSysSliding.size(); i < MAX_ISOSYS; i++)
	{
		CString str;
		str.Format("ID_ISO_SYS_SLIDING%u", i);
		long lID = (i < nCount) ? m_apIsoSysSliding[i]->GetID() : 0;

		DB_PTR->SetFieldToInteger(pTable, str, m_lID, lID);
	}

	for (i = 0, nCount = m_apIsoSysShear.size(); i < MAX_ISOSYS; i++)
	{
		CString str;
		str.Format("ID_ISO_SYS_SHEAR%u", i);
		long lID = (i < nCount) ? m_apIsoSysShear[i]->GetID() : 0;

		DB_PTR->SetFieldToInteger(pTable, str, m_lID, lID);
	}

	DB_PTR->Commit();

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::IsValid() const
{
	return (m_pBuilding && (m_pEqReal || m_pEqCone));
}

// Getting data *************************************************************
/////////////////////////////////////////////////////////////////////////////
void CProject::GetIsoSystems(vector<const CIsoSystem*>& aVector) const
{
	aVector.push_back(&m_IsoSysFixed);

	for (int i = 0, nCount = m_apIsoSysShear.size(); i < nCount; i++)
		aVector.push_back(m_apIsoSysShear[i]);

	for (i = 0, nCount = m_apIsoSysSliding.size(); i < nCount; i++)
		aVector.push_back(m_apIsoSysSliding[i]);

	for (i = 0, nCount = m_apIsoSysAFMS.size(); i < nCount; i++)
		aVector.push_back(m_apIsoSysAFMS[i]);
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::HasIsoSysAFMSID(long aID) const
{
	for (int i = 0, nCount = m_apIsoSysAFMS.size(); i < nCount; i++)
	{
		if (m_apIsoSysAFMS[i]->GetID() == aID)
			return true;
	}

	return false;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::HasIsoSysSlidingID(long aID) const
{
	for (int i = 0, nCount = m_apIsoSysSliding.size(); i < nCount; i++)
	{
		if (m_apIsoSysSliding[i]->GetID() == aID)
			return true;
	}

	return false;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::HasIsoSysShearID(long aID) const
{
	for (int i = 0, nCount = m_apIsoSysShear.size(); i < nCount; i++)
	{
		if (m_apIsoSysShear[i]->GetID() == aID)
			return true;
	}

	return false;
}

//***************************** Modifying data ******************************
/////////////////////////////////////////////////////////////////////////////
bool CProject::SetBuildingID(long aID)
{
	if (m_pBuilding)
	{
		delete m_pBuilding;
		m_pBuilding = NULL;
	}

	if (aID)
	{
		m_pBuilding = new CBuilding(aID);
	}

	m_bCalculated = false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::SetSecSystemID(long aID)
{
	if (m_pSecSystem)
	{
		delete m_pSecSystem;
		m_pSecSystem = NULL;
	}

	if (aID)
	{
		m_pSecSystem = new CSecSystem(aID);
	}

	m_bCalculated = false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::SetEqRealID(long aID)
{
	if (m_pEqReal)
	{
		delete m_pEqReal;
		m_pEqReal = NULL;
	}

	if (aID)
	{
		m_pEqReal = new CEarthquakeReal(aID);
	}

	m_bCalculated = false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::SetEqConeID(long aID)
{
	if (m_pEqCone)
	{
		delete m_pEqCone;
		m_pEqCone = NULL;
	}

	if (aID)
	{
		m_pEqCone = new CEarthquakeCone(aID);
	}

	m_bCalculated = false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::SetIsoSysAFMSID(long aID)
{
	if (!HasIsoSysAFMSID(aID))
		m_apIsoSysAFMS.push_back(new CIsoSystemAFMS(aID));

	m_bCalculated = false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::SetIsoSysSlidingID(long aID)
{
	if (!HasIsoSysSlidingID(aID))
		m_apIsoSysSliding.push_back(new CIsoSystemSliding(aID));

	m_bCalculated = false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::SetIsoSysShearID(long aID)
{
	if (!HasIsoSysShearID(aID))
		m_apIsoSysShear.push_back(new CIsoSystemShear(aID));

	m_bCalculated = false;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
void CProject::ClearIsoSystems()
{
	for (int i = 0, nCount = m_apIsoSysAFMS.size(); i < nCount; i++)
	{
		delete m_apIsoSysAFMS[i];
	}

	m_apIsoSysAFMS.clear();

	for (i = 0, nCount = m_apIsoSysSliding.size(); i < nCount; i++)
	{
		delete m_apIsoSysSliding[i];
	}

	m_apIsoSysSliding.clear();

	for (i = 0, nCount = m_apIsoSysShear.size(); i < nCount; i++)
	{
		delete m_apIsoSysShear[i];
	}

	m_apIsoSysShear.clear();

	ClearCalculusIsoSysData();
	m_bCalculated = false;
}

//******************************** Calculus *********************************
/////////////////////////////////////////////////////////////////////////////
const CCalculusIsoSysData* CProject::GetCalculusByIsoSystem(const CIsoSystem* aIsoSys) const
{
	for (int i = 0, nCount = m_apIsoSysData.size(); i < nCount; i++)
	{
		if (m_apIsoSysData[i]->GetIsoSystem() == aIsoSys)
		{
			return m_apIsoSysData[i];
		}
	}

	return NULL;
}

/////////////////////////////////////////////////////////////////////////////
int CProject::GetIndexByTime(double aTime) const
{
	CEarthquake* pEq;
	
	if (m_pEqReal)
		pEq = m_pEqReal;
	else
		pEq = m_pEqCone;

	if (aTime < 0.0 || aTime > pEq->GetTotalTime_SEC())
	{
		// Out of range
		return -1;
	}

	return int(aTime / pEq->GetStep_SEC());
}

/////////////////////////////////////////////////////////////////////////////
void CProject::ClearCalculusIsoSysData()
{
	for (int i = 0, nCount = m_apIsoSysData.size(); i < nCount; i++)
	{
		delete m_apIsoSysData[i];
	}

	m_apIsoSysData.clear();
	m_bCalculated = false;
}

/////////////////////////////////////////////////////////////////////////////
bool CProject::Calculate()
{
	CWaitCursor wait;

	if (m_bCalculated) // already calculated
		return true;

	CEarthquake* pEq = (CEarthquake*)GetEq();

	if (!m_pBuilding || !pEq)
		return false;

	ClearCalculusIsoSysData();

	if (!pEq->Calculate())
		return false;

	// Create calculus data -------------------------------------------------

	double dTimeStep = pEq->GetStep_SEC();
	double dTimeStep2 = 0.5 * pow(dTimeStep, 2.0);
	double dTimeStep3 = 0.1667 * pow(dTimeStep, 3.0);

	int nFloors = m_pBuilding->GetNumberOfFloors();
	int nIterations = int (pEq->GetTotalTime_SEC() / dTimeStep) + 1;

	vector<const CIsoSystem*> v;
	GetIsoSystems(v);

	for (int iIsoSys = 0, nIsoSys = v.size(); iIsoSys < nIsoSys; iIsoSys++)
	{
		m_apIsoSysData.push_back(new CCalculusIsoSysData(v[iIsoSys], nFloors, nIterations));
	}

	// Calculate ------------------------------------------------------------

	for (iIsoSys = 0; iIsoSys < nIsoSys; iIsoSys++)
	{
		CCalculusIsoSysData* pData = m_apIsoSysData[iIsoSys];

		double dTime = 0.0;

		for (int iIndex = 0; iIndex < nIterations; iIndex++)
		{
			for (int iFloor = 0; iFloor < nFloors; iFloor++)
			{
				// Absolute/relative data for this floor
				double dAbsDispA(0.0), dAbsVelA(0.0), dAbsAccelA(0.0), dAbsAssel2A(0.0);
				double dRelDispA(0.0), dRelVelA(0.0), dRelAccelA(0.0), dRelAssel2A(0.0);

				if (!iFloor)
				{
					if (v[iIsoSys]->GetMaxRelDisp_CM() == 0.0 || !iIndex)
					{
						// Absolute displacements - from earthquake
						dAbsDispA   = pEq->GetAbsDisp_CM(dTime);
						dAbsVelA    = pEq->GetAbsVel_CM_SEC(dTime);
						dAbsAccelA  = pEq->GetAbsAccel_CM_SEC2(dTime);
						dAbsAssel2A = pEq->GetAbsAccel2_CM_SEC3(dTime);
					}
					else
					{
						// Get relative data from the last iteration
						double dDisp   = pData->GetRelDisp_CM       (0, iIndex - 1);
						double dVel    = pData->GetRelVel_CM_SEC    (0, iIndex - 1);
						double dAccel  = pData->GetRelAccel_CM_SEC2 (0, iIndex - 1);
						double dAccel2 = pData->GetRelAccel2_CM_SEC3(0, iIndex - 1);

						// Update data using the Tailor's formula
						dRelDispA   = dDisp  + dTimeStep * dVel   + dTimeStep2 * dAccel + dTimeStep3 * dAccel2;
						dRelVelA    = dVel   + dTimeStep * dAccel + dTimeStep2 * dAccel2;
						dRelAccelA  = dAccel + dTimeStep * dAccel2;
						dRelAssel2A = dAccel2;

						// Get absolute data from the last iteration
						dDisp   = pData->GetAbsDisp_CM       (0, iIndex - 1);
						dVel    = pData->GetAbsVel_CM_SEC    (0, iIndex - 1);
						dAccel  = pData->GetAbsAccel_CM_SEC2 (0, iIndex - 1);
						dAccel2 = pData->GetAbsAccel2_CM_SEC3(0, iIndex - 1);

						// Update data using the Tailor's formula
						dAbsDispA   = dDisp  + dTimeStep * dVel   + dTimeStep2 * dAccel + dTimeStep3 * dAccel2;
						dAbsVelA    = dVel   + dTimeStep * dAccel + dTimeStep2 * dAccel2;
						dAbsAccelA  = dAccel + dTimeStep * dAccel2;
						dAbsAssel2A = dAccel2;

						// Relative data for the upper floor
						double dRelDispB(0.0), dRelVelB(0.0), dRelAccelB(0.0), dRelAssel2B(0.0);

						if (nFloors > 1)
						{
							// Get relative data for the upper floor from the last iteration
							dDisp   = pData->GetRelDisp_CM       (1, iIndex - 1);
							dVel    = pData->GetRelVel_CM_SEC    (1, iIndex - 1);
							dAccel  = pData->GetRelAccel_CM_SEC2 (1, iIndex - 1);
							dAccel2 = pData->GetRelAccel2_CM_SEC3(1, iIndex - 1);

							// Update data using the Tailor's formula
							dRelDispB   = dDisp  + dTimeStep * dVel   + dTimeStep2 * dAccel + dTimeStep3 * dAccel2;
							dRelVelB    = dVel   + dTimeStep * dAccel + dTimeStep2 * dAccel2;
							dRelAccelB  = dAccel + dTimeStep * dAccel2;
							dRelAssel2B = dAccel2;
						}

						// Get the formula coefficients (methods check floor bounds)
						double dMassA = m_pBuilding->GetFloorMass_KG(0);
						double dMassT = m_pBuilding->GetTotalMass_KG();

						double dSBT   = (v[iIsoSys]->GetType() == CIsoSystem::TYPE_SHEAR_HYS) ? dRelVelA : 0.0;

						double dSBTA  = v[iIsoSys]->GetDamping_KG_SEC  (dMassA, dMassT, dTime, dRelDispA, dRelVelA);
						double dSKAA  = v[iIsoSys]->GetRigidity_KG_SEC2(dMassA, dMassT, dTime, dRelDispA, dRelVelA);

						double dSBTB  = m_pBuilding->GetDamping_KG_SEC  (1, dRelDispB);
						double dSKAB  = m_pBuilding->GetRigidity_KG_SEC2(1, dRelDispB);

						// Update the absolute acceleration
						dAbsAccelA  = (dSBTB * dRelVelB   - dSBTA * dRelVelA + dSKAB * dRelDispB - dSKAA * dRelDispA) / dMassA;
						dAbsAssel2A = (dSBTB * dRelAccelB - dSBTA * dSBT     + dSKAB * dRelVelB  - dSKAA * dRelVelA ) / dMassA;

						// Update relative data according to formula: V(floor) = U(floor) - U(floor - 1)
						dRelDispA   = dAbsDispA   - pEq->GetAbsDisp_CM(dTime);
						dRelVelA    = dAbsVelA    - pEq->GetAbsVel_CM_SEC(dTime);
						dRelAccelA  = dAbsAccelA  - pEq->GetAbsAccel_CM_SEC2(dTime);
						dRelAssel2A = dAbsAssel2A - pEq->GetAbsAccel2_CM_SEC3(dTime);

						// Limit relative displacement
						if (dRelDispA > 0.0)
							dRelDispA = min(dRelDispA, +v[iIsoSys]->GetMaxRelDisp_CM());
						else
							dRelDispA = max(dRelDispA, -v[iIsoSys]->GetMaxRelDisp_CM());
					}
				}
				else if (!iIndex)
				{
					// Assume immediate transfer of absolute movement to upper floor
					dAbsDispA   = pData->GetAbsDisp_CM       (iFloor - 1, iIndex);
					dAbsVelA    = pData->GetAbsVel_CM_SEC    (iFloor - 1, iIndex);
					dAbsAccelA  = pData->GetAbsAccel_CM_SEC2 (iFloor - 1, iIndex);
					dAbsAssel2A = pData->GetAbsAccel2_CM_SEC3(iFloor - 1, iIndex);
				}
				else
				{
					// Get relative data from the last iteration
					double dDisp   = pData->GetRelDisp_CM       (iFloor, iIndex - 1);
					double dVel    = pData->GetRelVel_CM_SEC    (iFloor, iIndex - 1);
					double dAccel  = pData->GetRelAccel_CM_SEC2 (iFloor, iIndex - 1);
					double dAccel2 = pData->GetRelAccel2_CM_SEC3(iFloor, iIndex - 1);

					// Update data using the Tailor's formula
					dRelDispA   = dDisp  + dTimeStep * dVel   + dTimeStep2 * dAccel + dTimeStep3 * dAccel2;
					dRelVelA    = dVel   + dTimeStep * dAccel + dTimeStep2 * dAccel2;
					dRelAccelA  = dAccel + dTimeStep * dAccel2;
					dRelAssel2A = dAccel2;

					// Get absolute data from the last iteration
					dDisp   = pData->GetAbsDisp_CM       (iFloor, iIndex - 1);
					dVel    = pData->GetAbsVel_CM_SEC    (iFloor, iIndex - 1);
					dAccel  = pData->GetAbsAccel_CM_SEC2 (iFloor, iIndex - 1);
					dAccel2 = pData->GetAbsAccel2_CM_SEC3(iFloor, iIndex - 1);

					// Update data using the Tailor's formula
					dAbsDispA   = dDisp  + dTimeStep * dVel   + dTimeStep2 * dAccel + dTimeStep3 * dAccel2;
					dAbsVelA    = dVel   + dTimeStep * dAccel + dTimeStep2 * dAccel2;
					dAbsAccelA  = dAccel + dTimeStep * dAccel2;
					dAbsAssel2A = dAccel2;

					// Relative data for the upper floor
					double dRelDispB(0.0), dRelVelB(0.0), dRelAccelB(0.0), dRelAssel2B(0.0);

					if (iFloor < nFloors - 1)
					{
						// Get relative data for the upper floor from the last iteration
						dDisp   = pData->GetRelDisp_CM       (iFloor + 1, iIndex - 1);
						dVel    = pData->GetRelVel_CM_SEC    (iFloor + 1, iIndex - 1);
						dAccel  = pData->GetRelAccel_CM_SEC2 (iFloor + 1, iIndex - 1);
						dAccel2 = pData->GetRelAccel2_CM_SEC3(iFloor + 1, iIndex - 1);

						// Update data using the Tailor's formula
						dRelDispB   = dDisp  + dTimeStep * dVel   + dTimeStep2 * dAccel + dTimeStep3 * dAccel2;
						dRelVelB    = dVel   + dTimeStep * dAccel + dTimeStep2 * dAccel2;
						dRelAccelB  = dAccel + dTimeStep * dAccel2;
						dRelAssel2B = dAccel2;
					}

					// Get the formula coefficients (methods check floor bounds)
					double dMassA = m_pBuilding->GetFloorMass_KG    (iFloor);
					double dSBTA  = m_pBuilding->GetDamping_KG_SEC  (iFloor, dRelDispA);
					double dSKAA  = m_pBuilding->GetRigidity_KG_SEC2(iFloor, dRelDispA);
					double dSBTB  = m_pBuilding->GetDamping_KG_SEC  (iFloor + 1, dRelDispB);
					double dSKAB  = m_pBuilding->GetRigidity_KG_SEC2(iFloor + 1, dRelDispB);

					// Update the absolute acceleration
					dAbsAccelA  = (dSBTB * dRelVelB   - dSBTA * dRelVelA   + dSKAB * dRelDispB - dSKAA * dRelDispA) / dMassA;
					dAbsAssel2A = (dSBTB * dRelAccelB - dSBTA * dRelAccelA + dSKAB * dRelVelB  - dSKAA * dRelVelA ) / dMassA;

					// Update relative data according to formula: V(floor) = U(floor) - U(floor - 1)
					dRelDispA   = dAbsDispA   - pData->GetAbsDisp_CM       (iFloor - 1, iIndex);
					dRelVelA    = dAbsVelA    - pData->GetAbsVel_CM_SEC    (iFloor - 1, iIndex);
					dRelAccelA  = dAbsAccelA  - pData->GetAbsAccel_CM_SEC2 (iFloor - 1, iIndex);
					dRelAssel2A = dAbsAssel2A - pData->GetAbsAccel2_CM_SEC3(iFloor - 1, iIndex);
				}

				// Record the data

				pData->SetRelDisp_CM       (iFloor, iIndex, dRelDispA  );
				pData->SetRelVel_CM_SEC    (iFloor, iIndex, dRelVelA   );
				pData->SetRelAccel_CM_SEC2 (iFloor, iIndex, dRelAccelA );
				pData->SetRelAccel2_CM_SEC3(iFloor, iIndex, dRelAssel2A);

				pData->SetAbsDisp_CM       (iFloor, iIndex, dAbsDispA  );
				pData->SetAbsVel_CM_SEC    (iFloor, iIndex, dAbsVelA   );
				pData->SetAbsAccel_CM_SEC2 (iFloor, iIndex, dAbsAccelA );
				pData->SetAbsAccel2_CM_SEC3(iFloor, iIndex, dAbsAssel2A);

				// Set time corresponding to the maximum relative velocity

				int iMaxRelTimeIndex = GetIndexByTime(pData->GetMaxRelDispTime_SEC(iFloor));
				double dMaxRelDisp = fabs(pData->GetRelDisp_CM(iFloor, iMaxRelTimeIndex));

				if (dMaxRelDisp < fabs(dRelDispA))
				{
					pData->SetMaxRelDispTime_SEC(iFloor, dTime);
				}
			}

			// Increment the time
			dTime += dTimeStep;
		}
	}

	m_bCalculated = true;
	return true;
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetAbsDisp_CM(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetAbsDisp_CM(aFloor, iIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetAbsVel_CM_SEC(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetAbsVel_CM_SEC(aFloor, iIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetAbsAccel_CM_SEC2(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetAbsAccel_CM_SEC2(aFloor, iIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetAbsAccel2_CM_SEC3(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetAbsAccel2_CM_SEC3(aFloor, iIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetRelDisp_CM(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetRelDisp_CM(aFloor, iIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetRelVel_CM_SEC(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetRelVel_CM_SEC(aFloor, iIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetRelAccel_CM_SEC2(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetRelAccel_CM_SEC2(aFloor, iIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetRelAccel2_CM_SEC3(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetRelAccel2_CM_SEC3(aFloor, iIndex);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetMaxRelDispTime_SEC(const CIsoSystem* aIsoSys, int aFloor) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);

	if (!pData)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	return pData->GetMaxRelDispTime_SEC(aFloor);
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetForce_KN (const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	const CCalculusIsoSysData* pData = GetCalculusByIsoSystem(aIsoSys);
	int iIndex = GetIndexByTime(aTime);

	if (!pData || iIndex == -1)
	{
		// ASSERT(FALSE);
		return 0.0;
	}

	double dMass = m_pBuilding->GetFloorMass_KG(aFloor);
	double dAccel = pData->GetAbsAccel_CM_SEC2(aFloor, iIndex);

	return 0.00001 * dMass * dAccel;
}

/////////////////////////////////////////////////////////////////////////////
double CProject::GetShear_KN(const CIsoSystem* aIsoSys, int aFloor, double aTime) const
{
	double dShear = 0.0;

	for (int iFloor = aFloor, nFloors = m_pBuilding->GetNumberOfFloors(); iFloor < nFloors; iFloor++)
	{
		dShear += GetForce_KN (aIsoSys, iFloor, aTime);
	}

	return dShear;
}

//=================================== EOF ===================================
