/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			Project.h
//		DESCRIPTION:	interface for the CProject class
//
/////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_PROJECT_H__560F40EB_B114_42A3_88DD_A41C28E75D6F__INCLUDED_)
#define AFX_PROJECT_H__560F40EB_B114_42A3_88DD_A41C28E75D6F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "IsoSystem.h"
#include "Building.h"
#include "SecSystem.h"
#include "Earthquake.h"


#define FIXED_SYSTEM_ID 0


//***************************************************************************
//                       CCalculusFloorData CLASS
//***************************************************************************

class CCalculusFloorData
{
public:

	CCalculusFloorData(int aNumIterations) :
	m_dMaxRelDispTime(0.0)
	{
		m_adAbsDisp.resize(aNumIterations);
		m_adAbsVel.resize(aNumIterations);
		m_adAbsAccel.resize(aNumIterations);
		m_adAbsAccel2.resize(aNumIterations);

		m_adRelDisp.resize(aNumIterations);
		m_adRelVel.resize(aNumIterations);
		m_adRelAccel.resize(aNumIterations);
		m_adRelAccel2.resize(aNumIterations);
	}

protected:

	vector<double> m_adAbsDisp;
	vector<double> m_adAbsVel;
	vector<double> m_adAbsAccel;
	vector<double> m_adAbsAccel2;

	vector<double> m_adRelDisp;
	vector<double> m_adRelVel;
	vector<double> m_adRelAccel;
	vector<double> m_adRelAccel2;

	double m_dMaxRelDispTime;

	friend class CCalculusIsoSysData;
};


//***************************************************************************
//                       CCalculusIsoSysData CLASS
//***************************************************************************

class CCalculusIsoSysData
{
	const CIsoSystem* m_pIsoSystem;
	vector<CCalculusFloorData> m_aFloorData;

public:

	CCalculusIsoSysData(const CIsoSystem* aIsoSystem, int aNumFloors, int aNumIterations) :
	m_pIsoSystem(aIsoSystem)
	{
		CCalculusFloorData cfd(aNumIterations);
		m_aFloorData.resize(aNumFloors, cfd);
	}

public:

	inline const CIsoSystem* GetIsoSystem() const { return m_pIsoSystem; }

	inline double GetAbsDisp_CM       (int aFloor, int aIndex) const { return m_aFloorData[aFloor].m_adAbsDisp  [aIndex]; }
	inline double GetAbsVel_CM_SEC    (int aFloor, int aIndex) const { return m_aFloorData[aFloor].m_adAbsVel   [aIndex]; }
	inline double GetAbsAccel_CM_SEC2 (int aFloor, int aIndex) const { return m_aFloorData[aFloor].m_adAbsAccel [aIndex]; }
	inline double GetAbsAccel2_CM_SEC3(int aFloor, int aIndex) const { return m_aFloorData[aFloor].m_adAbsAccel2[aIndex]; }

	inline double GetRelDisp_CM       (int aFloor, int aIndex) const { return m_aFloorData[aFloor].m_adRelDisp  [aIndex]; }
	inline double GetRelVel_CM_SEC    (int aFloor, int aIndex) const { return m_aFloorData[aFloor].m_adRelVel   [aIndex]; }
	inline double GetRelAccel_CM_SEC2 (int aFloor, int aIndex) const { return m_aFloorData[aFloor].m_adRelAccel [aIndex]; }
	inline double GetRelAccel2_CM_SEC3(int aFloor, int aIndex) const { return m_aFloorData[aFloor].m_adRelAccel2[aIndex]; }

	inline double GetMaxRelDispTime_SEC(int aFloor) const { return m_aFloorData[aFloor].m_dMaxRelDispTime; }

	// Set ------------------------------------------------------------------

	inline void SetAbsDisp_CM       (int aFloor, int aIndex, double aValue) { m_aFloorData[aFloor].m_adAbsDisp  [aIndex] = aValue; }
	inline void SetAbsVel_CM_SEC    (int aFloor, int aIndex, double aValue) { m_aFloorData[aFloor].m_adAbsVel   [aIndex] = aValue; }
	inline void SetAbsAccel_CM_SEC2 (int aFloor, int aIndex, double aValue) { m_aFloorData[aFloor].m_adAbsAccel [aIndex] = aValue; }
	inline void SetAbsAccel2_CM_SEC3(int aFloor, int aIndex, double aValue) { m_aFloorData[aFloor].m_adAbsAccel2[aIndex] = aValue; }

	inline void SetRelDisp_CM       (int aFloor, int aIndex, double aValue) { m_aFloorData[aFloor].m_adRelDisp  [aIndex] = aValue; }
	inline void SetRelVel_CM_SEC    (int aFloor, int aIndex, double aValue) { m_aFloorData[aFloor].m_adRelVel   [aIndex] = aValue; }
	inline void SetRelAccel_CM_SEC2 (int aFloor, int aIndex, double aValue) { m_aFloorData[aFloor].m_adRelAccel [aIndex] = aValue; }
	inline void SetRelAccel2_CM_SEC3(int aFloor, int aIndex, double aValue) { m_aFloorData[aFloor].m_adRelAccel2[aIndex] = aValue; }

	inline void SetMaxRelDispTime_SEC(int aFloor, double aValue) { m_aFloorData[aFloor].m_dMaxRelDispTime = aValue; }
};


//***************************************************************************
//                             CProject CLASS
//***************************************************************************

class CProject  
{
	long m_lID;
	CString m_strDescription;

	CBuilding* m_pBuilding;
	CSecSystem* m_pSecSystem;
	CEarthquakeReal* m_pEqReal;
	CEarthquakeCone* m_pEqCone;

	CIsoSystemFixed m_IsoSysFixed;
	vector<CIsoSystemAFMS*> m_apIsoSysAFMS;
	vector<CIsoSystemSliding*> m_apIsoSysSliding;
	vector<CIsoSystemShear*> m_apIsoSysShear;

public:

	CProject(long aID = ID_NEW_RECORD);
	CProject(const CProject& aProject);
	const CProject& operator = (const CProject& aProject);
	~CProject();

	bool Save();
	bool IsValid() const;

	// Getting data ---------------------------------------------------------

	inline long GetID() const { return m_lID; }
	inline const char* GetDescription() const { return m_strDescription; }

	inline const CBuilding* GetBuilding() const { return m_pBuilding; }
	inline const CSecSystem* GetSecSystem() const { return m_pSecSystem; }

	inline const CEarthquakeReal* GetEqReal() const { return m_pEqReal; }
	inline const CEarthquakeCone* GetEqCone() const { return m_pEqCone; }
	inline const CEarthquake* GetEq() const { if (m_pEqReal) return m_pEqReal; return m_pEqCone; }

	inline const CIsoSystemFixed* GetIsoSystemFixed() const { return &m_IsoSysFixed; }
	inline const vector<CIsoSystemAFMS*>& GetIsoSysAFMS() const { return m_apIsoSysAFMS; }
	inline const vector<CIsoSystemSliding*>& GetIsoSysSliding() const { return m_apIsoSysSliding; }
	inline const vector<CIsoSystemShear*>& GetIsoSysShear() const { return m_apIsoSysShear; }

	void GetIsoSystems(vector<const CIsoSystem*>& aVector) const;

	bool HasIsoSysAFMSID(long aID) const;
	bool HasIsoSysSlidingID(long aID) const;
	bool HasIsoSysShearID(long aID) const;

	// Setting data ---------------------------------------------------------

	inline bool SetDescription(const char* aValue) { m_strDescription = aValue; return true; }

	bool SetBuildingID(long aID);
	bool SetSecSystemID(long aID);
	bool SetEqRealID(long aID);
	bool SetEqConeID(long aID);

	bool SetIsoSysAFMSID(long aID);
	bool SetIsoSysSlidingID(long aID);
	bool SetIsoSysShearID(long aID);

	void ClearIsoSystems();

	// Calculus -------------------------------------------------------------

protected:

	bool m_bCalculated;
	vector<CCalculusIsoSysData*> m_apIsoSysData;

protected:

	const CCalculusIsoSysData* GetCalculusByIsoSystem(const CIsoSystem* aIsoSys) const;
	int GetIndexByTime(double aTime) const;

public:

	bool IsCalculated() const { return m_bCalculated; }
	void ClearCalculusIsoSysData();
	bool Calculate();

	double GetAbsDisp_CM       (const CIsoSystem* aIsoSys, int aFloor, double aTime) const;
	double GetAbsVel_CM_SEC    (const CIsoSystem* aIsoSys, int aFloor, double aTime) const;
	double GetAbsAccel_CM_SEC2 (const CIsoSystem* aIsoSys, int aFloor, double aTime) const;
	double GetAbsAccel2_CM_SEC3(const CIsoSystem* aIsoSys, int aFloor, double aTime) const;

	double GetRelDisp_CM       (const CIsoSystem* aIsoSys, int aFloor, double aTime) const;
	double GetRelVel_CM_SEC    (const CIsoSystem* aIsoSys, int aFloor, double aTime) const;
	double GetRelAccel_CM_SEC2 (const CIsoSystem* aIsoSys, int aFloor, double aTime) const;
	double GetRelAccel2_CM_SEC3(const CIsoSystem* aIsoSys, int aFloor, double aTime) const;

	double GetMaxRelDispTime_SEC(const CIsoSystem* aIsoSys, int aFloor) const;

	double GetForce_KN(const CIsoSystem* aIsoSys, int aFloor, double aTime) const;
	double GetShear_KN(const CIsoSystem* aIsoSys, int aFloor, double aTime) const;
};

#endif // !defined(AFX_PROJECT_H__560F40EB_B114_42A3_88DD_A41C28E75D6F__INCLUDED_)

//=================================== EOF ===================================
