/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			SecSystem.cpp
//		DESCRIPTION:	implementation of the CSecSystem class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "SecSystem.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//***************************************************************************
//                             CSecSystem CLASS
//***************************************************************************

//***************************** Setup/cleanup *******************************
/////////////////////////////////////////////////////////////////////////////
CSecSystem::CSecSystem(long aID) :
m_lID(aID),
m_strDescription(""),
m_lBuildingID(0),
m_lFloorNumber(1),
m_dMass_KG(100.00),
m_dDampingRatio(0.01)
{
	if (m_lID)
	{
		IDbTable* pTable = g_Application.GetDatabase()->GetSecSystemsTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		long lFieldID;

		BSTR bstrDesc;
		pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
		pRS->GetFieldAsString(m_lID, lFieldID, &bstrDesc);

		if (bstrDesc)
		{
			m_strDescription = bstrDesc;
			SysFreeString(bstrDesc);
		}

		pTable->GetFieldIDByName(A2BSTR("MASS_KG"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dMass_KG);

		pTable->GetFieldIDByName(A2BSTR("DAMPING_RATIO"), &lFieldID);
		pRS->GetFieldAsFloat(m_lID, lFieldID, &m_dDampingRatio);

		pTable->GetFieldIDByName(A2BSTR("ID_BUILDING"), &lFieldID);
		pRS->GetFieldAsInteger(m_lID, lFieldID, &m_lBuildingID);

		pTable->GetFieldIDByName(A2BSTR("FLOOR_NUMBER"), &lFieldID);
		pRS->GetFieldAsInteger(m_lID, lFieldID, &m_lFloorNumber);
	}
}

/////////////////////////////////////////////////////////////////////////////
CSecSystem::~CSecSystem()
{
}

//******************************* Archiving *********************************
/////////////////////////////////////////////////////////////////////////////
bool CSecSystem::Save()
{
	if (!IsValid())
	{
		return false;
	}

	IDbTable* pTable = g_Application.GetDatabase()->GetSecSystemsTable();
	IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

	if (!m_lID)
	{
		// Create record
		pRS->AddRecord(DB_PRIMARY_KEY_AUTO_GENERATE, &m_lID);
		ASSERT(m_lID);
	}

	long lFieldID;
	pTable->GetFieldIDByName(A2BSTR("DESCRIPTION"), &lFieldID);
	pRS->SetFieldToString(m_lID, lFieldID, m_strDescription.IsEmpty() ? NULL : A2BSTR(m_strDescription));

	pTable->GetFieldIDByName(A2BSTR("MASS_KG"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dMass_KG);

	pTable->GetFieldIDByName(A2BSTR("DAMPING_RATIO"), &lFieldID);
	pRS->SetFieldToFloat(m_lID, lFieldID, m_dDampingRatio);

	pTable->GetFieldIDByName(A2BSTR("ID_BUILDING"), &lFieldID);
	pRS->SetFieldToInteger(m_lID, lFieldID, m_lBuildingID);

	pTable->GetFieldIDByName(A2BSTR("FLOOR_NUMBER"), &lFieldID);
	pRS->SetFieldToInteger(m_lID, lFieldID, m_lFloorNumber);

	g_Application.GetDatabase()->Commit();

	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CSecSystem::IsValid() const
{
	if (m_lBuildingID > 0 && m_lFloorNumber > -1 && m_dMass_KG > 0.0 && m_dDampingRatio > 0.0)
	{
		IDbTable* pTable = g_Application.GetDatabase()->GetBuildingsTable();
		IDbRecordSet* pRS = g_Application.GetDatabase()->TableToRecordSet(pTable);

		BOOL bValidID;
		pRS->IsRecordID(m_lBuildingID, &bValidID); 

		if (bValidID)
		{
			CString str;
			str.Format("MASS%u_KG", (UINT)m_lFloorNumber);

			long lFieldID;
			pTable->GetFieldIDByName(A2BSTR(str), &lFieldID);

			double dFloorMass;
			pRS->GetFieldAsFloat(m_lBuildingID, lFieldID, &dFloorMass);

			return dFloorMass > 0.0;
		}
	}

	return false;
}

/////////////////////////////////////////////////////////////////////////////
CString CSecSystem::GetParamDesc() const
{
	CString str;
	str.Format(
		"Floor #%d; Mass: %.2f kg; Damping ratio: %.4f",
		m_lFloorNumber,
		m_dMass_KG,
		m_dDampingRatio);

	return str;
}

//***************************** Modifying data ******************************
/////////////////////////////////////////////////////////////////////////////
bool CSecSystem::SetDescription(const char* aValue) 
{ 
	m_strDescription = aValue; 
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CSecSystem::SetBuildingID(long aValue) 
{ 
	if (aValue < 0)
	{
		return false;
	}

	m_lBuildingID = aValue; 
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CSecSystem::SetFloorNumber(long aValue) 
{ 
	if (aValue < 0)
	{
		return false;
	}

	m_lFloorNumber = aValue; 
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CSecSystem::SetMass_KG(double aValue) 
{ 
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dMass_KG = aValue; 
	return true;
}

/////////////////////////////////////////////////////////////////////////////
bool CSecSystem::SetDampingRatio(double aValue) 
{ 
	if (aValue <= 0.0)
	{
		return false;
	}

	m_dDampingRatio = aValue; 
	return true;
}

//=================================== EOF ===================================
