/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			Table.cpp
//		DESCRIPTION:	implementation of the CTable class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "Table.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#define CELL_MARGIN_X 5
#define CELL_MARGIN_Y 3
#define FONT_HEIGHT  20

//***************************************************************************
//                               CTable CLASS
//***************************************************************************

CFont CTable::m_Font;

//**************************** Setup/cleanup ********************************
/////////////////////////////////////////////////////////////////////////////
CTable::CTable()
{
	if (!HFONT(m_Font))
	{
		m_Font.CreateFont( 
			FONT_HEIGHT, 
			0, 
			0, 
			0, 
			FW_NORMAL, 
			0, 
			0, 
			0, 
			DEFAULT_CHARSET, 
			OUT_DEFAULT_PRECIS, 
			CLIP_DEFAULT_PRECIS, 
			DEFAULT_QUALITY, 
			DEFAULT_PITCH, 
			"Courier");
	}
}

/////////////////////////////////////////////////////////////////////////////
CTable::~CTable()
{
}

//***************************************************************************
/////////////////////////////////////////////////////////////////////////////
SIZE CTable::GetSize() const
{
	CSize size(0, 0);
	int nColumns = m_aColumns.size();

	if (nColumns)
	{
		int nTitleLines = m_astrTitleLines.size();

		if (nTitleLines)
		{
			size.cy += (nTitleLines * (FONT_HEIGHT + CELL_MARGIN_Y) + CELL_MARGIN_Y);
		}

		int iCellHeight = (FONT_HEIGHT + 2 * CELL_MARGIN_Y);
		size.cy += iCellHeight;

		// Calculate vertical dimension
		size.cy += iCellHeight * m_aColumns[0].m_astrValues.size();

		// Calculate horizontal dimension
		for (int i = 0; i < nColumns; i++)
		{
			size.cx += m_aColumns[i].m_iCellWidth;
		}
	}

	return size;
}

/////////////////////////////////////////////////////////////////////////////
int CTable::GetCellHeight() const
{
	return (FONT_HEIGHT + 2 * CELL_MARGIN_Y);
}

/////////////////////////////////////////////////////////////////////////////
int CTable::GetNumberOfCells() const
{
	return m_aColumns[0].m_astrValues.size();
}

/////////////////////////////////////////////////////////////////////////////
int CTable::AddColumn(const char* aName, int aCellWidth, EJustify aJustify)
{
	COLUMN_DATA cd;
	cd.m_strName = aName;
	cd.m_iCellWidth = aCellWidth;
	cd.m_eJustify = aJustify;

	int nColumns = m_aColumns.size();

	if (nColumns)
	{
		// Reserve the same number of values as in the 1st column
		cd.m_astrValues.resize(m_aColumns[0].m_astrValues.size());
	}

	m_aColumns.push_back(cd);
	return nColumns;
}

/////////////////////////////////////////////////////////////////////////////
void CTable::Resize(int aNumberOfRows)
{
	aNumberOfRows = max(aNumberOfRows, 0);

	for (int i = 0, nColumns = m_aColumns.size(); i < nColumns; i++)
	{
		// Resize the column values
		m_aColumns[i].m_astrValues.resize(aNumberOfRows);
	}
}

/////////////////////////////////////////////////////////////////////////////
void CTable::SetSell(int aColIndex, int aRowIndex, const char* aValue)
{
	if (aColIndex >= 0 && aColIndex < m_aColumns.size() || 
		aRowIndex >= 0 && aRowIndex < m_aColumns[0].m_astrValues.size())
	{
		// Set the cell's value
		m_aColumns[aColIndex].m_astrValues[aRowIndex] = aValue;
	}
}

/////////////////////////////////////////////////////////////////////////////
void CTable::OnDraw(CDC* aDC, const POINT* aOffset)
{
	if (m_aColumns.size())
	{
		CFont* pOldFont = aDC->SelectObject(&m_Font);

		TEXTMETRIC tm;
		aDC->GetTextMetrics(&tm);

		aDC->SetBkMode(TRANSPARENT);

		SIZE size = GetSize();
		POINT pnt = *aOffset;

		// Draw title -------------------------------------------------------

		int nTitleLines = m_astrTitleLines.size();

		if (nTitleLines)
		{
			aDC->MoveTo(pnt.x, pnt.y);
			aDC->LineTo(pnt.x + size.cx, pnt.y);
			
			int iStepY = FONT_HEIGHT + CELL_MARGIN_Y;

			for (int iTitleLine = 0; iTitleLine < nTitleLines; iTitleLine++)
			{
				aDC->TextOut(pnt.x + CELL_MARGIN_X, pnt.y + CELL_MARGIN_Y, m_astrTitleLines[iTitleLine]);

				pnt.y += iStepY;
			}

			pnt.y += CELL_MARGIN_Y;
		}

		// Draw column labels -----------------------------------------------

		int iCellHeight = (FONT_HEIGHT + 2 * CELL_MARGIN_Y);

		CBrush brush(RGB(230, 215, 180));
		CRect rect(pnt.x, pnt.y, pnt.x + size.cx, pnt.y + iCellHeight);
		aDC->FillRect(rect, &brush);

		aDC->MoveTo(pnt.x, pnt.y);
		aDC->LineTo(pnt.x + size.cx, pnt.y);

		for (int iCol = 0, nColumns = m_aColumns.size(); iCol < nColumns; iCol++)
		{
			int x = pnt.x + (m_aColumns[iCol].m_iCellWidth - m_aColumns[iCol].m_strName.GetLength() * tm.tmAveCharWidth) / 2;
			aDC->TextOut(x, pnt.y + CELL_MARGIN_Y, m_aColumns[iCol].m_strName);
			pnt.x += m_aColumns[iCol].m_iCellWidth;
		}

		pnt.x = aOffset->x;
		pnt.y += iCellHeight;

		aDC->MoveTo(pnt.x, pnt.y);
		aDC->LineTo(pnt.x + size.cx, pnt.y);

		// Draw cells -------------------------------------------------------

		for (int iRow = 0, nRows = m_aColumns[0].m_astrValues.size(); iRow < nRows; iRow++)
		{
			for (iCol = 0; iCol < nColumns; iCol++)
			{
				CString str(m_aColumns[iCol].m_astrValues[iRow]);
				double x = pnt.x + CELL_MARGIN_X;

				switch (m_aColumns[iCol].m_eJustify)
				{
				case JUSTIFY_CENTER_DECIMAL:
					{
						int nChars = str.GetLength();
						int iDecimalPos = str.Find('.', 0);

						if (iDecimalPos == -1)
						{
							iDecimalPos++;
						}

						x = pnt.x + m_aColumns[iCol].m_iCellWidth / 2 - tm.tmAveCharWidth * iDecimalPos;
					}
					break;

				case JUSTIFY_RIGHT:
					{
						x = pnt.x + (m_aColumns[iCol].m_iCellWidth - tm.tmAveCharWidth * str.GetLength()) / 2;
					}
					break;

				default: // left alignment
					break;
				}

				aDC->TextOut(x, pnt.y + CELL_MARGIN_Y, str);
				pnt.x += m_aColumns[iCol].m_iCellWidth;
			}

			pnt.x = aOffset->x;
			pnt.y += iCellHeight;

			aDC->MoveTo(pnt.x, pnt.y);
			aDC->LineTo(pnt.x + size.cx, pnt.y);
		}

		// Draw column dividers ---------------------------------------------

		pnt = *aOffset;

		int iTitleHeight = nTitleLines * (FONT_HEIGHT + CELL_MARGIN_Y) + CELL_MARGIN_Y;

		aDC->MoveTo(pnt.x, pnt.y);
		aDC->LineTo(pnt.x, pnt.y + size.cy);

		for (iCol = 0; iCol < nColumns - 1; iCol++)
		{
			pnt.x += m_aColumns[iCol].m_iCellWidth;

			aDC->MoveTo(pnt.x, pnt.y + iTitleHeight);
			aDC->LineTo(pnt.x, pnt.y + size.cy);
		}

		pnt.x += m_aColumns[iCol].m_iCellWidth;
		aDC->MoveTo(pnt.x, pnt.y);
		aDC->LineTo(pnt.x, pnt.y + size.cy);

		aDC->SelectObject(pOldFont);
	}
}

//=================================== EOF ===================================
