/////////////////////////////////////////////////////////////////////////////
//
//	Copyright  VALENTIN SHUSTOV 2006. All Rights Reserved.
//
//		FILE:			ViewEqConeTimeHistoryDlg.cpp
//		DESCRIPTION:	implementation of the CViewEqConeTimeHistoryDlg class
//
/////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "epet.h"
#include "ViewEqConeTimeHistoryDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//***************************************************************************
//                      CViewEqConeTimeHistoryDlg CLASS
//***************************************************************************

BEGIN_MESSAGE_MAP(CViewEqConeTimeHistoryDlg, CDialog)
	//{{AFX_MSG_MAP(CViewEqConeTimeHistoryDlg)
	ON_BN_CLICKED(IDC_BUTTON_IMPORT, OnButtonImport)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
CViewEqConeTimeHistoryDlg::CViewEqConeTimeHistoryDlg(CEarthquakeCone& aEqCone, CWnd* pParent) : 
CDialog(CViewEqConeTimeHistoryDlg::IDD, pParent),
m_EqCone(aEqCone)
{
	//{{AFX_DATA_INIT(CViewEqConeTimeHistoryDlg)
	//}}AFX_DATA_INIT
}

/////////////////////////////////////////////////////////////////////////////
BOOL CViewEqConeTimeHistoryDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Create column headers ------------------------------------------------

	m_listCtrl.InsertColumn(0,  "Time, sec",        LVCFMT_LEFT, 120);
	m_listCtrl.InsertColumn(1,  "Displacement, cm", LVCFMT_LEFT, 120);

	m_EqCone.Calculate();

	double dStep = m_EqCone.GetStep_SEC();
	double dTotalTime = m_EqCone.GetTotalTime_SEC();

	vector<double> times;
	vector<double> disp;
	int i = 0;

	for (double dTime = 0.0; dTime <= dTotalTime; dTime += dStep)
	{
		double dDisp = m_EqCone.GetAbsDisp_CM(dTime);

		times.push_back(dTime);
		disp.push_back(dDisp);

		CString str;
		str.Format("%.2f", dTime);
		int nItem = m_listCtrl.InsertItem(i++, str);

		str.Format("%+.2f", dDisp);
		m_listCtrl.SetItemText(nItem, 1, str);
	}

	m_Graph.AddGraph(times, disp);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
void CViewEqConeTimeHistoryDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CViewEqConeTimeHistoryDlg)
	DDX_Control(pDX, IDC_LIST_HISTORY, m_listCtrl);
	//}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
void CViewEqConeTimeHistoryDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	CWnd* pWnd = GetDlgItem(IDC_VIEW_HISTORY);
	ASSERT(pWnd);

	CRect rect;
	pWnd->GetWindowRect(rect);
	ScreenToClient(rect);

	m_Graph.OnDraw(&dc, rect);
}

/////////////////////////////////////////////////////////////////////////////
void CViewEqConeTimeHistoryDlg::OnButtonImport() 
{
	static char BASED_CODE szFilter[] = "Text Files (*.txt)|*.txt||";

	char szDir[MAX_PATH];
	GetCurrentDirectory(sizeof(szDir), szDir);
	strcat(szDir, "\\cone.txt");

	CFileDialog dlg(FALSE, NULL, szDir, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter, this);

	if (dlg.DoModal() == IDOK)
	{
		CFile file(dlg.GetPathName(), CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive);

		double dStep = m_EqCone.GetStep_SEC();
		double dTotalTime = m_EqCone.GetTotalTime_SEC();

		for (double dTime = 0.0; dTime <= dTotalTime; dTime += dStep)
		{
			CString str;
			str.Format("%.2f, %.2f\r\n", dTime, m_EqCone.GetAbsDisp_CM(dTime));
			file.Write(str, str.GetLength());
		}
	}
}

//=================================== EOF ===================================
